/**@name LIBRARY: cpcConfigGenericFunctions.ctl

@author: Ivan Koblik (IT-CO-FE)
@author: Nikolay Kulman (IT-CO-FE)
@author: Geraldine Thomas (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 14.11.2003
@brief This library contains functions called from other GCS scripts.
Modifications History:
  $Log: cpcConfigGenericFunctions.ctl,v $
    26/07/2012: Alexey
        [UCPC-920] store alert dpes in mail/sms categories instead of dp

    16/05/2012: Alexey
        [UCPC-787] AlSt always acknowledgeable

    27/03/2012: Alexey
        [UCPC-733] removed hook for custom PCO interlocks

    23/03/2012: Alexey
        [UCPC-723] modify cpcConfigGenericFunctions_setAlertDetails in order to have interlocks always acknowledgable

    20/01/2012: Alexey
        check archive filter only when it's found

    12/12/2011: Alexey
        [UCPC-621] modified alarm setting procedure

    21/10/2011: Alexey
        fixed consistence checking of cpcConfigGenericFunctions_check5RangesAlert

    03/10/2011: Alexey
        [UCPC-562] updated 5alert processing - removed normal position from AI/AO

    16/09/2011: Alexey
        [UCPC-525] removed hasMaskEvent properties
        refactored cpcConfigGenericFunctions_setMaskEvent - made it generic

    14/09/2011: Alexey
        [UCPC-524] strip metainfo data after split

    02/09/2011: Alexey
        boolean dpe always with old/new archive/smoothing

    08/08/2011: Alexey
        allow to create not-alarm bool dpe

    02/08/2011: Alexey
        [UCPC-437] H/L thresholds never ack

    28/07/2011: Alexey
        [UCPC-423] fixed masking event on the importation stage

    28/07/2011: Alexey
        [UCPC-436] store sms message

    14/07/2011: Alexey
        introduced stubAlarm parameter

    08/07/2011: Alexey
        [UCPC-384] removed "parameters" field from the meta-info block

    06/07/2011: Alexey
        [UCPC-336] in cpcConfigGenericFunctions_setSmoothing use straight deadband value (removed range reference)

    01/07/2011: Alexey
        [UCPC-364] check params before import

    2011/05/30 Alexey
        added cpcConfigGenericFunctions_getStringConstantValueByName

    2011/05/30 Alexey
        [UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

    2011/05/10 amerezhi
        [IS-527] process metainfo with API
        [UCPC-268] added second ALIAS

    2011/05/09 amerezhi
        [UCPC-235] added metainfo (types relationships configuraiton)
        used CPC length constants - UN_CPC_CONFIG_ALLCOMMON_LENGTH and UN_CPC_CONFIG_COMMON_LENGTH
        ported unConfigGenericFunctions_check5RangesAlert to cpcConfigGenericFunctions_check5RangesAlert to use CPC length constants

    21/04/2011: amerezhi
        [UCPC-262] fixed configuring alarm for S7

    21/04/2011: amerezhi
        added alarm acknowledge and 5 range alarm support

    13/04/2011: amerezhi
        updated SMS_CATEGORIES processing according new API [IS-518]

    11/04/2011: amerezhi
        - provided config functionalities for SMS_CATEGORIES

    07/04/2011: amerezhi
        - set bool archive for alarm dpe

    05/04/2011: amerezhi
        - stub SMS_CATEGORIES processing

    28/03/2011: amerezhi
        - removed checking hasArchive for evStsReg dpe

  Revision 1.20  2010/09/17 16:33:00  amerezhi
  The cpcConfigGenericFunctions_checkDescription is fixed

  Revision 1.19  2008/09/04 14:48:54  gthomas
  -moved back to version 1.17

  Revision 1.18  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.17  2007/11/27 10:47:11  ikoblik
  Tested + bug fixes according to the November, 21st code review.

  Revision 1.16  2007/11/26 16:09:49  ikoblik
  Small bug fixes to getPlcDpName

  Revision 1.15  2007/11/22 10:04:33  gamx
  cpcConfigGenericFunctions_getPlcDpName is added

  Revision 1.14  2007/11/15 08:38:52  ikoblik
  cpcConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.13  2007/11/14 15:33:07  ikoblik
  cpcConfigGenericFunctions code review comments implementation

  Revision 1.12  2007/10/30 16:19:51  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

Internal Functions :
    cpcConfigGenericFunctions_checkDescription(dyn_string dsConfigs, int iStringToCheck, dyn_string &exceptionInfo)
    cpcConfigGenericFunctions_setDigitalAlertWithMessage(dyn_string dsConfigs, string sDpName, string sDescription, bool bOkRange,
      bool bActive, bool bSMS, string sClass, dyn_string &exceptionInfo)
    cpcConfigGenericFunctions_processArchives(int iToSkip, bool hasArchive, dyn_string& dsConfigs)
    bool cpcConfigGenericFunctions_checkConfigLength(int iToSkip, dyn_string& dsConfigs, bool &bHasArchive)
    void cpcConfigGenericFunctions_checkArchiveConfig(int iToSkip, dyn_string& dsConfigs, dyn_string& exceptionInfo)
    void cpcConfigGenericFunctions_checkBool(string name, string bSource, bool& bRes, dyn_string &exceptionInfo)
*/

//----------------------------------------------------------------------------------------------------------------------------------
// cpcConfigGenericFunctions_checkDeviceConfig
void cpcConfigGenericFunctions_checkDeviceConfig(string deviceType, dyn_string dsConfigs, dyn_string &exceptionInfo) {
    if(dynlen(dsConfigs) < UN_CONFIG_COMMON_LENGTH) { // totally broken config
        fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    }

    bool bHasArchive;

    // check length
    if(!cpcConfigGenericFunctions_checkConfigLength(deviceType, dsConfigs, bHasArchive)) {
        fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " " + getCatStr("unGeneration","BADINPUTS"),"");
        return;
    }

    // check archives
    if (bHasArchive) {
        cpcConfigGenericFunctions_checkArchiveConfig(deviceType, dsConfigs, exceptionInfo);
        if (dynlen(exceptionInfo) > 0) {
            fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " " + getCatStr("unGeneration","BADINPUTS"),"");
            return;
        }
    }

    string plcType = cpcConfigGenericFunctions_getFrontEndType();

    //// check device specific params
    mapping deviceAddresses = cpcConfigGenericFunctions_fetchDeviceAddresses(deviceType);
    dyn_string deviceAdrNames = mappingKeys(deviceAddresses);

    // check addresses
    for (int i = 1; i <= dynlen(deviceAdrNames); i++) {
        string currentAddrName = deviceAdrNames[i];
        mapping currentAddrProps = deviceAddresses[currentAddrName];
        if (plcType != SOFT_FE_DPTYPE) {
          if (currentAddrProps["dataType"] == CPC_BOOL) {
              if (currentAddrProps["bitPosition"] < 0 || currentAddrProps["bitPosition"] > 15) {
                  fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + currentAddrName + " has wrong bit position", "");
              }
          } else {
              cpcConfigGenericFunctions_checkDeviceAddress(dsConfigs, plcType, deviceType, currentAddrName, currentAddrProps, bHasArchive, exceptionInfo);
          }
          if (dynlen(exceptionInfo) > 0) { return; }
        }
        if (currentAddrProps["isAlarm"]) { // check alarm
            int normalPosition = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "NORMAL_POSITION");
            if (normalPosition > 0) {
                unConfigGenericFunctions_checkNormalPosition(dsConfigs[UN_CONFIG_COMMON_LENGTH + normalPosition], exceptionInfo);
            }
            if (dynlen(exceptionInfo) > 0) { return; }
            if (currentAddrProps["hasAcknowledge"]) {
                int acknowledgeAlarmPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "ACK_ALARM");
                if (acknowledgeAlarmPos > 0) {
                    bool val;
                    cpcConfigGenericFunctions_checkBool("ACK_ALARM", dsConfigs[UN_CONFIG_COMMON_LENGTH + acknowledgeAlarmPos], val, exceptionInfo);
                } else {
                    fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no acknowledge type for " + currentAddrName,"");
                }
            }
        }

        if (currentAddrProps["hasSmooth"]) {
            int defaultDeadbandPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive,  "DEADBAND");
            int defaultDeadbandTypePos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive, "DEADBAND_TYPE");
            if (defaultDeadbandPos > 0 && defaultDeadbandTypePos > 0) {
                unConfigGenericFunctions_checkDeadBand(dsConfigs[UN_CONFIG_COMMON_LENGTH + defaultDeadbandPos], dsConfigs[UN_CONFIG_COMMON_LENGTH + defaultDeadbandTypePos], exceptionInfo);
            } else {
                fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no deadband for " + currentAddrName,"");
            }
        }
        if (currentAddrProps["hasPvRange"]) {
            int defaultRangeMaxPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive, "RANGEMAX");
            int defaultRangeMinPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive, "RANGEMIN");
            if (defaultRangeMaxPos > 0 && defaultRangeMinPos > 0) {
                unConfigGenericFunctions_checkRange(dsConfigs[UN_CONFIG_COMMON_LENGTH + defaultRangeMinPos], dsConfigs[UN_CONFIG_COMMON_LENGTH + defaultRangeMaxPos], exceptionInfo);
            } else {
                fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no range for " + currentAddrName,"");
            }
        }
        if (currentAddrProps["hasUnit"]) {
            int defaultUnitPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive, "UNIT");
            if (defaultUnitPos > 0) {
                // @TODO: not checked ever
            } else {
                fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no unit for " + currentAddrName,"");
            }
        }
        if (currentAddrProps["hasFormat"]) {
            int defaultFormatPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive, "FORMAT");
            if (defaultFormatPos > 0) {
                cpcConfigGenericFunctions_checkFormat(dsConfigs[UN_CONFIG_COMMON_LENGTH + defaultFormatPos], exceptionInfo);
            } else {
                fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no format for " + currentAddrName,"");
            }
        }
        if (currentAddrProps["hasDescription"]) {
            int descPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive, "DESCRIPTION");
            if (descPos > 0) {
                cpcConfigGenericFunctions_checkDescription(dsConfigs, dsConfigs[UN_CONFIG_COMMON_LENGTH + descPos], exceptionInfo);
            } else {
                fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no description for " + currentAddrName,"");
            }
        }
        if (currentAddrProps["hasArchive"]) {
            if (strpos(currentAddrName, "evStsReg") == -1 && currentAddrProps["dataType"] != CPC_BOOL) {
                int archivePos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive, "ARCHIVE_ACTIVE");
                int timeFilterPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, currentAddrName, bHasArchive, "ARCHIVE_TIME_FILTER");
                if (archivePos > 0) {
                    string timeFilter = "";
                    if (timeFilterPos > 0) {
                        timeFilter = dsConfigs[UN_CONFIG_COMMON_LENGTH + timeFilterPos];
                        // @TODO: current checks are not full and covers only special case of time filter
                        unConfigGenericFunctions_checkArchiveParameters(timeFilter, dsConfigs[UN_CONFIG_COMMON_LENGTH + archivePos], exceptionInfo);
                    }
                } else {
                    fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no archive for " + currentAddrName,"");
                }
            }
        }
        if (currentAddrProps["5RangesAlerts"]) {
            int acknowledgeAlarmPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "ACK_ALARM");
            if (acknowledgeAlarmPos > 0) {
                bool acknowledgeAlarm;
                cpcConfigGenericFunctions_checkBool("ACK_ALARM", dsConfigs[UN_CONFIG_COMMON_LENGTH + acknowledgeAlarmPos], acknowledgeAlarm, exceptionInfo);
                if (dynlen(exceptionInfo) > 0) { return; }
            } else {
                fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no acknowledge type for " + currentAddrName,"");
            }
            int llLimitPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "LLLIMIT");
            int lLimitPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "LLIMIT");
            int hLimitPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "HLIMIT");
            int hhLimitPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "HHLIMIT");
            int alarmActivePos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "ALARM_ACTIVE");
            if (alarmActivePos > 0 && llLimitPos > 0 && lLimitPos > 0 && hLimitPos > 0 && hhLimitPos > 0) {
                cpcConfigGenericFunctions_check5RangesAlert(dsConfigs, alarmActivePos,
                                                           makeDynInt(llLimitPos, lLimitPos, hLimitPos, hhLimitPos),
                                                           exceptionInfo);
            } else {
                fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no alarm active or not all limits for " + currentAddrName,"");
            }
        }

        if (dynlen(exceptionInfo) > 0) { return; }
    }

    int parametersPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", bHasArchive, "PARAMETERS");
    if (parametersPos > 0) {
        cpcConfigGenericFunctions_checkParameters(deviceType, dsConfigs[UN_CONFIG_COMMON_LENGTH + parametersPos], exceptionInfo);
        if (dynlen(exceptionInfo) > 0) { return; }
    }

    int smsCategoriesPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", bHasArchive, "SMS_CATEGORIES");
    if (smsCategoriesPos > 0) {
        dyn_string categories = strsplit(dsConfigs[UN_CONFIG_COMMON_LENGTH + smsCategoriesPos], ",");
        unProcessAlarm_checkCategory(categories, exceptionInfo);
    }
    if (dynlen(exceptionInfo) > 0) { return; }

    int maskEventPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", bHasArchive, "MASKEVENT");
    if (maskEventPos > 0) {
        bool maskEvent;
        cpcConfigGenericFunctions_checkBool("MASKEVENT", dsConfigs[UN_CONFIG_COMMON_LENGTH + maskEventPos], maskEvent, exceptionInfo);
    }
    if (dynlen(exceptionInfo) > 0) { return; }

    string customFunction = "CPC_";
    if (plcType == SOFT_FE_DPTYPE) {
        customFunction += "SOFT_FE_";
    }
    customFunction += cpcConfigGenericFunctions_getShortTypeName(deviceType) + "Config_checkCustomConfig";
    if (isFunctionDefined(customFunction)) {
        evalScript(exceptionInfo, "dyn_string main(dyn_string configLine, bool bHasArchive) {" + 
                         "dyn_string exceptionInfo;" +
                         customFunction  + "(configLine, bHasArchive, exceptionInfo);" +
                         "return exceptionInfo; }", makeDynString(), dsConfigs, bHasArchive);
        if (dynlen(exceptionInfo) > 0) { return; }
    }
}

void cpcConfigGenericFunctions_checkDeviceAddress(dyn_string dsConfigs, string plcType, string deviceType, string addressName, mapping addrProps, bool bHasArchive, dyn_string &exceptionInfo) {
    int addrPosition = cpcConfigGenericFunctions_getDPAddressPosition(deviceType, addressName);

    if (addrPosition < 1) {
        fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkImportConfig: " + deviceType + " has no position constant for " + addressName, "");
        return;
    }

    if(plcType == UN_PLC_DPTYPE) {
        unConfigGenericFunctions_checkAddresses(makeDynString(dsConfigs[UN_CONFIG_COMMON_LENGTH + addrPosition]), exceptionInfo);
    } else {
        unConfigGenericFunctions_checkAddressS7(makeDynString(dsConfigs[UN_CONFIG_COMMON_LENGTH + addrPosition]),
                                                cpcConfigGenericFunctions_translateDataTypeForS7Check(addrProps["dataType"]),
                                                exceptionInfo);	
    }
}

//----------------------------------------------------------------------------------------------------------------------------------
// cpcConfigGenericFunctions_setDeviceConfig
void cpcConfigGenericFunctions_setDeviceConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
    string deviceType = dsConfigs[UN_CONFIG_OBJECTGENERAL_OBJECT];
    string sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

    int deviceLength = UN_CONFIG_ALLCOMMON_LENGTH + cpcConfigGenericFunctions_getDPConfigLength(deviceType) + cpcConfigGenericFunctions_getDPConfigAdditionalLength(deviceType);
    bool bHasArchive = dynlen(dsConfigs) == deviceLength + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;

    string plcType = cpcConfigGenericFunctions_getFrontEndType();

    cpcConfigGenericFunctions_processArchives(UN_CONFIG_ALLCOMMON_LENGTH + cpcConfigGenericFunctions_getDPConfigLength(deviceType), bHasArchive, dsConfigs);

    mapping deviceAddresses = cpcConfigGenericFunctions_fetchDeviceAddresses(deviceType);
    dyn_string deviceAdrNames = mappingKeys(deviceAddresses);

    for (int i = 1; i <= dynlen(deviceAdrNames); i++) {
        string currentAddrName = deviceAdrNames[i];
        mapping currentAddrProps = deviceAddresses[currentAddrName];
        if (plcType != SOFT_FE_DPTYPE) {
            if (currentAddrProps["dataType"] == CPC_BOOL) {
                cpcConfigGenericFunctions_setAlert(dsConfigs, deviceType, currentAddrName, currentAddrProps, plcType, currentAddrProps["dpe"], exceptionInfo);
            } else {
                int addrPosition = cpcConfigGenericFunctions_getDPAddressPosition(deviceType, currentAddrName);
                cpcConfigGenericFunctions_setAddress(dsConfigs, currentAddrName, currentAddrProps, addrPosition, plcType, exceptionInfo);
            }
        }
        if (currentAddrProps["isAlarm"] && !currentAddrProps["stubAlarm"]) {
            cpcConfigGenericFunctions_setAlertDetails(dsConfigs, deviceType, currentAddrName, currentAddrProps, plcType, currentAddrProps["dpe"], exceptionInfo);
        }
        if (currentAddrProps["hasUnit"]) {
            cpcConfigGenericFunctions_setUnit(dsConfigs, deviceType, currentAddrName, currentAddrProps, plcType, bHasArchive, exceptionInfo);
        }
        if (currentAddrProps["hasFormat"]) {
            cpcConfigGenericFunctions_setFormat(dsConfigs, deviceType, currentAddrName, currentAddrProps, plcType, bHasArchive, exceptionInfo);
        }
        if (currentAddrProps["hasSmooth"]) {
            cpcConfigGenericFunctions_setSmoothing(dsConfigs, deviceType, currentAddrName, currentAddrProps, plcType, bHasArchive, exceptionInfo);
        }
        if (currentAddrProps["hasPvRange"]) {
            cpcConfigGenericFunctions_setRange(dsConfigs, deviceType, currentAddrName, currentAddrProps, plcType, bHasArchive, exceptionInfo);
        }
        if (currentAddrProps["hasDescription"]) {
            cpcConfigGenericFunctions_setDescription(dsConfigs, deviceType, currentAddrName, currentAddrProps, plcType, bHasArchive, exceptionInfo);
        }
        if (currentAddrProps["hasArchive"]) {
            if (strpos(currentAddrName, "evStsReg") == 0) {
                cpcConfigGenericFunctions_setEvArchive(dsConfigs, currentAddrProps["address"], exceptionInfo);
            } else {
                cpcConfigGenericFunctions_setArchive(dsConfigs, deviceType, currentAddrName, currentAddrProps, plcType, bHasArchive, exceptionInfo);
            }
        }
        if (currentAddrProps["5RangesAlerts"]) {
            bool acknowledgeAlarm, alarmActive, bSMS;
            int acknowledgeAlarmPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "ACK_ALARM");
            cpcConfigGenericFunctions_checkBool("ACK_ALARM", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + acknowledgeAlarmPos], acknowledgeAlarm, exceptionInfo);
            int alarmActivePos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "ALARM_ACTIVE");
            cpcConfigGenericFunctions_checkBool("ALARM_ACTIVE", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + alarmActivePos], alarmActive, exceptionInfo);
            int smsCategoriesPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "SMS_CATEGORIES");
            bool bSMS = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + smsCategoriesPos] != "";
            int normalPosition = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "NORMAL_POSITION");
            int llLimitPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "LLLIMIT");
            int lLimitPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "LLIMIT");
            int hLimitPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "HLIMIT");
            int hhLimitPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "HHLIMIT");
            int enabledLimits = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "ENABLED_LIMITS");

            cpcConfigGenericFunctions_set5RangesAlert(dsConfigs, dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + currentAddrProps["address"],
                                                    "Position Status", acknowledgeAlarm, alarmActive, bSMS,
                                                    makeDynInt(llLimitPos, lLimitPos, hLimitPos, hhLimitPos), enabledLimits,
                                                    exceptionInfo);
        }

        if (dynlen(exceptionInfo) > 0) { return; }
    }

    int parametersPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", bHasArchive, "PARAMETERS");
    if (parametersPos > 0) {
        cpcConfigGenericFunctions_setParameters(deviceType, dsConfigs, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + parametersPos], exceptionInfo);
        if (dynlen(exceptionInfo) > 0) { return; }
    }

    cpcConfigGenericFunctions_setMetainfo(deviceType, dsConfigs, bHasArchive, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) { return; }

    int smsCategoriesPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", bHasArchive, "SMS_CATEGORIES");
    if (smsCategoriesPos > 0) {
        dyn_string categories = strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + smsCategoriesPos], ",");
        for (int i = 1; i <= dynlen(deviceAdrNames); i++) {
            string currentAddrName = deviceAdrNames[i];
            mapping currentAddrProps = deviceAddresses[currentAddrName];
            if ((currentAddrProps["isAlarm"] && !currentAddrProps["stubAlarm"]) || currentAddrProps["5RangesAlerts"]) {
                if (smsCategoriesPos > 0) {
                    unProcessAlarm_removeDpFromList(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + currentAddrProps["address"], exceptionInfo);
                    if (dynlen(categories) > 0) {
                        unProcessAlarm_createCategory(categories, "", makeDynString(), exceptionInfo);
                        for (int i = 1; i <= dynlen(categories); i++) {
                            unProcessAlarm_addDpToList(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + currentAddrProps["address"], categories[i], exceptionInfo);
                        }
                    }
                }
            }
        }
    }

    int smsMessagesPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", bHasArchive, "SMS_MESSAGE");
    if (smsMessagesPos > 0) {
        unGenericDpFunctions_setKeyDeviceConfiguration(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_CONFIG_SMS_MESSAGE, makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + smsMessagesPos]), exceptionInfo);
    }
    
    int maskEventPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", bHasArchive, "MASKEVENT");
    if (maskEventPos > 0) {
        bool maskEvent;
        cpcConfigGenericFunctions_checkBool("MASKEVENT", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + maskEventPos], maskEvent, exceptionInfo);
        cpcConfigGenericFunctions_setMaskEvent(deviceType, dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], maskEvent, exceptionInfo);
    }

    string customFunction = "CPC_";
    if (plcType == SOFT_FE_DPTYPE) {
        customFunction += "SOFT_FE_";
    }
    customFunction += cpcConfigGenericFunctions_getShortTypeName(deviceType) + "Config_setCustomConfig";
    if (isFunctionDefined(customFunction)) {
        evalScript(exceptionInfo, "dyn_string main(dyn_string configLine, bool bHasArchive) {" + 
                         "dyn_string exceptionInfo;" +
                         customFunction  + "(configLine, bHasArchive, exceptionInfo);" +
                         "return exceptionInfo; }", makeDynString(), dsConfigs, bHasArchive);
    }
}

void cpcConfigGenericFunctions_checkParameters(string deviceType, string parameters, dyn_string &exceptionInfo) {
    dyn_string possibleKeys;
    dyn_string props = strsplit(parameters, ",");
    dyn_string devicesKeys = cpcConfigGenericFunctions_getDeviceParamNames(deviceType);
    for (int i = 1; i <= dynlen(props); i++) {
        dyn_string pair = strsplit(props[i], "=");
        if (dynlen(pair) != 2) {
            fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkParameters:" + getCatStr("unGeneration","BADDATA"), "");
            return;
        } else if (!dynContains(devicesKeys, pair[1])) {
            fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkParameters: wrong property " + pair[1] + ".", "");
            return;
        } else {
            dynAppend(possibleKeys, pair[1]);
        }
    }
    if (dynlen(possibleKeys) != dynlen(devicesKeys)) {
        fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkParameters: wrong amount of parameters", "");
        return;
    }
    dynUnique(possibleKeys);
    if (dynlen(possibleKeys) != dynlen(devicesKeys)) {
        fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkParameters: wrong amount of parameters", "");
        return;
    }
}

void cpcConfigGenericFunctions_setParameters(string deviceType, dyn_string dsConfigs, string parameters, dyn_string &exceptionInfo) {
    dyn_string props = strsplit(parameters, ",");
    for (int i = 1; i <= dynlen(props); i++) {
        dyn_string prop = strsplit(props[i], "=");
        cpcGenericDpFunctions_setDeviceProperty(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], prop[1], prop[2], exceptionInfo);
    }
}

// cpcConfigGenericFunctions_setMetainfo - set meta information for device (i.e. master/parents/children/type)
void cpcConfigGenericFunctions_setMetainfo(string deviceType, dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
    int masterNamePos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", hasArchive, "MASTER_NAME");
    int parentsPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", hasArchive, "PARENTS");
    int childrenPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", hasArchive, "CHILDREN");
    int typePos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", hasArchive, "TYPE");
    int secondAliasPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, "", hasArchive, "SECOND_ALIAS");

    if (masterNamePos > 0) {
        unGenericDpFunctions_setKeyDeviceConfiguration(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_CONFIG_MASTER_NAME_KEY, makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + masterNamePos]), exceptionInfo);
        unGenericDpFunctions_setKeyDeviceConfiguration(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_CONFIG_PARENTS_KEY, splitMetainfo(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + parentsPos]), exceptionInfo);
        unGenericDpFunctions_setKeyDeviceConfiguration(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_CONFIG_CHILDREN_KEY, splitMetainfo(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + childrenPos]), exceptionInfo);
        unGenericDpFunctions_setKeyDeviceConfiguration(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_CONFIG_TYPE_KEY, splitMetainfo(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + typePos]), exceptionInfo);
        unGenericDpFunctions_setKeyDeviceConfiguration(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_CONFIG_SECOND_ALIAS_KEY, splitMetainfo(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + secondAliasPos]), exceptionInfo);
    }
}

dyn_string splitMetainfo(string meta) {
    dyn_string result = strsplit(meta, ",");
    for (int i = 1; i <= dynlen(result); i++) {
        result[i] = strrtrim(strltrim(result[i]));
    }
    return result;
}

//----------------------------------------------------------------------------------------------------------------------------------
// Alert configuration functions
//----------------------------------------------------------------------------------------------------------------------------------
void cpcConfigGenericFunctions_setAlert(dyn_string dsConfigs, string deviceType, string alertName, mapping alertProps, string plcType, string regName, dyn_string &exceptionInfo) {
    string addressReference;

    if (plcType == UN_PLC_DPTYPE) {
        unConfigGenericFunctions_getUnicosAddressReference(makeDynString(DPATTR_ADDR_MODE_INPUT_SPONT, // alerts for inputs only
                                                                        dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE],
                                                                        dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_NUMBER], 
                                                                        false,  // @TODO: is it correct?
                                                                        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + cpcConfigGenericFunctions_getDPAddressPosition(deviceType, regName)],
                                                                        dsConfigs[UN_CONFIG_OBJECTGENERAL_EVENT16]), addressReference);
    } else {
        unConfigGenericFunctions_getUnicosAddressReferenceS7(makeDynString(DPATTR_ADDR_MODE_INPUT_SPONT, 
                                                                           dsConfigs[UN_CONFIG_OBJECTGENERAL_PLCNAME], 
                                                                           dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + cpcConfigGenericFunctions_getDPAddressPosition(deviceType, regName)]), 
                                                                           addressReference);
    }

    if (addressReference == "") {
        fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_setAlert: can't create addressReference for " + alertName,"");
        return;
    }

    if (plcType == UN_PLC_DPTYPE) {
        unConfigGenericFunctions_setStsRegAlarmBitAddress(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + alertProps["address"],
                                                          dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
                                                          addressReference,
                                                          alertProps["bitPosition"],
                                                          exceptionInfo);
    } else {
        unConfigGenericFunctions_setStsRegAlarmBitAddressS7(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + alertProps["address"],
                                                            dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
                                                            addressReference,
                                                            alertProps["bitPosition"],
                                                            exceptionInfo);
    }
}

void cpcConfigGenericFunctions_setAlertDetails(dyn_string dsConfigs, string deviceType, string alertName, mapping alertProps, string plcType, string regName, dyn_string &exceptionInfo) {
    string bitDescription, sDomain, sNature, sDeviceAlias, sDeviceLinks;
    dyn_string dsbitDescription, dsAlertTexts, dsAlertClasses;

    unGenericDpFunctions_getBitDescription(dsConfigs[UN_CONFIG_OBJECTGENERAL_OBJECT], regName, alertProps["bitPosition"], bitDescription);
    if (bitDescription == "") {	// Bit description was not found
        bitDescription = alertName;
    }

    // @TODO: 2Enrique : is it useful?
    string sDomainConfig = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DOMAIN];
    dyn_string ds = strsplit(sDomainConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
    if(dynlen(ds) == UN_CONFIG_UNICOS_DOMAIN_AND_ACCESS_CONTROL_LEN) { // access control + domain present
      sDomain = ds[2];
    } else if (strpos(sDomainConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR) >= 0) { // if UN_ACCESS_CONTROL_DOMAIN_SEPARATOR in sDomain config --> domain = ""
      sDomain = "";
    } else {
      sDomain = sDomainConfig;
    }

    // @TODO: 2Enrique : is it useful?
    string sNatureConfig = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_NATURE];
    ds = strsplit(sNatureConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
    // if no ACC priv assume list of nature
    if(dynlen(ds) >= (UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN -1)) { 
      if(dynlen(ds) == UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN)
        sNature = ds[4];
      else // dynlen = UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN -1 --> consider nature = ""
        sNature = "";
    } else {
      sNature = sNatureConfig;
    }

    // alias is deviceName only
    unConfigGenericFunctions_getAliasLink(dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS], sDeviceAlias, sDeviceLinks);
    
    dsbitDescription[UN_CONFIG_ALARM_DOMAIN] = sDomain;
    dsbitDescription[UN_CONFIG_ALARM_NATURE] = sNature;
    dsbitDescription[UN_CONFIG_ALARM_ALIAS] = sDeviceAlias;
    dsbitDescription[UN_CONFIG_ALARM_DESCRIPTION] = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DESCRIPTION];
    dsbitDescription[UN_CONFIG_ALARM_NAME] = bitDescription;
    dsbitDescription[UN_CONFIG_ALARM_DEFAULTPANEL] = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DEFAULTPANEL];

    bool bOkRange, alarmExists, alarmActive, bSMS;
    int iLevel;
    // set alarm
    int normalPosition = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "NORMAL_POSITION");
    unConfigGenericFunctions_getNormalPosition(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + normalPosition], alarmExists, alarmActive, bOkRange, iLevel, bSMS);

    int descriptionPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "ALARM_DESCRIPTION");
    string alertMessageOK, alertMessageBAD;
    if (descriptionPos < 1) {
        string descriptionTextConstant = "UN_CONFIG_" + strtoupper(deviceType) + "_" + strtoupper(alertName) + "_ALERT_TEXT";
        string descriptionText = cpcConfigGenericFunctions_getStringConstantValueByName(descriptionTextConstant);
        if (descriptionText != CPC_UNDEFINED_CONSTANT_VALUE) {
            alertMessageOK  = "";
            alertMessageBAD = descriptionText;
        } else {
            alertMessageOK  = UN_CONFIG_DEFAULT_ALERT_DIGITAL_TEXT_OK;
            alertMessageBAD = UN_CONFIG_DEFAULT_ALERT_DIGITAL_TEXT_BAD;
        }
    } else {
        alertMessageOK  = "";
        alertMessageBAD = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + descriptionPos];
    }
    dsAlertTexts = bOkRange ? makeDynString(alertMessageBAD, alertMessageOK) : makeDynString(alertMessageOK, alertMessageBAD);

    string alertClassPostfixConstant = "UN_CONFIG_" + strtoupper(deviceType) + "_" + strtoupper(alertName) + "_ALERT_CLASS_POSTFIX";
    string alertClassPostfix = cpcConfigGenericFunctions_getStringConstantValueByName(alertClassPostfixConstant);
    if (alertClassPostfix == CPC_UNDEFINED_CONSTANT_VALUE) {
        alertClassPostfix = "";
    }
    bool acknowledgeAlarm = false;
    if (alertName == "TStopISt" || alertName == "FuStopISt" || alertName == "StartISt" || alertName == "AlSt") {
        acknowledgeAlarm = true;
    }
    if (alertProps["hasAcknowledge"]) {
        int acknowledgeAlarmPos = cpcConfigGenericFunctions_getDeviceConfigConstant(deviceType, "ACK_ALARM");
        cpcConfigGenericFunctions_checkBool("ACK_ALARM", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + acknowledgeAlarmPos], acknowledgeAlarm, exceptionInfo);
    }
    string alertClass = cpcConfigGenericFunctions_getAlertClass(acknowledgeAlarm, bSMS, alertClassPostfix);

    dsAlertClasses = bOkRange ? makeDynString(alertClass, "") : makeDynString("", alertClass);

    if (alarmExists) {
        cpcConfigGenericFunctions_setDigitalAlert(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + alertProps["address"], dsbitDescription, dsAlertTexts, dsAlertClasses, bOkRange, alarmActive,  bSMS, exceptionInfo);
    } else {
        fwAlertConfig_deleteDigital(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + alertProps["address"], exceptionInfo);
        unConfigGenericFunctions_setAlarmDescription(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + alertProps["address"], dsbitDescription, exceptionInfo);	// Set description for alert screen
    }
}

// unConfigGenericFunctions_setDigitalAlert
/**
Purpose: set digital alert of given datapoint

Parameters:
        sDpName: string, input, datapoint name
        dsdescription: dyn_string, input, datapoint description
        bOkRange: bool, input, false if ok range = false, true if Ok range = true
        active: bool, input, true if alert must be set to active
        exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
    . PVSS version: 3.0 
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cpcConfigGenericFunctions_setDigitalAlert(string sDpName, dyn_string dsdescription, dyn_string dsAlertTexts, dyn_string dsAlertClasses, bool bOkRange, bool active, bool bSMS, dyn_string &exceptionInfo) {
    int i, length, alertType;
    dyn_float dfLimitsGood;
    int isAlarmActive = -1;
    
    unConfigGenericFunctions_setAlarmDescription(sDpName, dsdescription, exceptionInfo);	// Set description for alert screen
    dpGet(sDpName + ":_alert_hdl.._type", alertType);
    switch (alertType) {
        case DPCONFIG_NONE:
            fwAlertConfig_createDigital(sDpName, 
                                       dsAlertTexts,
                                       dsAlertClasses,
                                       UN_CONFIG_DEFAULT_ALERT_PANEL,
                                       UN_CONFIG_DEFAULT_ALERT_PANEL_PARAMETERS,
                                       UN_CONFIG_DEFAULT_ALERT_HELP,
                                       exceptionInfo);
            if (active) {
                dpSetWait(sDpName + ":_alert_hdl.._active", active);
            }
            break;
        
        case DPCONFIG_ALERT_BINARYSIGNAL:
            fwAlertConfig_modifyDigital(sDpName, 
                                       dsAlertTexts,
                                       dsAlertClasses,
                                       UN_CONFIG_DEFAULT_ALERT_PANEL,
                                       UN_CONFIG_DEFAULT_ALERT_PANEL_PARAMETERS,
                                       UN_CONFIG_DEFAULT_ALERT_HELP,
                                       exceptionInfo);
            dpGet(sDpName + ":_alert_hdl.._active", isAlarmActive);
            if ((isAlarmActive != active) || (isAlarmActive == -1)) {
                dpSetWait(sDpName + ":_alert_hdl.._active", active);
            }
            break;
            
        case DPCONFIG_ALERT_NONBINARYSIGNAL:
        default:		
            fwException_raise(exceptionInfo,"ERROR:","cpcConfigGenericFunctions_setDigitalAlert :" + getCatStr("unGeneration","BADALERT"));
            break;
    }
}

//----------------------------------------------------------------------------------------------------------------------------------
// Address configuration functions
//----------------------------------------------------------------------------------------------------------------------------------
void cpcConfigGenericFunctions_setAddress(dyn_string dsConfigs, string regName, mapping regProperties, int regPosition, string plcType, dyn_string &exceptionInfo) {
    string addressReference;

    int dataType = cpcConfigGenericFunctions_translateDataType(plcType, regProperties["dataType"]);

    string addrMode = regProperties["isInput"] ? DPATTR_ADDR_MODE_INPUT_SPONT : DPATTR_ADDR_MODE_OUTPUT;

    if (plcType == UN_PLC_DPTYPE) {
        unConfigGenericFunctions_getUnicosAddressReference(makeDynString(addrMode, dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE],
                                                                        dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_NUMBER], 
                                                                        strpos(regName, "evStsReg") == 0,  // @TODO: is event. how to check more correct?
                                                                        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + regPosition],
                                                                        dsConfigs[UN_CONFIG_OBJECTGENERAL_EVENT16]), addressReference);
    } else {
        unConfigGenericFunctions_getUnicosAddressReferenceS7(makeDynString(addrMode, 
                                                                            dsConfigs[UN_CONFIG_OBJECTGENERAL_PLCNAME], 
                                                                            dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + regPosition]), 
                                                                            addressReference);
    }
    if (addressReference == "") {
        fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_setAddress: can't create addressReference","");
        return;
    }

    string addressType = plcType == UN_PLC_DPTYPE ? fwPeriphAddress_TYPE_MODBUS : fwPeriphAddress_TYPE_S7;
    fwPeriphAddress_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"],
                        makeDynString(addressType,
                                    dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER],
                                    addressReference,
                                    addrMode,
                                    dataType,
                                    UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
                                    "", "", "", "",
                                    regProperties["isInput"] ? UN_CONFIG_LOWLEVEL_ANALOG_PROCESS_INPUT : UN_CONFIG_LOWLEVEL_ANALOG_PROCESS_OUTPUT,
                                    0,
                                    UN_CONFIG_DEFAULT_ADDRESS_STARTING_TIME, UN_CONFIG_DEFAULT_ADDRESS_INTERVAL_TIME, ""),
                        exceptionInfo);
}

void cpcConfigGenericFunctions_setSmoothing(dyn_string dsConfigs, string deviceType, string regName, mapping regProperties, string plcType, bool bHasArchive, dyn_string &exceptionInfo) {
    float deadband;
    int deadbandType;

    int deadbandPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "DEADBAND");
    int deadbandTypePos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "DEADBAND_TYPE");

    if (deadbandPos > 0 && deadbandTypePos > 0) {
        deadband = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + deadbandPos];
        deadbandType = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + deadbandTypePos];

        switch(deadbandType) {
            case UN_CONFIG_DEADBAND_VALUE:
                deadbandType = deadband > 0 ? DPATTR_VALUE_SMOOTH : -1;
                break;
            case UN_CONFIG_DEADBAND_RELATIF:
                deadbandType = deadband > 0 ? DPATTR_VALUE_REL_SMOOTH : -1;
                break;
            case UN_CONFIG_DEADBAND_OLD_NEW:
                deadbandType = DPATTR_COMPARE_OLD_NEW;
                deadband = - 1;
                break;
            default:
                deadbandType = -1;
        }

        if (deadbandType == DPATTR_VALUE_SMOOTH || deadbandType == DPATTR_VALUE_REL_SMOOTH || deadbandType == DPATTR_COMPARE_OLD_NEW) {
            fwSmoothing_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"], deadbandType, deadband, UN_CONFIG_DEFAULT_SMOOTHING_TIME_FILTER, exceptionInfo);
        } else {
            fwSmoothing_delete(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"], exceptionInfo);
        }
    }
}

void cpcConfigGenericFunctions_getRange(dyn_string dsConfigs, string deviceType, string regName, string &rangeMax, string &rangeMin, bool bHasArchive) {
    int rangeMaxPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "RANGEMAX");
    int rangeMinPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "RANGEMIN");
    if (rangeMaxPos > 0 && rangeMinPos > 0) {
        rangeMin = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + rangeMinPos];
        rangeMax = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + rangeMaxPos];
    } else {
        rangeMin = "";
        rangeMax = "";
    }
}

void cpcConfigGenericFunctions_setRange(dyn_string dsConfigs, string deviceType, string regName, mapping regProperties, string plcType, bool bHasArchive, dyn_string &exceptionInfo) {
    float fMin, fMax;
    string sRangeMin, sRangeMax;
    cpcConfigGenericFunctions_getRange(dsConfigs, deviceType, regName, sRangeMax, sRangeMin, bHasArchive);
    if (sRangeMin != "" && sRangeMax != "") {
        //Format values
        fMin = sRangeMin;
        fMax = sRangeMax;

        if(fMin != fMax) { // if min!= max --> create pv_range config
            unConfigGenericFunctions_ScientificFormat(sRangeMin);
            unConfigGenericFunctions_ScientificFormat(sRangeMax);
            
            fwPvRange_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"],
                                      sRangeMin,
                                      sRangeMax,
                                      UN_CONFIG_DEFAULT_RANGE_NEGATE_RANGE,
                                      UN_CONFIG_DEFAULT_RANGE_IGNORE_OUTSIDE,
                                      UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MIN,
                                      UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MAX,
                                      exceptionInfo);
        } else { // if min == max --> delete pv_range
            fwPvRange_delete(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"], exceptionInfo);
        }
    } else {
        fwPvRange_delete(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"], exceptionInfo);
    }
}

void cpcConfigGenericFunctions_setUnit(dyn_string dsConfigs, string deviceType, string regName, mapping regProperties, string plcType, bool bHasArchive, dyn_string &exceptionInfo) {
    string unit;

    int unitPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "UNIT");
    unit = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + unitPos];

    unConfigGenericFunctions_setUnit(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"], unit, exceptionInfo);
}

// cpcConfigGenericFunctions_checkFormat
/**
Purpose: check format

Parameters:
        sFormat, string, input, format
        exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
    . PVSS version: 3.0 
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cpcConfigGenericFunctions_checkFormat(string sFormat, dyn_string &exceptionInfo){	
    string sTempFormat=strtoupper(sFormat);
    int pos, iFormat;
    
// if EXP->OK
    if(sTempFormat != UN_FACEPLATE_EXP) {
// check if fixed display
        pos = strpos(sTempFormat, UN_DISPLAY_VALUE_DIGIT_TYPE);
        if(pos > 0) { // fixed display
            sTempFormat = substr(sTempFormat, 0, pos);
            pos = sTempFormat;
            if(pos <=0) { // error
                fwException_raise(exceptionInfo,"ERROR","cpcConfigGenericFunctions_checkFormat: " + sFormat + getCatStr("unGeneration","DEBUGBADFORMAT"),"");
            }
        }
        else { // check ##.# format of xEXP
            pos = strpos(sTempFormat, UN_FACEPLATE_EXP);
            if(pos > 0) { // xEXP format
                strreplace(sTempFormat, UN_FACEPLATE_EXP, "");
                unConfigGenericFunctions_checkInt(sTempFormat, iFormat, exceptionInfo);
            }
            else { // ##.# format
                strreplace(sTempFormat, "#", "");
                if((sTempFormat != "") && (sTempFormat != ".")) { // error
                    fwException_raise(exceptionInfo,"ERROR","cpcConfigGenericFunctions_checkFormat: " + sFormat + getCatStr("unGeneration","DEBUGBADFORMAT"),"");
                }
            }
        }
    }
}

void cpcConfigGenericFunctions_setFormat(dyn_string dsConfigs, string deviceType, string regName, mapping regProperties, string plcType, bool bHasArchive, dyn_string &exceptionInfo) {
    string format;

    int formatPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "FORMAT");
    format = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + formatPos];

    unConfigGenericFunctions_setFormat(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"], format, exceptionInfo);
}

void cpcConfigGenericFunctions_setMaskEvent(string deviceType, string deviceName, bool isMasked, dyn_string &exceptionInfo)
{
    dyn_string dpes;
    mapping deviceAddresses = cpcConfigGenericFunctions_fetchDeviceAddresses(deviceType);
    dyn_string deviceAdrNames = mappingKeys(deviceAddresses);
    for (int i = 1; i <= dynlen(deviceAdrNames); i++) {
        string currentAddrName = deviceAdrNames[i];
        if (deviceAddresses[currentAddrName]["isInput"]) {
            if (deviceAddresses[currentAddrName]["dataType"] == CPC_BOOL) {
                dynAppend(dpes, deviceAddresses[currentAddrName]["address"]);
            } else if (strpos(currentAddrName, "evStsReg") == 0) {
                dynAppend(dpes, deviceAddresses[currentAddrName]["address"]);
            }
        }
    }
    unConfigGenericFunctions_setMaskEvent(deviceName, isMasked, dpes, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// cpcConfigGenericFunctions_checkDescription
/**
Reads commar seperated values string from datapoint configs and verifies that each of the
values is a pair of an identifier on the left side, assignment sign, and an integer on the right side.
For example: "value1=23,value2=10"

  @param[in]  dsConfigs       Full config line
  @param[in]  iStringToCheck  Commar seperated values string's index in the datapoint configs array.
  @param[out] exceptionInfo   All possible errors will be returned here
*/
cpcConfigGenericFunctions_checkDescription(dyn_string dsConfigs, string sLabels, dyn_string &exceptionInfo) {
    // if the string is empty, no problem.
    if (sLabels == CPC_CONFIG_EMPTY_DESCRIPTION_STRING) {
        return;
    }

    // if the string is not empty, check that each element is: anInt=aString
    dyn_string dsLabels = strsplit(sLabels, ",");

    // Check that each of the sub string is "n=string"
    for (int i = 1; i <= dynlen(dsLabels); i++) {
        int iCorrectValue;
        dyn_string dsValue = strsplit(dsLabels[i], "=");
        if (dynlen(dsValue) != 2) {
            fwException_raise(exceptionInfo, "ERROR", "cpcConfigGenericFunctions_checkDescription:" + getCatStr("unGeneration","BADDATA"), "");
            return;
        } else {
            //dsValue first element should be the integer
            unConfigGenericFunctions_checkInt(strrtrim(strltrim(dsValue[1])), iCorrectValue, exceptionInfo);
        }
    }
}

void cpcConfigGenericFunctions_setDescription(dyn_string dsConfigs, string deviceType, string regName, mapping regProperties, string plcType, bool bHasArchive, dyn_string &exceptionInfo) {
    int descPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "DESCRIPTION");

    string desc = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + descPos];
    if (desc == "") desc = CPC_CONFIG_EMPTY_DESCRIPTION_STRING;

    unConfigGenericFunctions_setDescription(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"], desc, exceptionInfo);
}

void cpcConfigGenericFunctions_setEvArchive(dyn_string dsConfigs, string address, dyn_string &exceptionInfo) {
    fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + address,
                  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT],
                  DPATTR_ARCH_PROC_SIMPLESM,
                  DPATTR_COMPARE_OLD_NEW,
                  0,
                  0,
                  exceptionInfo);
}

void cpcConfigGenericFunctions_setArchive(dyn_string dsConfigs, string deviceType, string regName, mapping regProperties, string plcType, bool bHasArchive, dyn_string &exceptionInfo) {
    bool bArchiveActive;
    float archiveTimeFilter = 0;
    int iArchiveType, iSmoothProcedure;
    float fReturnDeadBand;

    int archivePos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "ARCHIVE_ACTIVE");
    int timeFilterPos = cpcConfigGenericFunctions_getConfigPositionForProperty(deviceType, regName, bHasArchive, "ARCHIVE_TIME_FILTER");
    if (archivePos > 0) {
        if (timeFilterPos > 0) {
            archiveTimeFilter = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + timeFilterPos];
        }
        unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + archivePos], bArchiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
        unConfigGenericFunctions_getArchivingDeadband(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + archivePos], fReturnDeadBand);
        if (regProperties["dataType"] == CPC_BOOL) {
            iArchiveType = DPATTR_ARCH_PROC_SIMPLESM;
            iSmoothProcedure = DPATTR_COMPARE_OLD_NEW;
            fReturnDeadBand = -1;
            archiveTimeFilter = 0;
        }
    } else {
        bArchiveActive = false;
    }

    string archive = regProperties["dataType"] == CPC_BOOL ? dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] : dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG];

    if (bArchiveActive) {
        fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"],
                      archive,
                      iArchiveType,
                      iSmoothProcedure,
                      fReturnDeadBand,
                      archiveTimeFilter,
                      exceptionInfo);
    } else {
        fwArchive_delete(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + regProperties["address"], exceptionInfo);
    }
}

//------------------------------------------------------------------------------------------------------------------------

// cpcConfigGenericFunctions_check5RangesAlert
/**
Purpose : check 5 ranges _alert_hdl data

Parameters:
        dsConfigs, dyn_string, input, full config line
        iNormalPosition, int, input, index of normal position parameter
        diLimits, dyn_int, input, index of limits in dsConfig
                4 parameters : index of 1.LL 2.L 3.H 4.HH
        exceptionInfo, dyn_string, output, for errors

WARNING : each given index is "relative", i.e. convertIndex must be added to have absolute index

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
    . PVSS version: 3.0 
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/

cpcConfigGenericFunctions_check5RangesAlert(dyn_string dsConfigs, int alarmActivePos, dyn_int diLimits, dyn_string &exceptionInfo)
{
    float fLL, fL, fH, fHH;
    int iLevel;
    bool alarmActive, bOkRange, bError,bSMS;

    cpcConfigGenericFunctions_checkBool("ALARM_ACTIVE", dsConfigs[UN_CONFIG_COMMON_LENGTH + alarmActivePos], alarmActive, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) {
        return;
    }
    if (dynlen(diLimits) != 4) {
        fwException_raise(exceptionInfo,"ERROR","cpcConfigGenericFunctions_check5RangesAlert: wrong amount of alert limits: " + dynlen(diLimits) + " instead of 4.","");
        return;
    }
    float prev;
    bool prevInit = false;
    for (int i = 1; i <= 4; i++) {
        if (dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[i]] != "") {
            float current;
            dyn_string exceptionInfoTemp;
            string limit = dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[i]];
            unConfigGenericFunctions_checkFloat(limit, current, exceptionInfoTemp);
            if (dynlen(exceptionInfoTemp) > 0) {
                fwException_raise(exceptionInfo, "ERROR","cpcConfigGenericFunctions_check5RangesAlert: wrong limit #" + (5-i) + ": " + limit,"");
                return;
            }
            if (prevInit && prev >= current) {
                fwException_raise(exceptionInfo, "ERROR","cpcConfigGenericFunctions_check5RangesAlert: limits are not consistent ([limit#"+i+"] " + prev + " >= " + current + " [limit#"+(i-1)+"])","");
                return;
            }
            prev = current;
            prevInit = true;
        }
    }
}

// cpcConfigGenericFunctions_set5RangesAlert
/**
Purpose : set 5 ranges _alert_hdl on given dpe

Parameters:
        dsConfigs, dyn_string, input, full config line
        sDp, string, input, dpe
        description, string, input, description
        iNormalPosition, int, input, index of alarm active, exist parameter
        diLimits, dyn_int, input, index of limits in dsConfig
                4 parameters : index of 1.LL 2.L 3.H 4.HH
        exceptionInfo, dyn_string, output, for errors

WARNING : each given index is "relative", i.e. convertIndex must be added to have absolute index

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
    . PVSS version: 3.0 
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cpcConfigGenericFunctions_set5RangesAlert(dyn_string dsConfigs, string sDp, string description, bool acknowledgeAlarm, bool alarmActive, bool bSMS, dyn_int diLimits, int enabledLimitsPos, dyn_string &exceptionInfo) {
    bool alarmExists, bOkRange;
    dyn_string dsDescription;
    dyn_string dsAlarms, dsAlertClass;
    int iAlertType, iLevel;
    string sDeviceAlias, sDeviceLinks;
    string sDomain, sDomainConfig, sNature, sNatureConfig;
    dyn_string ds;
    
    dynClear(dsDescription);
    sDomainConfig = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DOMAIN];
    ds = strsplit(sDomainConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
    if(dynlen(ds) == UN_CONFIG_UNICOS_DOMAIN_AND_ACCESS_CONTROL_LEN) { // access control + domain present
        sDomain = ds[2];
    } else if(strpos(sDomainConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR) >= 0) { // if UN_ACCESS_CONTROL_DOMAIN_SEPARATOR in sDomain config --> domain = ""
        sDomain = "";
    } else {
        sDomain = sDomainConfig;
    }
     
    sNatureConfig = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_NATURE];
    ds = strsplit(sNatureConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
    // if no ACC priv assume list of nature
    if(dynlen(ds) >= (UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN -1)) { 
        if(dynlen(ds) == UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN) {
            sNature = ds[4];
        } else { // dynlen = UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN -1 --> consider nature = ""
            sNature = "";
        }
    } else {
        sNature = sNatureConfig;
    }
        
    dsDescription [UN_CONFIG_ALARM_DOMAIN] = sDomain;
    dsDescription [UN_CONFIG_ALARM_NATURE] = sNature;
// alias is the deviceName only
    unConfigGenericFunctions_getAliasLink(dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS], sDeviceAlias, sDeviceLinks);
    dsDescription [UN_CONFIG_ALARM_ALIAS] = sDeviceAlias;
    dsDescription [UN_CONFIG_ALARM_DESCRIPTION] = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DESCRIPTION];
    dsDescription [UN_CONFIG_ALARM_NAME] = description;
    dsDescription [UN_CONFIG_ALARM_DEFAULTPANEL] = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DEFAULTPANEL];

    alarmExists = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[1]] != "" || dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[2]] != "" ||
                  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[3]] != "" || dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[4]] != "";

    if (alarmExists) {
        dyn_float dfLevelLimits = makeDynFloat();

        dpGet(sDp+":_alert_hdl.._type", iAlertType);
        if(iAlertType != DPCONFIG_NONE) {
            fwAlertConfig_deleteAnalog(sDp, exceptionInfo);
        }

        cpcConfigGenericFunctions_set4LevelsAlert(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[1]],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[2]],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[3]],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[4]],
                                                    bSMS, acknowledgeAlarm, dfLevelLimits, dsAlarms, dsAlertClass);

        unConfigGenericFunctions_setAnalogAlert(sDp, dsDescription, dfLevelLimits, dsAlarms, dsAlertClass, alarmActive, exceptionInfo);
    } else {
        fwAlertConfig_deleteAnalog(sDp, exceptionInfo);
        // even if the alarm is deleted set the description
        unConfigGenericFunctions_setAlarmDescription(sDp, dsDescription, exceptionInfo);
    }
}

// cpcConfigGenericFunctions_set4LevelsAlert
/**
Purpose : set levels for AI/AO alarms according to the Normal Position

Parameters:
        iMode, int, input, valid levels ?
        dsLevelsAlarm, dyn_string, output, Levels to set
        dsLevelsAlertsClass, dyn_string, output, Alert Class to set
    
Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
    . PVSS version: 3.0 
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cpcConfigGenericFunctions_set4LevelsAlert(string LLparam, string Lparam, string Hparam, string HHparam,
                                            bool bSMS, bool acknowledgeAlarm, dyn_float &dfLevelLimits, dyn_string &dsLevelsAlarm, dyn_string &dsLevelsAlertsClass) {
    string sLLClass, sLClass, sHClass, sHHClass;

    sLLClass = "_cpcAnalogLL";
    sLClass  = "_cpcAnalogL";
    sHClass  = "_cpcAnalogH";
    sHHClass = "_cpcAnalogHH";
    if (acknowledgeAlarm) {
        sLLClass += "_Ack";
        sHHClass += "_Ack";
    }
    if (bSMS) {
        sLLClass += "_Mail";
        sLClass  += "_Mail";
        sHClass  += "_Mail";
        sHHClass += "_Mail";
    }
    sLLClass += ".";
    sLClass  += ".";
    sHClass  += ".";
    sHHClass += ".";

    dynClear(dsLevelsAlarm);
    dynClear(dsLevelsAlertsClass);

    if (LLparam != "") {
        dynAppend(dsLevelsAlarm, "LL");
        dynAppend(dsLevelsAlertsClass, sLLClass);
        dynAppend(dfLevelLimits, LLparam);
    }

    if (Lparam != "") {
        dynAppend(dsLevelsAlarm, "L");
        dynAppend(dsLevelsAlertsClass, sLClass);
        dynAppend(dfLevelLimits, Lparam);
    }

    dynAppend (dsLevelsAlarm, "Ok");
    dynAppend (dsLevelsAlertsClass, "");

    if (Hparam != "") {
        dynAppend(dsLevelsAlarm, "H");
        dynAppend(dsLevelsAlertsClass, sHClass);
        dynAppend(dfLevelLimits, Hparam);
    }

    if (HHparam != "") {
        dynAppend(dsLevelsAlarm, "HH");
        dynAppend(dsLevelsAlertsClass, sHHClass);
        dynAppend(dfLevelLimits, HHparam);
    }
}

string cpcConfigGenericFunctions_getAlertClass(bool acknowledgeAlarm, bool bSMS, string postfix = "")
{
    string alertClass = CPC_CONFIG_ALARM_BASE_CLASS + postfix;
    if (acknowledgeAlarm) {
        alertClass += CPC_CONFIG_ALARM_ACK_POSTFIX;
    }
    if (bSMS) {
        alertClass += CPC_CONFIG_ALARM_MAIL_POSTFIX;
    }
    return alertClass;
}

//----------------------------------------------------------------------------------------------------------------------------------
// unConfigXXX processing functions
//----------------------------------------------------------------------------------------------------------------------------------
dyn_string cpcConfigGenericFunctions_getDeviceParamNames(string deviceType) {
    string function = "CPC_" + cpcConfigGenericFunctions_getShortTypeName(deviceType) + "Config_getParamNames";
    dyn_string paramNames;
    evalScript(paramNames, "dyn_string main() {" + 
                    "dyn_string paramNames = " + function  + "();" +
                    "return paramNames; }", makeDynString());
    return paramNames;
}

mapping cpcConfigGenericFunctions_fetchDeviceAddresses(string device) {
    mapping addresses;
    evalScript(addresses, "mapping main() { return CPC_" + cpcConfigGenericFunctions_getShortTypeName(device) + "Config_getConfig(); }", makeDynString());

    dyn_string keys = mappingKeys(addresses);
    dyn_string allProps = makeDynString("hasUnit", "hasFormat", "hasSmooth", "hasArchive", "hasPvRange", "hasDescription", "isAlarm", "stubAlarm", "hasAcknowledge", "5RangesAlerts");
    string plcType = cpcConfigGenericFunctions_getFrontEndType();

    for (int i = 1; i <= dynlen(keys); i++) {
        string dpe = keys[i];
        for (int j = 1; j <= dynlen(allProps); j++) {
            if (!mappingHasKey(addresses[dpe], allProps[j])) {
                addresses[dpe][allProps[j]] = false;
            }
        }
        if (!mappingHasKey(addresses[dpe], "dataType")) {
            addresses[dpe]["dataType"] = CPC_BOOL;
        }
        addresses[dpe]["isInput"] = strpos(addresses[dpe]["address"], ".ProcessInput") == 0;
        if (plcType == SOFT_FE_DPTYPE)  {
          if (addresses[dpe]["hasSmooth"]) {
            DebugN("disable smooth options of " + device + "-" + dpe);
          }
          addresses[dpe]["hasSmooth"] = false;
        }
    }

    return addresses;
/*
    dyn_string propNames, propDPEs, hasArchive;

    dpGet(device + "Info" + "." + "properties.names", propNames);
    dpGet(device + "Info" + "." + "properties.dpes", propDPEs);
    dpGet(device + "Info" + "." + "configuration.archive.canHave", hasArchive);

    for (int i = 1; i <= dynlen(propNames); i++) {
        mapping prop;
        prop["name"] = propDPEs[i];
        prop["hasArchive"] = hasArchive[i];
        addresses[propNames[i]] = prop;
    }

    return addresses;
*/
}

int cpcConfigGenericFunctions_getDPConfigLength(string deviceName) {
    return (int) cpcConfigGenericFunctions_getDeviceConfigConstant(deviceName, "LENGTH");
}

int cpcConfigGenericFunctions_getDPConfigAdditionalLength(string deviceName) {
    int length = (int) cpcConfigGenericFunctions_getDeviceConfigConstant(deviceName, "ADDITIONAL_LENGTH");
    if (length < 0) {
        length = 0;
    }
    return length;
}

int cpcConfigGenericFunctions_getDPAddressPosition(string device, string address) {
    string addressPositionName = cpcConfigGenericFunctions_getConstantPrefix(cpcConfigGenericFunctions_getFrontEndType());
    addressPositionName += strtoupper(device) + "_ADDRESS_" + strtoupper(address);
    return (int) cpcConfigGenericFunctions_getConstantValueByName(addressPositionName);
}

// cpcConfigGenericFunctions_getConfigPositionForProperty
/**
Purpose: for given device type, periphery address name and property name return config position.
Function make a lookup for constant UN_CONFIG_[DEVICE]_[ADDRESS_NAME]_[PROP]. if not found then try constant UN_CONFIG_[DEVICE]_[PROP]
    
*/
int cpcConfigGenericFunctions_getConfigPositionForProperty(string deviceType, string addrName, bool hasArchive, string propName) {
    int position = 0;
    string prefix = cpcConfigGenericFunctions_getConstantPrefix(cpcConfigGenericFunctions_getFrontEndType());
    prefix += strtoupper(deviceType);
    if (strlen(addrName) > 0) {
        position = cpcConfigGenericFunctions_getConstantValueByName(prefix + "_ADDITIONAL_" + strtoupper(addrName) + "_" + strtoupper(propName));
    }
    if (position < 1) {
        position = cpcConfigGenericFunctions_getConstantValueByName(prefix + "_ADDITIONAL_" + strtoupper(propName));
    }
    if (position > 0) { //calc full position
        position += cpcConfigGenericFunctions_getDPConfigLength(deviceType);
        if (hasArchive) {
            position += UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;
        }
        return position;
    }
    if (position < 1 && strlen(addrName) > 0) {
        position = cpcConfigGenericFunctions_getConstantValueByName(prefix + "_" + strtoupper(addrName) + "_" + strtoupper(propName));
    }
    if (position < 1) {
        position = cpcConfigGenericFunctions_getConstantValueByName(prefix + "_" + strtoupper(propName));
    }
    return position;
}

int cpcConfigGenericFunctions_getDeviceConfigConstant(string deviceType, string constantName) {
    int result;
    string name = cpcConfigGenericFunctions_getConstantPrefix(cpcConfigGenericFunctions_getFrontEndType());
    name += strtoupper(deviceType) + "_" + constantName;
    evalScript(result, "int main() { int result = -1; try { result = " + name + "; } catch { result = -1;} return result; }", makeDynString());
    return result;
}
string cpcConfigGenericFunctions_getShortTypeName(string deviceType) {
    string dt = deviceType;
    strreplace(dt, "CPC_", "");
    return dt;
}
anytype cpcConfigGenericFunctions_getConstantValueByName(string name) {
    anytype result;
    evalScript(result, "anytype main() { anytype result = -1; try { result = " + name + "; } catch { result = -1;} return result; }", makeDynString());
    return result;
}

string cpcConfigGenericFunctions_getConstantPrefix(string plcType) {
    string prefix = "UN_CONFIG_";
    if (cpcConfigGenericFunctions_getFrontEndType() == SOFT_FE_DPTYPE) {
        prefix += "SOFT_FE_";
    }
    return prefix;
}

string cpcConfigGenericFunctions_getStringConstantValueByName(string name) {
    string result;
    evalScript(result, "anytype main() { string result = \"\"; try { result = " + name + "; } catch { result = CPC_UNDEFINED_CONSTANT_VALUE;} return result; }", makeDynString());
    return result;
}

//----------------------------------------------------------------------------------------------------------------------------------
// Basic functions
//----------------------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------------------
// cpcConfigGenericFunctions_checkConfigLength
/**
Check if the configs lenght is correct. Also reports if config has archiving in it.

  @return   Result of checking, true if OK, else false.
  @param[in]  deviceType 	DPT name
  @param[out] dsConfigs		Configs
  @param[out] bHasArchive returns true if passed configs have settings for archiving
*/
bool cpcConfigGenericFunctions_checkConfigLength(string deviceType, dyn_string dsConfigs, bool &bHasArchive) {
  int deviceLength = UN_CONFIG_COMMON_LENGTH + cpcConfigGenericFunctions_getDPConfigLength(deviceType) + cpcConfigGenericFunctions_getDPConfigAdditionalLength(deviceType);
  int configLength = dynlen(dsConfigs);
  
  if (configLength == deviceLength) {
    bHasArchive = false;
    return true;
  } else if (configLength == (deviceLength + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
    bHasArchive = true;
    return true;
  }
  return false;
}

//----------------------------------------------------------------------------------------------------------------------------------
// cpcConfigGenericFunctions_checkArchiveConfig
/**
Check if the configs lenght is correct for archives. Check if archive configs are correct

  @return   Result of checking, true if OK, else false.
  @param[in]  deviceType 	DPT name
  @param[out] dsConfigs 	Configs
  @param[out] exceptionInfo All possible errors will be returned here
*/
void cpcConfigGenericFunctions_checkArchiveConfig(string deviceType, dyn_string& dsConfigs, dyn_string& exceptionInfo) {
    int commonLength = UN_CONFIG_COMMON_LENGTH + cpcConfigGenericFunctions_getDPConfigLength(deviceType);
    if (dynlen(dsConfigs) == commonLength + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + cpcConfigGenericFunctions_getDPConfigAdditionalLength(deviceType)) {
        unConfigGenericFunctions_checkArchive(dsConfigs[commonLength + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
                                                dsConfigs[commonLength + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
                                                dsConfigs[commonLength + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
                                                exceptionInfo);
    } else {
        fwException_raise(exceptionInfo,"ERROR","cpcConfigGenericFunctions_checkArchiveConfig:" + getCatStr("unGeneration","BADLENGTH"), "");
    }
}

//----------------------------------------------------------------------------------------------------------------------------------
// cpcConfigGenericFunctions_processArchives
/**
Gets archiving configs from object and puts them to the common archiving configs.

  @param[in]  iToSkip   Num elements in dsConfigs to skip before common part. (ex. GCS_CONFIG_ANALOGPAR_LENGTH)
  @param[out] dsConfigs Configs
*/
cpcConfigGenericFunctions_processArchives(int iToSkip, bool hasArchive, dyn_string& dsConfigs) {
    if (hasArchive) {
        if(dsConfigs[iToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "") {
            dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[iToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
        }

        if(dsConfigs[iToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "") {
            dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[iToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
        }

        if(dsConfigs[iToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "") {
            dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[iToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
        }
    }
}

//----------------------------------------------------------------------------------------------------------------------------------
// cpcConfigGenericFunctions_checkBool
/**
Check if we have correct boolean value in input string and return it in bRes

  @param[in]  bSource       Input string. To check inside if it is correct boolean value.
  @param[out] bRes          Flag to indicate if we have correct value it will be returned here.
  @param[out] exceptionInfo All possible errors will be returned here
*/
void cpcConfigGenericFunctions_checkBool(string name, string bSource, bool& bRes, dyn_string &exceptionInfo) {
  bSource = strtolower(strrtrim(strltrim(bSource)));
  if(bSource == "true" || bSource == "1" || bSource == "y" || bSource == "yes") {
    bRes = true;
  } else if(bSource == "false" || bSource == "0" || bSource == "n" || bSource == "no") {
    bRes = false;
  } else {
    fwException_raise(exceptionInfo,"ERROR","cpcConfigGenericFunctions_checkBool: the value '" + bSource + "' is not correct for the " + name, "");
  }
}

string cpcConfigGenericFunctions_getFrontEndType() {
    string sFrontEndType;
    if(shapeExists("TextFieldPlcType")) {
        sFrontEndType = TextFieldPlcType.text;
        if ((sFrontEndType == UN_CONFIG_PREMIUM) || (sFrontEndType == UN_CONFIG_QUANTUM) || (sFrontEndType == UN_CONFIG_UNITY))
            sFrontEndType = UN_PLC_DPTYPE;
        else if ((sFrontEndType == UN_CONFIG_S7_400) || (sFrontEndType == UN_CONFIG_S7_300))
            sFrontEndType = S7_PLC_DPTYPE;
    } else {
        DebugN("cpcConfigGenericFunctions_getFrontEndType: can't detect front-end type");
    }
    return sFrontEndType;
}

int cpcConfigGenericFunctions_translateDataType(string plcType, int commonDataType) {
    if(plcType == UN_PLC_DPTYPE) {
        switch (commonDataType) {
            case CPC_INT16:	return PVSS_MODBUS_INT16;
            case CPC_INT32:	return PVSS_MODBUS_INT32;
            case CPC_UINT16:	return PVSS_MODBUS_UINT16;
            case CPC_BOOL:	return PVSS_MODBUS_BOOL;
            case CPC_FLOAT:	return PVSS_MODBUS_FLOAT;
        }
    } else {
        switch (commonDataType) {
            case CPC_INT16:	return PVSS_S7_INT16;
            case CPC_INT32:	return PVSS_S7_INT32;
            case CPC_UINT16:	return PVSS_S7_UINT16;
            case CPC_BOOL:	return PVSS_S7_BOOL;
            case CPC_FLOAT:	return PVSS_S7_FLOAT;
        }
    }
}

int cpcConfigGenericFunctions_translateDataTypeForS7Check(int commonDataType) {
    switch (commonDataType) {
        case CPC_INT16:	return UN_CONFIG_S7_PLC_DATATYPE_WORD;
        case CPC_INT32:	return UN_CONFIG_S7_PLC_DATATYPE_DWORD;
        case CPC_UINT16:	return UN_CONFIG_S7_PLC_DATATYPE_WORD;
        case CPC_BOOL:	return UN_CONFIG_S7_PLC_DATATYPE_BOOL; //@TODO: verify with reality 
        case CPC_FLOAT:	return UN_CONFIG_S7_PLC_DATATYPE_FLOAT;
    }
}
