/**@name LIBRARY: cpcAnalogInputConfig.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 31 10 2002

Modification History: 
    23/03/2012: Alexey
        [UCPC-255] add SOFT_FE

    26/10/2011: Alexey
        [UCPC-557] use cpcExportGenericFunctions_getKeyDeviceConfiguration instead of cpcExportGenericFunctions_getSmsMessage

	03/10/2011: Alexey
		[UCPC-562] removed normal position and added alert masked to the importation line

	28/07/2011: Alexey
		[UCPC-436] store sms message

	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	21/04/2011: amerezhi
		moved 5 ranges alerts to generic config lib

	29/03/2011: amerezhi
		- [UCPC-201] changes code structure according new model

	18/04/2006: Herve
		- implementation of mask event

	06/05/2005: Enrique
		- new function: S7_PLC_CPC_AnalogInput_checkConfig to check the S7_PLC configuration
		- new function: S7_PLC_CPC_AnalogInput_setConfig to set the S7_PLC configuration 
			
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_CPC_AnalogInput_checkConfig, _UnPlc_CPC_AnalogInput_setConfig

version 1.0

External Function : 
	. CPC_AnalogInputConfig_setConfig :  set configuration of given Analog Input datapoint
	. CPC_AnalogInputConfig_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Analog Input configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const string UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME = "CPC_AnalogInput";
const unsigned UN_CONFIG_CPC_ANALOGINPUT_LENGTH = 22;

const unsigned UN_CONFIG_CPC_ANALOGINPUT_UNIT					= 1;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_FORMAT					= 2;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_RANGEMAX				= 3;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_RANGEMIN				= 4;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_HHLIMIT				= 5;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_HLIMIT					= 6;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_LLIMIT					= 7;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_LLLIMIT				= 8;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ALARM_ACTIVE   		= 9;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_DEADBAND				= 10;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_DEADBAND_TYPE			= 11;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ARCHIVE_ACTIVE			= 12;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ARCHIVE_TIME_FILTER	= 13;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_SMS_CATEGORIES			= 14;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_SMS_MESSAGE			= 15;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ACK_ALARM				= 16;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDRESS_STSREG01		= 17;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDRESS_EVSTSREG01		= 18;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDRESS_POSST			= 19;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDRESS_HFSTORAUPOSRST	= 20;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDRESS_MANREG01		= 21;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDRESS_MPOSR			= 22;

const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDITIONAL_LENGTH          = 6;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDITIONAL_MASTER_NAME     = 2;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDITIONAL_PARENTS         = 3;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDITIONAL_CHILDREN        = 4;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDITIONAL_TYPE            = 5;
const unsigned UN_CONFIG_CPC_ANALOGINPUT_ADDITIONAL_SECOND_ALIAS    = 6;

mapping CPC_AnalogInputConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg01";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg01"] 		= props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg01";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg01"] 	= props;

	mappingClear(props);
	props["5RangesAlerts"]	= true;
	props["hasUnit"]		= true;
	props["hasFormat"]		= true;
	props["hasArchive"] 	= true;
	props["hasPvRange"] 	= true;
	props["hasSmooth"] 		= true;
	props["address"] 		= ".ProcessInput.PosSt";
	props["dataType"] 		= CPC_FLOAT;
	config["PosSt"] 		= props;

	mappingClear(props);
	props["hasUnit"]		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["address"] 		= ".ProcessInput.HFStOrAuPosRSt";
	props["dataType"] 		= CPC_FLOAT;
	config["HFStOrAuPosRSt"]= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.ManReg01";
	props["dataType"] 		= CPC_UINT16;
	config["ManReg01"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.MPosR";
	props["dataType"] 		= CPC_FLOAT;
	config["MPosR"] 		= props;

	return config;
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInputConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnalogInputConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_AnalogInputConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, configLine, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInputConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for AnalogInput object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnalogInputConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_AnalogInput_ExportConfig
/**
Purpose: Export AnalogInput Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_CPC_AnalogInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_get5AlertLimits(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getAlarmActive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getCategories(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getKeyDeviceConfiguration(currentDP, dsDpParameters, CPC_CONFIG_SMS_MESSAGE, exceptionInfo);
        cpcExportGenericFunctions_get5AlertAcknowledgeAlarm(currentDP, ".ProcessInput.PosSt", dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt", 		true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "HFStOrAuPosRSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosR", 	false, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_AnalogInput_ExportConfig
/**
Purpose: Export AnalogInput Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_CPC_AnalogInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_AnalogInput_ExportConfig(dsDpList, exceptionInfo);
}


const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_UNIT                  = 1;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_FORMAT                = 2;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_RANGEMAX              = 3;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_RANGEMIN              = 4;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_HHLIMIT               = 5;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_HLIMIT                = 6;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_LLIMIT                = 7;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_LLLIMIT               = 8;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_ALARM_ACTIVE          = 9;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_ARCHIVE_ACTIVE        = 10;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_ARCHIVE_TIME_FILTER   = 11;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_SMS_CATEGORIES        = 12;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_SMS_MESSAGE           = 13;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_ACK_ALARM             = 14;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_LENGTH                = 14;

const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_ADDITIONAL_LENGTH     = 1;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGINPUT_ADDITIONAL_MASKEVENT  = 1;

void SOFT_FE_CPC_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, configLine, exceptionInfo);
}

void SOFT_FE_CPC_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

void SOFT_FE_CPC_AnalogInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_get5AlertLimits(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getAlarmActive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getCategories(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getKeyDeviceConfiguration(currentDP, dsDpParameters, CPC_CONFIG_SMS_MESSAGE, exceptionInfo);
        cpcExportGenericFunctions_get5AlertAcknowledgeAlarm(currentDP, ".ProcessInput.PosSt", dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}
