/**@name LIBRARY: unAnalogParameter.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Teodora Miteva (IT-CO)
@author: Andres Abad Rodriguez (IT-CO-FE)
@date 15.10.2004
@brief This library contains AnalogStatus object functions.
Modifications History:
	$Log: unAnalogParameter.ctl,v $
    29/02/2012: Alexey
        [UCPC-684] "Set Value" available for "operator"

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    20/10/2011: Alexey
        use CPC_STSREG01_ARMRCPST instead of UN_STSREG01_MMOST for recipe bit

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate

	08/08/2011: Alexey
		fixed faceplate Disconnect animation

	25/05/2011: Alexey
		[UCPC-279] display event mask in the faceplate

	21/05/2011: Alexey
		[UCPC-271] display on widget and control event mask

	18/05/2011: Alexey
		use CPC functions to manage menu

	Revision 1.32  2010/04/22 05:59:14  gthomas
	Fixed #66271 (added display of Units on Widget)
	
	Revision 1.31  2009/03/31 14:04:23  gthomas
	in _WidgetAnimationCB(): moved animation fct call for quality (N is higher priority than O) at the top
	in _WidgetDisconnection(): removed AlertText animation
	in _FaceplateStatusAnimationCB(): removed animation check of PLC disconnection for Ranges and the Default value (not required as no PLC connection)
	in _ProvideAccessToButtons(): removed exceptionInfo as returns errors in log when monitor or no user are logged in.

	Revision 1.30  2008/11/12 08:23:20  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.29  2008/10/15 14:43:26  gthomas
	-Removed the <if statement> of the dpConnect in CPC_AnalogParameter_ButtonRegisterCB function
	to fix problem with select/deselect button animation.

	Revision 1.28  2008/07/30 14:19:52  gamx
	Changes in "WidgetDisconnection": objects "ControlStateText" and "eventState" have been removed as they are not presented on the widget panel

	Revision 1.27  2008/07/22 15:09:01  gthomas
	added missing ExceptionInfo missing in _ProvideAccessToButtons()

	Revision 1.26  2008/06/19 12:09:26  gthomas
	integrated new Access Control of UNICOS 4-1 (by instance)
	removed functions _ButtonUserAccessUn and _ButtonUserAccessGcs

	Revision 1.25  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.24  2008/05/26 12:18:16  gthomas
	Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
	to be used with PVSS 3.6 QT


	Revision 1.23  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.22  2008/01/22 12:52:44  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.21  2008/01/21 16:55:59  aabadz
	Fixed error found when we open the faceplate. We was calling to a incorrect function

	Revision 1.18  2008/01/10 15:24:07  aabadz
	Changes after code review and before tests

	Revision 1.17  2007/10/30 16:13:58  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

	17/12/07 Enrique (AB/CO)
	- Delete functions:
			AnalogParameter_EventList_Configure_Init()
			AnalogParameter_EventList_Configure_Clic()
		AnalogParameter_EventList_Configure_Save()
	- 06/12/06 Enrique (AB/CO)
	Change visualization color in the faceplate for min/max

*/
/*
Internal Functions :
	CPC_AnalogParameter_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
	CPC_AnalogParameter_HandleMenu (string sDeviceName, string sDpType, dyn_string menuList, int menuAnswer)
	CPC_AnalogParameter_FaceplateStatusRegisterCB (string sSystemStatus, bool bSystemConnected)
	CPC_AnalogParameter_FaceplateConnect (string sDeviceName)
	CPC_AnalogParameter_FaceplateDisconnect (string sDeviceName)
	CPC_AnalogParameter_FaceplateStatusAnimationCB (string sDpMPosRSt, float fMPosRSt, string sDp4, bool bMPosRStInvalid, ...)
	CPC_AnalogParameter_animatePlcDisconnection ()
	CPC_AnalogParameter_ButtonRegisterCB (string sSystemStatus, bool bSystemConnected)
	CPC_AnalogParameter_ButtonAnimationCB (string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, string sDpMPosRSt, float fMPosRStValue, string sDpMPosRStBad, bool bMPosRStBad)
	CPC_AnalogParameter_ButtonUserAccess (string sDpName, string sDpType, dyn_string &dsAccess)
	CPC_AnalogParameter_UserLoginGetButtonState (string sDeviceName, string sType, dyn_string &dsReturnData)
	CPC_AnalogParameter_ButtonSetState (string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
	CPC_AnalogParameter_ButtonDisconnection ()
	CPC_AnalogParameter_WidgetRegisterCB (string sSystemStatus, bool bSystemConnected)
	CPC_AnalogParameter_WidgetConnect (string sDeviceName, string sPlcName, string sSystemName)
	CPC_AnalogParameter_WidgetDisconnect (string sDeviceName, string sPlcName, string sSystemName)
	CPC_AnalogParameter_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, ...)
	CPC_AnalogParameter_WidgetDisconnection (string sWidgetType)
	AnalogParameter_EventList_Configure_Init (dyn_string &exceptionInfo, const bool bUpdate=false)
	AnalogParameter_EventList_Configure_Clic (int iRow, string sColumn, string sValue, dyn_string &exceptionInfo)
	AnalogParameter_EventList_Configure_Save (dyn_string &exceptionInfo)
*/


//------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName Device name
	@param[in]	sDpType Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] menuList Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
CPC_AnalogParameter_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_MASK_POSST_TEXT,
			 UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	deviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	menuList 	List of requested action
	@param[in]	menuAnswer 	Selected action from the menuList
*/
CPC_AnalogParameter_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
}

//------------------------------------------------------------------------------------
//CPC_AnalogParameter_WidgetRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_AnalogParameter_WidgetRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sPlcName, sSystemName;
	string sPlcDpName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (sDeviceName == "")	// In case of disconnection
	{
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch(iAction)
	{
		case UN_ACTION_DISCONNECT:
			CPC_AnalogParameter_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
				if(dpTypeName(sDeviceName) == UN_CONFIG_CPC_ANALOGPARAMETER_DPT_NAME)
				CPC_AnalogParameter_WidgetConnect(sDeviceName, sPlcName, sSystemName);
			else{
				g_bUnSystemAlarmPlc = false;
				CPC_AnalogParameter_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_AnalogParameter_WidgetDisconnect(sDeviceName, sPlcName, sSystemName);
			CPC_AnalogParameter_WidgetDisconnection (g_sWidgetType);
			break;
		case UN_ACTION_NOTHING: break;
		default:
				break;
	}
}


//------------------------------------------------------------------------------------------------------------
//CPC_AnalogParameter_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
CPC_AnalogParameter_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes=0;
	if(g_bUnSystemAlarmPlc)
	{
		g_sMPosRStFormat = dpGetFormat(sDeviceName + ".ProcessInput.MPosRSt");
		g_sMPosRStUnit = dpGetUnit(sDeviceName + ".ProcessInput.MPosRSt");
		g_sPosStFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosSt");
		g_sPosStUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosSt");

		iRes = dpConnect("CPC_AnalogParameter_WidgetAnimationCB",
				sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				sDeviceName + ".statusInformation.selectedManager",
				sDeviceName + ".ProcessInput.MPosRSt",
				sDeviceName + ".ProcessInput.MPosRSt:_online.._invalid",
				sDeviceName + ".ProcessInput.PosSt",
				sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
				sDeviceName + ".eventMask");
		g_bCallbackConnected = (iRes >= 0);
	}
	else
		CPC_AnalogParameter_WidgetDisconnection(g_sWidgetType);
}


//------------------------------------------------------------------------------------------------------------
//CPC_AnalogParameter_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
CPC_AnalogParameter_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes;
	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("CPC_AnalogParameter_WidgetAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.MPosRSt",
			sDeviceName + ".ProcessInput.MPosRSt:_online.._invalid",
			sDeviceName + ".ProcessInput.PosSt",
			sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			sDeviceName + ".eventMask");
		g_bCallbackConnected = !(iRes >= 0);
	}
}


//------------------------------------------------------------------------------------------------------------
//CPC_AnalogParameter_WidgetAnimationCB
/**
Animate widget
*/
CPC_AnalogParameter_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
		string sDpMPosRSt, float fMPosRSt, string sDp4, bool bMPosRStInvalid,
		string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
		string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sEvtMask, int iMask)
{
    string sBody1Text, sBody1Color, sDisplay1Text, ssDisplay1Color, sColorSelect;
    string sFormattedValue, sFormattedValueBody, sWarningLetter, sWarningColor;
    bool bLockVisible;

  //data quality information: "N" is of higher priority than "O"
 if((!bSystemIntegrityAlarmEnabled)||(iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}
    
	sFormattedValueBody = unGenericObject_FormatValue(g_sMPosRStFormat, fMPosRSt);
	//control animation
	if (bMPosRStInvalid)
	{
		sBody1Text = sFormattedValueBody + " " + g_sMPosRStUnit;
		sBody1Color = "unDataNotValid";

		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}
	else
	{
    
		sBody1Text = sFormattedValueBody + " " + g_sMPosRStUnit;
		sBody1Color = "cpcColor_Widget_Request";
	}

	sFormattedValue = unGenericObject_FormatValue(g_sPosStFormat, fPosSt);
	//control animation
	if (bPosStInvalid)
	{
		sDisplay1Text = sFormattedValue + " " + g_sPosStUnit;
		ssDisplay1Color = "unDataNotValid";

		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}
	else
	{
		sDisplay1Text = sFormattedValue + " " + g_sPosStUnit;
		ssDisplay1Color = "cpcColor_Widget_Status";
	}
 
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
	if(g_bSystemConnected)
	{
		setMultiValue ("Body1", "text", sBody1Text, 
			       "Body1", "foreCol", sBody1Color,           
			       "Display1", "text", sDisplay1Text, 
			       "Display1", "foreCol", ssDisplay1Color, 
			       "WarningText", "text", sWarningLetter, 
			       "WarningText", "foreCol", sWarningColor,
			       "SelectArea", "foreCol", sColorSelect, 
			       "LockBmp", "visible", bLockVisible,
			       "WidgetArea", "visible", true);
	}
	if(g_bSystemConnected) {
		eventState.visible = iMask == 0;
	}

}


//------------------------------------------------------------------------------------------------------------
//CPC_AnalogParameter_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
CPC_AnalogParameter_WidgetDisconnection(string sWidgetType)
{
	setMultiValue("Body1", "foreCol", "unDataNoAccess", 
			 "Display1","foreCol","unDataNoAccess", 
			 "WarningText", "text", "",
			 "SelectArea", "foreCol", "", "LockBmp", "visible", false, "WidgetArea", "visible", false);
	eventState.visible	= false;
}



//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemName 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_AnalogParameter_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sSystemName, sPlcName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcDpName;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);
		if(bRemote)
			g_bSystemConnected = bSystemConnected;
		else
			g_bSystemConnected = true;

		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)
			g_bUnSystemAlarmPlc = true;
		else
			g_bUnSystemAlarmPlc = false;

		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				CPC_AnalogParameter_animatePlcDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				CPC_AnalogParameter_FaceplateConnect(sDeviceName);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				CPC_AnalogParameter_FaceplateDisconnect(sDeviceName);
				CPC_AnalogParameter_animatePlcDisconnection();
				break;
			case UN_ACTION_NOTHING: break;
			default:
				break;
		}
	}

}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_FaceplateConnect
/**
Do faceplate dpConnect

	@param[in]	sDeviceName	Device DP name
*/
CPC_AnalogParameter_FaceplateConnect(string sDeviceName)
{
	int iRes=0;
	dyn_string exceptionInfo;
	dyn_string sPlcDpName;
	string sPlcName, sSystemName;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
// get the PLC name
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
		else g_bUnSystemAlarmPlc = false;

	if (g_bUnSystemAlarmPlc)
	{
		g_sMPosRStFormat = dpGetFormat(sDeviceName + ".ProcessInput.MPosRSt");
		g_sMPosRStUnit = dpGetUnit(sDeviceName + ".ProcessInput.MPosRSt");
		g_sPosStFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosSt");
		g_sPosStUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosSt");

		iRes = dpConnect("CPC_AnalogParameter_FaceplateStatusAnimationCB",
			 sDeviceName + ".ProcessInput.StsReg01",
			 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			 sDeviceName + ".ProcessInput.MPosRSt",
			 sDeviceName + ".ProcessInput.MPosRSt:_online.._invalid",
			 sDeviceName + ".ProcessInput.PosSt",
			 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
			 sDeviceName + ".ProcessInput.PosSt:_pv_range.._min",
			 sDeviceName + ".ProcessInput.PosSt:_pv_range.._max",
			 sDeviceName + ".ProcessOutput.DfltVal",
			 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			 sDeviceName + ".eventMask");
	}
	else
		CPC_AnalogParameter_animatePlcDisconnection();

	g_bCallbackConnected = (iRes >= 0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	sDeviceName	Device DP name
*/
CPC_AnalogParameter_FaceplateDisconnect(string sDeviceName)
{
	int iRes = -1;
	string sPlcName, sSystemName;
	string sPlcDpName;
	dyn_string exceptionInfo;

	if (g_bUnSystemAlarmPlc)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
		else g_bUnSystemAlarmPlc = false;

		iRes = dpDisconnect("CPC_AnalogParameter_FaceplateStatusAnimationCB",
			 sDeviceName + ".ProcessInput.StsReg01",
			 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			 sDeviceName + ".ProcessInput.MPosRSt",
			 sDeviceName + ".ProcessInput.MPosRSt:_online.._invalid",
			 sDeviceName + ".ProcessInput.PosSt",
			 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
			 sDeviceName + ".ProcessInput.PosSt:_pv_range.._min",
			 sDeviceName + ".ProcessInput.PosSt:_pv_range.._max",
			 sDeviceName + ".ProcessOutput.DfltVal",
			 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			 sDeviceName + ".eventMask");

		g_bCallbackConnected = !(iRes >= 0);
	}
	else
	{
		CPC_AnalogParameter_WidgetDisconnection(g_sWidgetType);
		g_bCallbackConnected = false;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
CPC_AnalogParameter_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
		 string sDpMPosRSt, float fMPosRSt, string sDp4, bool bMPosRStInvalid,
		 string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
		 string sDp7, float fPosStMin, string sDp8, float fPosStMax,
		 string sDp10, float fDefaultVal,
		 string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
		 string sDpEvt, int iMask)
{
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sMPosRStFormat, g_sMPosRStUnit, fMPosRSt, "RqValue", "cpcColor_Faceplate_Request",
			bMPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosSt, "CurVal_Status", "cpcColor_Faceplate_Status",
			bPosStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fDefaultVal, "DefaultValue", "unDisplayValue_Parameter",0);
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosStMin, "RangePosStMin", "unDisplayValue_Parameter",0);
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosStMax, "RangePosStMax", "unDisplayValue_Parameter",0);
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ArmedRecipe", getBit(bit32StsReg01, CPC_STSREG01_ARMRCPST), 1, "cpcColor_Faceplate_Status",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_animatePlcDisconnection
/**
Animate faceplate disconnection
*/
CPC_AnalogParameter_animatePlcDisconnection()
{
    unGenericObject_DisplayValueDisconnect(makeDynString("RqValue", "CurVal_Status", "DefaultValue", "RangePosStMin", "RangePosStMax"));
    unGenericObject_ColorBoxDisconnect(makeDynString("ArmedRecipe", "eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
CPC_AnalogParameter_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS)
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	else
	{
		dynAppend(exceptionInfo, "AnalogParameter_ButtonRegisterCB: the list of functions is not correct - number of functions is less than expected");
		return;
	}

	if (sFunction != "")
	{
		evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" + sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, sDpType);

			g_dsUserAccess=dsUserAccess;
	}
	else
		fwException_raise(exceptionInfo,"ERROR", "AnalogParameter_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

}

//---------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_ButtonRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_AnalogParameter_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction, sSystemName;
	int iAction, iRes;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;
	string sPlcName;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				CPC_AnalogParameter_ButtonDisconnection();
				break;
			case UN_ACTION_DPCONNECT:

				sDpType=UN_CONFIG_CPC_ANALOGPARAMETER_DPT_NAME;
				unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
				if (dynlen(exceptionInfo) <= 0)	CPC_AnalogParameter_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
				
				iRes = dpConnect("CPC_AnalogParameter_ButtonAnimationCB",
						sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
						sDeviceName + ".statusInformation.selectedManager",
						sDeviceName + ".ProcessInput.MPosRSt",
						sDeviceName + ".ProcessInput.MPosRSt:_online.._aut_inv");
				g_bCallbackConnected = (iRes >= 0);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				CPC_AnalogParameter_ButtonsDisconnect(sDeviceName);
				CPC_AnalogParameter_ButtonDisconnection();
				break;
			case UN_ACTION_NOTHING: break;
			default:
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
//CPC_AnalogParameter_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
CPC_AnalogParameter_ButtonsDisconnect(string deviceName)
{
	int iRes = -1;

	iRes = dpDisconnect("CPC_AnalogParameter_ButtonAnimationCB",
				deviceName + ".statusInformation.selectedManager:_lock._original._locked",
				deviceName + ".statusInformation.selectedManager",
				deviceName + ".ProcessInput.MPosRSt",
				deviceName + ".ProcessInput.MPosRSt:_online.._aut_inv");
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_ButtonAnimationCB
/**
Animate buttons

@note This function works with an unicos button panel and needs global variables declared in this panel -
		"g_dsUserAccess" (dyn_string). Button identifier is added to this variable if user has access.
*/
CPC_AnalogParameter_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
    string sDpMPosRSt, float fMPosRStValue, string sDpMPosRStBad, bool bMPosRStBad)
{
	CPC_AnalogParameter_ButtonSetState(unGenericDpFunctions_getDpName(sDpMPosRSt), UN_CONFIG_CPC_ANALOGPARAMETER_DPT_NAME, g_dsUserAccess,
		makeDynString(bLocked, sSelectedManager, fMPosRStValue, bMPosRStBad, 0, 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_ButtonUserAccess
/**
User access for buttons

	@param[in] 		sDpName	Device name
	@param[in] 		sDpType	dpType
	@param[out] 	dsAccess	List of allowed buttons
*/
CPC_AnalogParameter_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	
	bool operator, expert, admin;
		dyn_bool dbPermissions;
		dyn_string exceptionInfo;
		string sOperAction, sExpAction, sAdminAction;
		bool bActionDefined = false;
		
		unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
		if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
		(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
			bActionDefined=true;
			operator = dbPermissions[2];
			expert = dbPermissions[3];
			admin = dbPermissions[4];
			dsAccess = makeDynString();
		if (operator){
			if(bActionDefined)
				dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
			else
			    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, CPC_FACEPLATE_BUTTON_PARAMETER_SET_VALUE));
		}
		if (expert){
			if(bActionDefined)
		            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		        else
			    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_MASKEVENT));
		}
	        if(admin){
	        	if(bActionDefined)
		           dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	sDeviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
CPC_AnalogParameter_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	int iRes = -1;
	string sDpLock, sDpSelectedManager, sSelectedManager;
	bool bLocked;

	sDpLock = sDeviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = sDeviceName + ".statusInformation.selectedManager";
	if (dpExists(sDeviceName))
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);

	if (iRes < 0) // Errors during dpget
	{
		sDpLock = "";
		bLocked = false;
		sDpSelectedManager = "";
		sSelectedManager = "";
	}
	dsReturnData=makeDynString(bLocked, sSelectedManager, 0, 0, 0, 0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
CPC_AnalogParameter_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled;
	dyn_string dsButtons;
	int i, iDsButtonsLength;
	string localManager;
	bool bLocked, bStsReg01Bad;
	string sSelectedManager;
	bit32 stsReg01Value;

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];

	dsButtons = makeDynString(CPC_FACEPLATE_BUTTON_PARAMETER_SET_VALUE);// Each button except "Select" button
	iDsButtonsLength = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state

	for(i = 1; i <= iDsButtonsLength; i++)
	{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
		{
			default:
				break;
		}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled && bSelected);
	}
	// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));

}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogParameter_ButtonDisconnection
/**
Animate disconnection
*/
CPC_AnalogParameter_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, CPC_FACEPLATE_BUTTON_PARAMETER_SET_VALUE));
}


//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_AnalogParameter__ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: XP.
	. distributed system: yes.
*/

CPC_AnalogParameter_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
    float value;
    string sTime;

    dpGet(sDeviceName+".ProcessInput.PosSt", value,
          sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);

    dsReturnData[1] = sTime;
    dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " "
                + dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}
