/**@name LIBRARY: unAnalogStatus.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains AnalogStatus object functions.
Modifications History:
    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    03/11/2011: Alexey
        use stepPLC code from cpcStepPLC.ctl; same for transitionPLC
        fixed widget animation on distributed

	18/05/2011: Alexey
		use CPC functions to manage menu

	2011/04/08 amerezhi
		[UCPC-190] renamed AuPosRSt->PosSt for unCPC6
	
	$Id: unAnalogStatus.ctl 46255 2010-09-16 07:38:35Z gthomas $:
	Fixed variable typo in CPC_AnalogStatus_WidgetAnimationCB()
			
	$Log: unAnalogStatus.ctl,v $
	Revision 1.19  2009/03/27 14:15:58  gthomas
	 in _WidgetRegisterCB() added evaluation on widget type for Word2AnalogStatus
	
	Revision 1.18  2009/03/25 10:07:01  gthomas
	-Fixed animation problem when the $sIdentifier of the widget refers to a DPname of a different DP type.
	Fixes made in _WidgetRegisterCB()

	Revision 1.17  2008/07/23 10:02:25  gthomas
	-removed commented code (for error handling)

	Revision 1.16  2008/06/13 12:29:47  aabadz
	Modifications asked by UNICOS for their version 4.X

	Revision 1.15  2008/06/03 08:01:29  aabadz
	Fixed error in background color animation

	Revision 1.14  2008/05/26 12:18:18  gthomas
	Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
	to be used with PVSS 3.6 QT

	Revision 1.13  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.12  2007/12/05 11:15:34  aabadz
	Corrections following the document from code review. Version after testing

	Revision 1.11  2007/11/27 14:15:14  aabadz
	Corrections following the document from code review

	Revision 1.10  2007/10/30 16:14:45  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

		17/12/2007 Enrique (AB/CO)
		- Delete functions:
				AnalogStatus_EventList_Configure_Init()
				AnalogStatus_EventList_Configure_Clic()
				AnalogStatus_EventList_Configure_Save()

		16/01/2008 : Arnaud AB/CO
		- Modify  WidgetAnimationCB to use it with UNICOS step and transition widgets


	  	20/08/2007: Enrique AB/CO
	  	-function CPC_AnalogStatus_ObjectListGetValueTime added.
		11/05/2007: Herve AB/CO


		11/05/2004: Herve AB/CO
		- function CPC_AnalogStatus_WidgetRegisterCB
		bug in case the PLC is not defined and if the PVSS system name contains '-',
		call unGenericObject_GetPLCNameFromDpName to get the PLC DPName instead (this function corrects the bugs).

*/
/*
Internal Functions :
	CPC_AnalogStatus_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
	CPC_AnalogStatus_HandleMenu (string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
	CPC_AnalogStatus_WidgetRegisterCB (string sDp, bool bSystemConnected)
	CPC_AnalogStatus_WidgetConnect (string deviceName, string sPlcName, string sSystemName)
	CPC_AnalogStatus_WidgetDisconnect (string deviceName, string sPlcName, string sSystemName)
	CPC_AnalogStatus_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, string sDpPosSt, float fPosSt, ...)
	CPC_AnalogStatus_WidgetDisconnection (string sWidgetType)
*/


//------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatus_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName Device name
	@param[in]	sDpType Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] menuList Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
CPC_AnalogStatus_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(CPC_POPUPMENU_DIAG_INFO_TEXT, UN_POPUPMENU_TREND_TEXT);

	cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatus_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	deviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	menuList 	List of requested action
	@param[in]	menuAnswer 	Selected action from the menuList
*/
CPC_AnalogStatus_HandleMenu(string deviceName, string sDpType, dyn_string sMenuList, int sMenuAnswer)
{
	dyn_string exceptionInfo;

	cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, sMenuList, sMenuAnswer, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatus_WidgetRegisterCB
/**
Register callback function

	@param[in]	sDp 	Flag for distributed system
	@param[in] 	bSystemConnected 	Flag
*/
CPC_AnalogStatus_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string sDeviceName = "", sPlcName = "", sSystemName = "";
	int iAction = UN_ACTION_NOTHING;
	dyn_string exceptionInfo;
	bool bRemote = false;
    int iRes;

	sDeviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (sDeviceName == "") { // In case of disconnection
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

	if ("" != sSystemName) {
		unDistributedControl_isRemote(bRemote, sSystemName);
	}

	if(bRemote) {
		g_bSystemConnected = bSystemConnected;
	} else {
		g_bSystemConnected = true;
	}

	if(sPlcName != "") {
		string sPlcDpName = "";
		dyn_string dsPlcDpNameExceptions;
		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);
		g_bUnSystemAlarmPlc = 0 == dynlen(dsPlcDpNameExceptions);
	} else {
		g_bUnSystemAlarmPlc = false;
	}

	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch (iAction) {
		case UN_ACTION_DISCONNECT:
			CPC_AnalogStatus_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
			if((dpTypeName(sDeviceName) == UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME) || (dpTypeName(sDeviceName) == UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME)) {
				CPC_AnalogStatus_WidgetConnect(sDeviceName, sPlcName, sSystemName);
			} else {
				g_bUnSystemAlarmPlc = false;
				CPC_AnalogStatus_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
            CPC_AnalogStatus_WidgetDisconnect(sDeviceName, sPlcName, sSystemName);
            CPC_AnalogStatus_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:break;
		default:
			break;
	}
}

//--------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatus_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	deviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/


CPC_AnalogStatus_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes;
    string sPlcDpName = "";
    dyn_string dsPlcDpNameExceptions;

    if (g_bUnSystemAlarmPlc) {
        g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
        g_sPosStUnit = dpGetUnit(deviceName + ".ProcessInput.PosSt");

        cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);
        if (0 == dynlen(dsPlcDpNameExceptions)) {
            iRes = dpConnect("CPC_AnalogStatus_WidgetAnimationCB",
                             deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                             deviceName + ".statusInformation.selectedManager",
                             deviceName + ".ProcessInput.PosSt",
                             deviceName + ".ProcessInput.PosSt:_online.._invalid",
                             sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                             sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled");
            g_bCallbackConnected = (iRes >= 0);
        }
    } else {
        CPC_AnalogStatus_WidgetDisconnection (g_sWidgetType);
    }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatus_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	deviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
CPC_AnalogStatus_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	if (g_bUnSystemAlarmPlc)
	{
		string sPlcDpName = "";

		dyn_string dsPlcDpNameExceptions;
		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);

		if (dynlen(dsPlcDpNameExceptions) == 0)
		{
			iRes = dpDisconnect("CPC_AnalogStatus_WidgetAnimationCB",
				deviceName + ".statusInformation.selectedManager:_lock._original._locked",
				deviceName + ".statusInformation.selectedManager",
				deviceName + ".ProcessInput.PosSt",
				deviceName + ".ProcessInput.PosSt:_online.._invalid",
				sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName + ".alarm",
				sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName + ".enabled");
			g_bCallbackConnected = !(iRes >= 0);
		}
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatus_WidgetAnimationCB
/**
Animate widget
*/
CPC_AnalogStatus_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                                  string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid,
                                  string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
{
	string sFormattedValue;
	string sBody1Text, sBody1Color;
	string sWarningLetter, sWarningColor; // put new variables

	if((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)) // put here if sentance
	{
		sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}

	sFormattedValue = unGenericObject_FormatValue(g_sPosStFormat, fPosSt);
	sBody1Text = sFormattedValue + " " + g_sPosStUnit;
	// 1. Control animation. Check if the datapoint is invalid, then will be displayed in blue colors
	if (bPosStInvalid) {
		sBody1Color = "unDataNotValid";
		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	} else {
		sBody1Color = "cpcColor_Widget_Status";
	}

    if(g_bSystemConnected){
        setMultiValue("Body1", "text", sBody1Text,"Body1", "foreCol", sBody1Color,
                      "WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
                      "WidgetArea", "visible", true);
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatus_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
CPC_AnalogStatus_WidgetDisconnection(string sWidgetType)
{
    setMultiValue("Body1", "foreCol", "unDataNoAccess","Body1", "backCol", "unWidget_Background", 
                   "WarningText", "text", "", "WidgetArea", "visible", false);
}




// CPC_AnalogStatus__ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: XP.
	. distributed system: yes.
*/

CPC_AnalogStatus_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.PosSt", value,
	      sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.PosSt:_online.._bad", bBad);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.PosSt");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	// Status to visualize
	sStatus="OK";
	dsReturnData[4] = sStatus;
	// Colors
	sColor="unEventList_ForeColor";
	dsReturnData[5]=sColor;
}
