/**@name LIBRARY: cpcAnalogStatusConfig.ctl

@author: Artem Burmyakov (IT-CO-FE)
@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set AnalogStatus configuration.
Modifications History:
  $Log: CPC_AnalogStatusConfig.ctl,v $
    23/03/2012: Alexey
        [UCPC-252] add SOFT_FE

	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params

	2011/05/31 Alexey
		[UCPC-297] removed MaskEvent

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

  2011/04/08 amerezhi
  [UCPC-190] updated import for unCPC6

  2011/02/08 amerezhi
  Check and set device using UNCCPCConfig' general functions

  Revision 1.26  2008/07/29 08:00:12  gamx
  fixes in "getArchiveSettings" function:
  archive parameters are handled correctly + the order of parameters is not fixed

  Revision 1.25  2008/06/13 12:29:47  aabadz
  Modifications asked by UNICOS for their version 4.X

  Revision 1.24  2008/01/11 14:10:55  gamx
  reviewed naming of variables

  Revision 1.23  2007/12/03 14:01:04  gamx
  fully checked version, fixed syntactic errors

  Revision 1.22  2007/12/03 13:20:37  gamx
  "dedband" is replaced with "DEADBAND"

  Revision 1.21  2007/11/29 11:32:53  gamx
  Functions to check and set WS as As have been moved from here to WS library.

  Revision 1.20  2007/11/28 16:16:21  gamx
  modifications after code review:
  changes to be consistent with naming conventions,
  introduced constants (to keep dpt name) and the set of external functions (to extract archive settings),
  exception checks are added.

  Revision 1.19  2007/11/15 08:38:52  ikoblik
  unConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.18  2007/11/14 16:39:31  ikoblik
  unConfigGenericFunctions code review comments implementation

  Revision 1.17  2007/10/30 16:20:52  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	mapping CPC_AnalogStatusConfig_getConfig()
	CPC_AnalogStatusConfig_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
	CPC_AnalogStatusConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
*/

// Constants
const string UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME = "CPC_AnalogStatus";
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_LENGTH				= 9;

const unsigned UN_CONFIG_CPC_ANALOGSTATUS_UNIT					= 1;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_FORMAT				= 2;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_DEADBAND				= 3;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_DEADBAND_TYPE			= 4;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_RANGEMAX				= 5;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_RANGEMIN				= 6;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ARCHIVE_ACTIVE		= 7;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ARCHIVE_TIME_FILTER	= 8;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDRESS_POSST			= 9;

const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_LENGTH          = 5;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_MASTER_NAME     = 1;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_PARENTS         = 2;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_CHILDREN        = 3;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_TYPE            = 4;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_SECOND_ALIAS    = 5;

mapping CPC_AnalogStatusConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["hasArchive"] 	= true;
	props["hasPvRange"] 	= true;
	props["address"] 		= ".ProcessInput.PosSt";
	props["dataType"] 		= CPC_FLOAT;
	config["PosSt"] 		= props;

	return config;
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatusConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnalogStatusConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_AnalogStatusConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogStatusConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for AnalogStatus object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnalogStatusConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_AnalogStatus_ExportConfig
/**
Purpose: Export AnalogStatus Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_CPC_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	string sObject = UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME;
	int amount = dynlen(dsDpList);
	for(int i = 1; i <= amount; i++) {
		dyn_string dsDpParameters;
		string currentDP = dsDpList[i];
		string sLimitOn, sLimitOff;

		if(!dpExists(currentDP)) continue;

		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
		cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
		cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
		cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
		cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
		cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt", true, dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_AnalogStatus_ExportConfig
/**
Purpose: Export AnalogStatus Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_CPC_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_AnalogStatus_ExportConfig(dsDpList, exceptionInfo);
}


const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_UNIT                 = 1;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_FORMAT               = 2;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_RANGEMAX             = 3;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_RANGEMIN             = 4;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_ARCHIVE_ACTIVE       = 5;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_ARCHIVE_TIME_FILTER  = 6;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_LENGTH               = 6;


void SOFT_FE_CPC_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

void SOFT_FE_CPC_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

void SOFT_FE_CPC_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString(), dsDpParameters);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

