/**@name LIBRARY: cpcDigitalAlarmConfig.ctl

@author: Alexey Merezhin (EN-ICE-SIC)
@date 09.05.2011
@brief This library contains the function used to set DigitalAlarm configuration.  Based on Alarm and DigitalDelayedAlarm.
Modifications History:
    26/10/2011: Alexey
        [UCPC-557] use cpcExportGenericFunctions_getKeyDeviceConfiguration instead of cpcExportGenericFunctions_getSmsMessage

    21/09/2011: Alexey
        [UCPC-555] added alarm level

	01/09/2011: Alexey
		[UCPC-518] removed archive time filter

    24/08/2011: Alexey
        [UCPC-466] import/export delay time parameter

	28/07/2011: Alexey
		[UCPC-436] store sms message

	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params

	2011/06/28 Alexey
		[UCPC-351] fixed ISt bit mapping

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)
		set archive for evStsReg
*/
/*
Internal Functions :
  CPC_DigitalAlarmConfig_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_CPC_DigitalAlarm_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_CPC_DigitalAlarm_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  CPC_DigitalAlarmConfig_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_CPC_DigitalAlarm_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_CPC_DigitalAlarm_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  mapping CPC_DigitalAlarmConfig_getConfig ()
*/

const string UN_CONFIG_CPC_DIGITALALARM_DPT_NAME = "CPC_DigitalAlarm";
const unsigned UN_CONFIG_CPC_DIGITALALARM_LENGTH = 9;

const unsigned UN_CONFIG_CPC_DIGITALALARM_ARCHIVE_ACTIVE		= 1;
const unsigned UN_CONFIG_CPC_DIGITALALARM_SMS_CATEGORIES		= 2;
const unsigned UN_CONFIG_CPC_DIGITALALARM_SMS_MESSAGE			= 3;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ACK_ALARM				= 4;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ALARM_LEVEL			= 5;
const unsigned UN_CONFIG_CPC_DIGITALALARM_NORMAL_POSITION		= 6;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDRESS_STSREG01		= 7;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDRESS_EVSTSREG01	= 8;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDRESS_MANREG01		= 9;

const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDITIONAL_LENGTH          = 7;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDITIONAL_PARAMETERS      = 2;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDITIONAL_MASTER_NAME     = 3;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDITIONAL_PARENTS         = 4;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDITIONAL_CHILDREN        = 5;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDITIONAL_TYPE            = 6;
const unsigned UN_CONFIG_CPC_DIGITALALARM_ADDITIONAL_SECOND_ALIAS    = 7;


mapping CPC_DigitalAlarmConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["address"]		= ".ProcessInput.StsReg01";
	props["dataType"]		= CPC_UINT16;
	config["StsReg01"]		= props;

	mappingClear(props);
	props["hasArchive"]		= true;
	props["address"]		= ".ProcessInput.evStsReg01";
	props["dataType"]		= CPC_INT32;
	config["evStsReg01"]	= props;

	mappingClear(props);
	props["hasAcknowledge"]	= true;
	props["hasArchive"]		= true;
	props["address"]		= ".ProcessInput.ISt";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= CPC_STSREG01_IST;
	props["isAlarm"]		= true;
	config["ISt"]			= props;

	mappingClear(props);
	props["address"]		= ".ProcessOutput.ManReg01";
	props["dataType"]		= CPC_UINT16;
	config["ManReg01"]		= props;

	return config;
}

dyn_string CPC_DigitalAlarmConfig_getParamNames() {
	return makeDynString(CPC_PARAMS_ALDT);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarmConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_DigitalAlarmConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_DIGITALALARM_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_DIGITALALARM_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_DigitalAlarmConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_DigitalAlarm_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_DIGITALALARM_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_DigitalAlarm_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_DIGITALALARM_DPT_NAME, configLine, exceptionInfo);
}

CPC_DigitalAlarmConfig_checkCustomConfig(dyn_string configLine, bool bHasArchive, dyn_string &exceptionInfo)
{
    int alarmLevel;
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_DIGITALALARM_ALARM_LEVEL], alarmLevel, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) return;
    if (alarmLevel < 0 || alarmLevel > 3) {
        fwException_raise(exceptionInfo, "ERROR", "CPC_DigitalAlarmConfig_checkCustomConfig: wrong alarm level: '" + configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_DIGITALALARM_ALARM_LEVEL] + "'", "");
        return;
    }
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarmConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for DigitalAlarm object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_DigitalAlarmConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_DIGITALALARM_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_DIGITALALARM_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_DIGITALALARM_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_DigitalAlarm_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_DigitalAlarm_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

CPC_DigitalAlarmConfig_setCustomConfig(dyn_string configLine, bool bHasArchive, dyn_string &exceptionInfo) {
    unGenericDpFunctions_setKeyDeviceConfiguration(configLine[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_CONFIG_ALARM_LEVEL, makeDynString(configLine[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_DIGITALALARM_ALARM_LEVEL]), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_DigitalAlarm_ExportConfig
/**
Purpose: Export DigitalAlarm Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_CPC_DigitalAlarm_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_DIGITALALARM_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters, ".ProcessInput.ISt");
        dynRemove(dsDpParameters, dynlen(dsDpParameters));
        cpcExportGenericFunctions_getCategories(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getKeyDeviceConfiguration(currentDP, dsDpParameters, CPC_CONFIG_SMS_MESSAGE, exceptionInfo);
        cpcExportGenericFunctions_getAcknowledgeAlarm(currentDP, ".ProcessInput.ISt", dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getKeyDeviceConfiguration(currentDP, dsDpParameters, CPC_CONFIG_ALARM_LEVEL, exceptionInfo);
        cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.ISt"), dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.ISt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_DigitalAlarm_ExportConfig
/**
Purpose: Export DigitalAlarm Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_CPC_DigitalAlarm_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_DigitalAlarm_ExportConfig(dsDpList, exceptionInfo);
}
