/**@name LIBRARY: unDigitalParameter.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Teodora Miteva (IT-CO-FE)
@date 01.11.2004
@brief This library contains DigitalParameter object functions.
Modifications History:
	$Log: unDigitalParameter.ctl,v $
    29/02/2012: Alexey
        [UCPC-684] "Set Value" available for "operator"

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    20/10/2011: Alexey
        use CPC_STSREG01_ARMRCPST instead of UN_STSREG01_MMOST for recipe bit

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate

	25/05/2011: Alexey
		[UCPC-279] display event mask in the faceplate

	21/05/2011: Alexey
		[UCPC-271] display on widget and control event mask

	18/05/2011: Alexey
		use CPC functions to manage menu

	26/04/2011: amerezhi
		- [UCPC-191] changes code structure according new model; updated for unCPC6

	Revision 1.25  2009/04/02 12:08:04  gthomas
	in _ProvideAccessToButtons(): removed exceptionInfo as returns errors in log when monitor or no user are logged in
	
	Revision 1.24  2009/03/31 15:02:54  gthomas
	in _WidgetDisconnection(): removed AlertText animation
	in _FaceplateStatusAnimationCB(): removed animation check of PLC disconnection for Ranges and the Default value (not required as no PLC connection)

	Revision 1.23  2008/11/12 08:26:03  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.22  2008/10/15 14:44:09  gthomas
	-Removed the <if statement> of the dpConnect in unAnalogParameter_ButtonRegisterCB function
	to fix problem with select/deselect button animation.

	Revision 1.21  2008/07/30 14:20:29  gamx
	Changes in "WidgetDisconnection": objects "ControlStateText" and "eventState" have been removed as they are not presented on the widget panel

	Revision 1.20  2008/07/23 10:02:32  gthomas
	-removed commented code (for error handling)

	Revision 1.19  2008/06/19 12:09:26  gthomas
	integrated new Access Control of UNICOS 4-1 (by instance)
	removed functions _ButtonUserAccessUn and _ButtonUserAccessGcs

	Revision 1.18  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.17  2008/06/13 12:29:47  aabadz
	Modifications asked by UNICOS for their version 4.X

	Revision 1.16  2008/05/26 12:18:19  gthomas
	Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
	to be used with PVSS 3.6 QT

	Revision 1.15  2008/01/22 12:52:44  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.14  2008/01/10 15:24:07  aabadz
	Changes after code review and before tests

	Revision 1.13  2007/10/30 16:21:48  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	CPC_DigitalParameter_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList)
	CPC_DigitalParameter_HandleMenu (string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer)
	CPC_DigitalParameter_FaceplateStatusRegisterCB (string sSystemStatus, bool bSystemConnected)
	CPC_DigitalParameter_FaceplateConnect (string sDeviceName)
	CPC_DigitalParameter_FaceplateDisconnect (string sDeviceName)
	CPC_DigitalParameter_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid, string sDp10, bool bDefaultVal, string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
	CPC_DigitalParameter_animatePlcDisconnection ()
	CPC_DigitalParameter_ButtonRegisterCB (string sSystemStatus, bool bSystemConnected)
	CPC_DigitalParameter_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid)
	CPC_DigitalParameter_ButtonUserAccess (string sDpName, string sDpType, dyn_string &dsAccess)
	CPC_DigitalParameter_UserLoginGetButtonState (string sDeviceName, string sType, dyn_string &dsReturnData)
	CPC_DigitalParameter_ButtonSetState (string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
	CPC_DigitalParameter_ButtonDisconnection ()
	CPC_DigitalParameter_WidgetRegisterCB (string sSystemStatus, bool bSystemConnected)
	CPC_DigitalParameter_WidgetConnect (string sDeviceName, string sPlcName, string sSystemName)
	CPC_DigitalParameter_WidgetDisconnect (string sDeviceName, string sPlcName, string sSystemName)
	CPC_DigitalParameter_WidgetAnimationCB(string sDpLock, bool bLocked,string sDpManager, string sSelectedManager, string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid, string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
	CPC_DigitalParameter_WidgetDisconnection (string sWidgetType)
	DigitalParameter_EventList_Configure_Init (dyn_string &exceptionInfo, const bool bUpdate=false)
	DigitalParameter_EventList_Configure_Clic (int iRow, string sColumn, string sValue, dyn_string &exceptionInfo)
	DigitalParameter_EventList_Configure_Save (dyn_string &exceptionInfo)
*/


//------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName Device name
	@param[in]	sDpType Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] dsMenuList Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
CPC_DigitalParameter_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_MASK_POSST_TEXT,
			 UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
}


//------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	deviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	dsMenuList 	List of requested action
	@param[in]	menuAnswer 	Selected action from the dsMenuList
*/
CPC_DigitalParameter_HandleMenu(string deviceName, string sDpType, dyn_string dsMenuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, dsMenuList, menuAnswer, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_DigitalParameter_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName,sSystemName, sPlcName, sPlcDpName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);
		if(bRemote)
			g_bSystemConnected = bSystemConnected;
		else
			g_bSystemConnected = true;

		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)
			g_bUnSystemAlarmPlc = true;
		else
			g_bUnSystemAlarmPlc = false;

		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				CPC_DigitalParameter_animatePlcDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				CPC_DigitalParameter_FaceplateConnect (sDeviceName);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				CPC_DigitalParameter_FaceplateDisconnect (sDeviceName);
				CPC_DigitalParameter_animatePlcDisconnection();
				break;
			case UN_ACTION_NOTHING: break;
			default:
				break;
		}

	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_FaceplateConnect
/**
Do faceplate dpConnect

	@param[in]	sDeviceName	Device DP name
*/
CPC_DigitalParameter_FaceplateConnect(string sDeviceName)
{
	int iRes=0;
	dyn_string exceptionInfo;
	string sPlcName, sPlcDpName, sSystemName;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
// get the PLC name
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpConnect("CPC_DigitalParameter_FaceplateStatusAnimationCB",
			 sDeviceName + ".ProcessInput.StsReg01",
			 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			 sDeviceName + ".ProcessOutput.DfltVal",
			 sPlcDpName + ".alarm",
			 sPlcDpName + ".enabled",
			 sDeviceName + ".eventMask");

		g_bCallbackConnected = (iRes >= 0);
	}
	else
		CPC_DigitalParameter_animatePlcDisconnection();


}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	sDeviceName	Device DP name
*/
CPC_DigitalParameter_FaceplateDisconnect(string sDeviceName)
{
	int iRes=0;
	string sPlcName, sSystemName;

	if (g_bUnSystemAlarmPlc)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

		iRes = dpDisconnect("CPC_DigitalParameter_FaceplateStatusAnimationCB",
							sDeviceName + ".ProcessInput.StsReg01",
							sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
							sDeviceName + ".ProcessOutput.DfltVal",
							sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
							sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
							sDeviceName + ".eventMask");
		g_bCallbackConnected = !(iRes >= 0);
	}
	else
	{
			CPC_DigitalParameter_WidgetDisconnection(g_sWidgetType);
			g_bCallbackConnected = false;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
CPC_DigitalParameter_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
		 string sDp10, bool bDefaultVal,
		 string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
		 string sDpEvt, int iMask)
{
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("PosSt", getBit(bit32StsReg01, UN_STSREG01_POSST), 1, "cpcColor_Faceplate_Status",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("MOnR", getBit(bit32StsReg01, CPC_STSREG01_MONRST), 1, "cpcColor_Faceplate_Status",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("MOffR", getBit(bit32StsReg01, CPC_STSREG01_MOFFRST), 1, "cpcColor_Faceplate_Status",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ArmedRecipe", getBit(bit32StsReg01, CPC_STSREG01_ARMRCPST), 1, "cpcColor_Faceplate_Status",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    cpcGenericObject_CheckboxAnimate("DefaultValue", bDefaultVal);
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_animatePlcDisconnection
/**
Animate faceplate disconnection
*/
CPC_DigitalParameter_animatePlcDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("PosSt", "MOnR", "MOffR", "ArmedRecipe", "eventState"));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
CPC_DigitalParameter_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS)
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	else
	{
		dynAppend(exceptionInfo, "CPC_DigitalParameter_ProvideAccessToButtons: the list of functions is not correct - number of functions is less than expected");
		return;
	}

	if (sFunction != "")
	{
		evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" + sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, sDpType);

		g_dsUserAccess=dsUserAccess;
	}
	else
		fwException_raise(exceptionInfo,"ERROR", "CPC_DigitalParameter_ProvideAccessToButtons:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

}

//---------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_ButtonRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_DigitalParameter_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction;
	int iAction, iRes;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
	{
		case UN_ACTION_DISCONNECT:
			CPC_DigitalParameter_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			// call the CPC_DigitalParameter_ButtonUserAccess to get the list of allowed buttons.
			sDpType= UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME;
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			if (dynlen(exceptionInfo) <= 0)	CPC_DigitalParameter_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
			
			iRes = dpConnect("CPC_DigitalParameter_ButtonAnimationCB",
					sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
					sDeviceName + ".statusInformation.selectedManager",
					sDeviceName + ".ProcessInput.StsReg01",
					sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");
			g_bCallbackConnected = (iRes >= 0);
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_DigitalParameter_ButtonsDisconnect(sDeviceName);
			CPC_DigitalParameter_ButtonDisconnection();

			break;
		case UN_ACTION_NOTHING: break;
		default:
			break;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
//CPC_DigitalParameter_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
CPC_DigitalParameter_ButtonsDisconnect(string sDeviceName)
{
	int iRes = -1;

	iRes = dpDisconnect("CPC_DigitalParameter_ButtonAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.StsReg01",
			sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_ButtonAnimationCB
/**
Animate buttons

@note This function works with an unicos button panel and needs global variables declared in this panel -
		"g_dsUserAccess" (dyn_string). Button identifier is added to this variable if user has access.
*/
CPC_DigitalParameter_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
									string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid)
{
	CPC_DigitalParameter_ButtonSetState(unGenericDpFunctions_getDpName(sStsReg01), UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, g_dsUserAccess,
				makeDynString(bLocked, sSelectedManager, bit32StsReg01, bStsReg01Invalid, 0, 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_ButtonUserAccess
/**
User access for buttons

	@param[in] 		sDpName	Device name
	@param[in] 		sDpType	dpType
	@param[out] 	dsAccess	List of allowed buttons
	
*/
CPC_DigitalParameter_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
		bool operator, expert, admin;
		dyn_bool dbPermissions;
		dyn_string exceptionInfo;
		string sOperAction, sExpAction, sAdminAction;
		bool bActionDefined = false;
		
		unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
		if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
		(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
			bActionDefined=true;
			operator = dbPermissions[2];
			expert = dbPermissions[3];
			admin = dbPermissions[4];
			dsAccess = makeDynString();
		if (operator){
			if(bActionDefined)
				dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
		    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, CPC_FACEPLATE_BUTTON_DIGITALPARAMETER_SET_VALUE));
		}
		if (expert){
			if(bActionDefined)
		            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		        else
			    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_MASKEVENT));
		}
		 if(admin){
		       	if(bActionDefined)
		           dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	sDeviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
CPC_DigitalParameter_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	int iRes = -1;
	string sDpLock, sDpSelectedManager, sSelectedManager;
	bool bLocked;

	sDpLock = sDeviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = sDeviceName + ".statusInformation.selectedManager";
	if (dpExists(sDeviceName))
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);

	if (iRes < 0) // Errors during dpget
	{
		sDpLock = "";
		bLocked = false;
		sDpSelectedManager = "";
		sSelectedManager = "";
	}
	dsReturnData=makeDynString(bLocked, sSelectedManager, 0, 0, 0, 0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
CPC_DigitalParameter_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled;
	dyn_string dsButtons;
	int i, iDsButtonsLength;
	string localManager;
	bool bLocked, bStsReg01Bad;
	string sSelectedManager;
	bit32 stsReg01Value;

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];

	dsButtons = makeDynString(CPC_FACEPLATE_BUTTON_DIGITALPARAMETER_SET_VALUE);// Each button except "Select" button
	iDsButtonsLength = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i = 1; i <= iDsButtonsLength; i++)
	{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled && bSelected);
	}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameter_ButtonDisconnection
/**
Animate disconnection
*/
CPC_DigitalParameter_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, CPC_FACEPLATE_BUTTON_DIGITALPARAMETER_SET_VALUE));
}


// CPC_DigitalParameter_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: XP.
	. distributed system: yes.
*/

CPC_DigitalParameter_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.StsReg01", value, sDeviceName+".ProcessInput.StsReg01:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.StsReg01"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.StsReg01");

}
//----------------------------------------------------------------------------------------------
//CPC_DigitalParameter_WidgetRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_DigitalParameter_WidgetRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sSystemName, sPlcName, sPlcDpName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getWidgetDpName ($sIdentifier);
	if (sDeviceName == "") //in case of disconnection
	{
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	//get the Plc name
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)
		g_bUnSystemAlarmPlc = true;
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
	{
		case UN_ACTION_DISCONNECT:
			CPC_DigitalParameter_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
			if(dpTypeName(sDeviceName) == UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME)
				CPC_DigitalParameter_WidgetConnect(sDeviceName, sPlcName, sSystemName);
			else{
				g_bUnSystemAlarmPlc = false;
				CPC_DigitalParameter_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_DigitalParameter_WidgetDisconnect(sDeviceName, sPlcName, sSystemName);
			CPC_DigitalParameter_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING: break;
		default:
			break;
	}
}


//----------------------------------------------------------------------------------------------------------------
//CPC_DigitalParameter_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
CPC_DigitalParameter_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes=0;

	if(g_bUnSystemAlarmPlc) {
		iRes = dpConnect ("CPC_DigitalParameter_WidgetAnimationCB",
						  sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
						  sDeviceName + ".statusInformation.selectedManager",
						  sDeviceName + ".ProcessInput.StsReg01",
						  sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
						  sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
						  sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
						  sDeviceName + ".eventMask");
		g_bCallbackConnected = (iRes >= 0);
	} else {
		CPC_DigitalParameter_WidgetDisconnection(g_sWidgetType);
	}
}


//--------------------------------------------------------------------------------------------------------------------------
//CPC_DigitalParameter_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
CPC_DigitalParameter_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes;
	if(g_bUnSystemAlarmPlc) {
		iRes = dpDisconnect("CPC_DigitalParameter_WidgetAnimationCB",
							sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
							sDeviceName + ".statusInformation.selectedManager",
							sDeviceName + ".ProcessInput.StsReg01",
							sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
							sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
							sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
							sDeviceName + ".eventMask");
		g_bCallbackConnected = !(iRes >= 0);
	}
}


//------------------------------------------------------------------------------------------------------------------------------
//CPC_DigitalParameter_WidgetAnimationCB
/**
Animate widget
*/
CPC_DigitalParameter_WidgetAnimationCB(string sDpLock, bool bLocked,string sDpManager, string sSelectedManager,
		string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
		string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sEvtMask, int iMask)
{
	string sbody1Color, sbody2Color, ColorSelect;
	string sWarningLetter, sWarningColor;
	bool bLockVisible;

	if((!bSystemIntegrityAlarmEnabled)||(iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}

	if (bStsReg01Invalid) {
		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
		setMultiValue("Body1","foreCol","unDataNotValid","Body1","backCol","",
					  "Body2","foreCol","unDataNotValid","Body2","backCol","");
	} else {
		sbody1Color = "cpcColor_Widget_Status";
		sbody2Color = "cpcColor_Widget_Request";
		setMultiValue("Body1", "foreCol", sbody1Color,
					  "Body2", "foreCol", sbody2Color);
		if (getBit(bit32StsReg01, UN_STSREG01_POSST) == 1) {
			setValue("Body1", "backCol", sbody1Color);
			setValue("Body1", "fill", "[solid]");
		} else {
			setValue("Body1", "backCol", "");
		}
		if (getBit(bit32StsReg01, CPC_STSREG01_MONRST) == 1) {
			setValue("Body2", "backCol", sbody2Color);
			setValue("Body2", "fill", "[solid]");
		} else {
			setValue("Body2", "backCol", "");
		}
	}

	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, ColorSelect, bLockVisible);
	if(g_bSystemConnected) {
		setMultiValue("WarningText","text",sWarningLetter,
                 "WarningText","foreCol",sWarningColor,
                 "SelectArea", "foreCol", ColorSelect, 
                 "LockBmp", "visible", bLockVisible,
                 "WidgetArea", "visible", true);
	}
	if(g_bSystemConnected) {
		eventState.visible = iMask == 0;
	}	
}


//----------------------------------------------------------------------------------------------------------------------------
//CPC_DigitalParameter_WidgetDisconnection
/**
Animate widget disconnection
	@param[in] sWidgetType Widget type. ex: "heater"
*/
CPC_DigitalParameter_WidgetDisconnection(string sWidgetType)
{
	setMultiValue("Body1","foreCol","unDataNoAccess",
				  "Body1","backCol","unWidget_Background",
				  "Body1","fill","[solid]",
				  "Body2","foreCol","unDataNoAccess",
				  "Body2","backCol","unWidget_Background",
				  "Body2","fill","[solid]",
				  "WarningText", "text", "", 
				  "SelectArea", "foreCol", "", 
				  "LockBmp", "visible", false,
				  "WidgetArea", "visible", false);
	eventState.visible = false;
}
