/**@name LIBRARY: cpcDigitalParameterConfig.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 24.07.2003
@brief This library contains the function used to set DigitalParameter configuration.
Modifications History:
  $Log: CPC_DigitalParameterConfig.ctl,v $
    23/03/2012: Alexey
        [UCPC-253] add SOFT_FE

	21/03/2012: Alexey
		configure PosSt as stubbed

	01/09/2011: Alexey
		[UCPC-518] removed archive time filter

	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	2011/05/10 amerezhi
		[UCPC-191] added PosSt dpe

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)

	26/04/2011: amerezhi
		- [UCPC-191] changes code structure according new model; updated for unCPC6

  2011/02/08 amerezhi
  Check and set device using UNCCPCConfig' general functions

  Revision 1.29  2010/10/08 14:59:04  amerezhi
  Setting UN_CONFIG_CPC_DIGITALPARAMETER_DEFAULT_VALUE is fixed

  Revision 1.28  2009/04/14 13:35:04  gthomas
  commented Debug Statements

  Revision 1.27  2009/03/10 15:50:00  gthomas
  added UNICOS Changes made on 17/02/2009
  modifications in CPC_DigitalParameterConfig_setCommonConfig function: added case to deal with strings "True" or "true"

  Revision 1.26  2008/07/28 12:49:35  gamx
  exception handling has been restored

  Revision 1.25  2008/06/19 12:23:17  gthomas
  Moved back variable in global lib as it used in faceplate pnl and library

  Revision 1.24  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.23  2008/06/09 14:54:26  aabadz
  Result after move some constants, only use use inside this file, from gcsObjects_declarations.

  Revision 1.22  2008/01/22 12:52:44  aabadz
  Fixed:
  - Naming
  - Call to other functions to reuse code
  - Some error check

  Revision 1.21  2007/11/15 08:38:52  ikoblik
  unConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.20  2007/11/14 16:39:31  ikoblik
  unConfigGenericFunctions code review comments implementation

  Revision 1.19  2007/10/30 16:21:05  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
  CPC_DigitalParameterConfig_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_CPC_DigitalParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_CPC_DigitalParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  CPC_DigitalParameterConfig_setCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  CPC_DigitalParameterConfig_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_CPC_DigitalParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_CPC_DigitalParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  CPC_DigitalParameterConfig_checkCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  CPC_DigitalParameterConfig_setBoolBlock (string plcType, dyn_string dsConfigs, int dataType, dyn_string dsInputDps, dyn_int diInputAddresses, dyn_string dsOutputDps, dyn_int diOutputAddresses, dyn_int diCommonData, dyn_string &exceptionInfo)
*/

// Constants
const string UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME = "CPC_DigitalParameter";
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_LENGTH = 5;

const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ARCHIVE_ACTIVE 		= 1;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDRESS_STSREG01 		= 2;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDRESS_EVSTSREG01	= 3;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDRESS_MANREG01		= 4;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_DEFAULT_VALUE 		= 5;

const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDITIONAL_LENGTH          = 6;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDITIONAL_MASTER_NAME     = 2;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDITIONAL_PARENTS         = 3;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDITIONAL_CHILDREN        = 4;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDITIONAL_TYPE            = 5;
const unsigned UN_CONFIG_CPC_DIGITALPARAMETER_ADDITIONAL_SECOND_ALIAS    = 6;

mapping CPC_DigitalParameterConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg01";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg01"] 		= props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg01";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg01"] 	= props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["isAlarm"]		= true;
	props["stubAlarm"]		= true;
	props["address"] 		= ".ProcessInput.PosSt";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= UN_STSREG01_POSST;
	config["PosSt"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.ManReg01";
	props["dataType"] 		= CPC_UINT16;
	config["ManReg01"] 		= props;

	return config;
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameterConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_DigitalParameterConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_DigitalParameterConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_DigitalParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_DigitalParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, configLine, exceptionInfo);
}

CPC_DigitalParameterConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
	bool bValue;
	cpcConfigGenericFunctions_checkBool("DEFAULT_VALUE", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_DIGITALPARAMETER_DEFAULT_VALUE], bValue, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalParameterConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for DigitalParameter object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_DigitalParameterConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_DigitalParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_DigitalParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

CPC_DigitalParameterConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
	bool bDefaultValue;
	string sDefaultValueConfigured = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_DIGITALPARAMETER_DEFAULT_VALUE];
	cpcConfigGenericFunctions_checkBool("DEFAULT_VALUE", sDefaultValueConfigured, bDefaultValue, exceptionInfo);
	dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DfltVal", bDefaultValue);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_DigitalParameter_ExportConfig
/**
Purpose: Export DigitalParameter Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_CPC_DigitalParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        dynRemove(dsDpParameters, dynlen(dsDpParameters));
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01",	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
        cpcExportGenericFunctions_getBoolDefaultValue(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_DigitalParameter_ExportConfig
/**
Purpose: Export DigitalParameter Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_CPC_DigitalParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_DigitalParameter_ExportConfig(dsDpList, exceptionInfo);
}


const unsigned UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_ARCHIVE_ACTIVE       = 1;
const unsigned UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_DEFAULT_VALUE        = 2;
const unsigned UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_AUTONOMOUS           = 3;
const unsigned UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_LENGTH               = 3;

const unsigned UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_ADDITIONAL_LENGTH    = 1;
const unsigned UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_ADDITIONAL_MASKEVENT = 1;

void SOFT_FE_CPC_DigitalParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME, configLine, exceptionInfo);
}

CPC_SOFT_FE_DigitalParameterConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
    bool val;
    cpcConfigGenericFunctions_checkBool("AUTONOMOUS", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_AUTONOMOUS], val, exceptionInfo);
    if (dynlen(exceptionInfo) > 0) return;
    cpcConfigGenericFunctions_checkBool("DEFAULT_VALUE", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_DEFAULT_VALUE], val, exceptionInfo);
}

void SOFT_FE_CPC_DigitalParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

CPC_SOFT_FE_DigitalParameterConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
  bool bDefaultValue;
  cpcConfigGenericFunctions_checkBool("DEFAULT_VALUE", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_DEFAULT_VALUE], bDefaultValue, exceptionInfo);
  dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DfltVal", bDefaultValue);

  bool autonomous;
  cpcConfigGenericFunctions_checkBool("AUTONOMOUS", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_CPC_DIGITALPARAMETER_AUTONOMOUS], autonomous, exceptionInfo);
  if (autonomous) {
    string sDp = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.StsReg01";
    unGenericDpFunctions_createDpeDpFunction(sDp);
    dyn_string params = makeDynString(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.ManReg01:_original.._value");
    dpSetWait(sDp + ":_dp_fct.._param", params, sDp + ":_dp_fct.._fct", "cpcDigitalParameterDPFunc_getStsReg01Value(\"" + sDp + "\", p1)");

    sDp = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.PosSt";
    unGenericDpFunctions_createDpeDpFunction(sDp);
    params = makeDynString(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.StsReg01:_original.._value");
    dpSetWait(sDp + ":_dp_fct.._param", params, sDp + ":_dp_fct.._fct", "p1");
  }
}

void SOFT_FE_CPC_DigitalParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_DIGITALPARAMETER_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        dynRemove(dsDpParameters, dynlen(dsDpParameters));
        cpcExportGenericFunctions_getBoolDefaultValue(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getAutonomous(currentDP, dsDpParameters, ".ProcessInput.StsReg01");
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

