/**@name LIBRARY: cpcLocalConfig.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
    26/10/2011: Alexey
        [UCPC-557] use cpcExportGenericFunctions_getKeyDeviceConfiguration instead of cpcExportGenericFunctions_getSmsMessage

	01/09/2011: Alexey
		[UCPC-518] removed archive time filter

	28/07/2011: Alexey
		[UCPC-436] store sms message

	08/07/2011: Alexey
		[UCPC-389] moved "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)
		set archive for evStsReg

	06/05/2011: amerezhi
		fixed PosAl bit according Object_ManReg_StsReg_Mapping.xls

	21/04/2011: amerezhi
		added alarm acknowledge flag

	11/04/2011: amerezhi
		- fixed UN_CONFIG_CPC_LOCAL_SMS_CATEGORY => UN_CONFIG_CPC_LOCAL_SMS_CATEGORIES

	07/04/2011 amerezhi
		[UCPC-209] updated according unCPC6 specs

	18/04/2006: Herve
		- implementation of mask event

	14/06/2005: Enrique
		- S7 adaptation
			. S7_PLC_CPC_Local_checkConfig
			. S7_PLC_CPC_Local_setConfig
				
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_CPC_Local_checkConfig, _UnPlc_CPC_Local_setConfig

version 1.0

External Function : 
	. CPC_LocalConfig_setConfig :  set configuration of given Local datapoint
	. CPC_LocalConfig_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Local configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const string UN_CONFIG_CPC_LOCAL_DPT_NAME = "CPC_Local";
const unsigned UN_CONFIG_CPC_LOCAL_LENGTH = 8;

const unsigned UN_CONFIG_CPC_LOCAL_ARCHIVE_ACTIVE		= 1;
const unsigned UN_CONFIG_CPC_LOCAL_SMS_CATEGORIES		= 2;
const unsigned UN_CONFIG_CPC_LOCAL_SMS_MESSAGE			= 3;
const unsigned UN_CONFIG_CPC_LOCAL_ACK_ALARM			= 4;
const unsigned UN_CONFIG_CPC_LOCAL_NORMAL_POSITION		= 5;
const unsigned UN_CONFIG_CPC_LOCAL_ADDRESS_STSREG01		= 6;
const unsigned UN_CONFIG_CPC_LOCAL_ADDRESS_EVSTSREG01	= 7;
const unsigned UN_CONFIG_CPC_LOCAL_ADDRESS_MANREG01		= 8;

const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_LENGTH          = 7;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_PARAMETERS      = 2;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_MASTER_NAME     = 3;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_PARENTS         = 4;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_CHILDREN        = 5;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_TYPE            = 6;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_SECOND_ALIAS    = 7;


mapping CPC_LocalConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg01";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg01"] 		= props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg01";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg01"] 	= props;

	mappingClear(props);
	props["hasAcknowledge"] = true;
	props["hasArchive"] 	= true;
	props["isAlarm"]		= true;
	props["address"] 		= ".ProcessInput.PosAl";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= CPC_STSREG01_POSALST;
	config["PosAl"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.ManReg01";
	props["dataType"] 		= CPC_UINT16;
	config["ManReg01"] 		= props;

	return config;
}

dyn_string CPC_LocalConfig_getParamNames() {
	return makeDynString(CPC_PARAMS_HFON, CPC_PARAMS_HFOFF);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_LocalConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_LocalConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_LocalConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_Local_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_LOCAL_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_Local_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_LOCAL_DPT_NAME, configLine, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_LocalConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for Local object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_LocalConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;

	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_Local_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_Local_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_Local_ExportConfig
/**
Purpose: Export Local Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_Local_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_LOCAL_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters, ".ProcessInput.PosAl");
        dynRemove(dsDpParameters, dynlen(dsDpParameters));
        cpcExportGenericFunctions_getCategories(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getKeyDeviceConfiguration(currentDP, dsDpParameters, CPC_CONFIG_SMS_MESSAGE, exceptionInfo);
        cpcExportGenericFunctions_getAcknowledgeAlarm(currentDP, ".ProcessInput.PosAl", dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.PosAl"), dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.PosAl"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_Local_ExportConfig
/**
Purpose: Export Local Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_Local_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_Local_ExportConfig(dsDpList, exceptionInfo);
}
