/**@name LIBRARY: cpcMassFlowController.ctl
@author Alexey Merezhin EN/ICE/PLC
@date 11.01.2011
@brief This library contains MassFlowController object functions.
Modifications History:
    13/09/2012: Alexey
        [UCPC-931] add alarms item to the right click menu

    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    14/05/2012: Alexey
        [UCPC-803][UCPC-824] use common ConfigW bit and animate it in the widget

    29/02/2012: Alexey
        animate unit in the widget
        [UCPC-684] "allow restart" available for "operator"

    16/02/2012: Alexey
        [UCPC-673] animate inhibit totalizer

    20/01/2012: Alexey
        [UCPC-599] ported from CPC5 gcsMassFlowController

*/
/*
Internal Functions :
	CPC_MassFlowController_ObjectListGetValueTime (string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
	CPC_MassFlowController_AcknowledgeAlarm (string sDpName, string sDpType, dyn_string &dsNeedAck)
	CPC_MassFlowController_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
	CPC_MassFlowController_HandleMenu (string sDeviceName, string sDpType, dyn_string menuList, int menuAnswer)
	CPC_MassFlowController_FaceplateStatusRegisterCB (string sDp, bool bSystemConnected)
	CPC_MassFlowController_FaceplateConnect (string sDeviceName)
	CPC_MassFlowController_FaceplateDisconnect (string sDeviceName)
	CPC_MassFlowController_FaceplateStatusAnimationCB (string sStsReg01, bit32 bit32StsReg01, string sDp2, ... )
	CPC_MassFlowController_AnimatePlcDisconnection()
	CPC_MassFlowController_ButtonRegisterCB (string sDp, bool bSystemConnected)
	CPC_MassFlowController_ButtonAnimationCB (string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
	CPC_MassFlowController_ButtonUserAccess (string sDpName, string sDpType, dyn_string &dsAccess)
	CPC_MassFlowController_UserLoginGetButtonState (string sDeviceName, string sType, dyn_string &dsReturnData)
	CPC_MassFlowController_ButtonSetState (string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
	CPC_MassFlowController_ButtonDisconnection ()
	CPC_MassFlowController_WidgetRegisterCB (string sDp, bool bSystemConnected)
	CPC_MassFlowController_WidgetConnect (string sDeviceName, string sPlcName, string sSystemName)
	CPC_MassFlowController_WidgetDisconnect (string sDeviceName, string sPlcName, string sSystemName)
	CPC_MassFlowController_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, ... )
	CPC_MassFlowController_WidgetDisconnection (string sWidgetType)
	CPC_MassFlowController_EventList_Configure_Init (dyn_string &exceptionInfo, const bool bUpdate=false)
	CPC_MassFlowController_EventList_Configure_Clic (int iRow, string sColumn, string sValue, dyn_string &exceptionInfo)
	CPC_MassFlowController_EventList_Configure_Save (dyn_string &exceptionInfo)
	CPC_MassFlowController_CheckParametersAccess (string sUser, string sDeviceName, bool &granted)
*/


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ObjectListGetValueTime
/**
Function called from the objectList to return the time and value

	@param[in]	sDeviceName		Device name
	@param[in]	sDeviceType		Unicos object
	@param[out]	dsReturnData	Return value
*/

CPC_MassFlowController_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
    float value;
    string sTime;

    dpGet(sDeviceName+".ProcessInput.PosSt", value,
          sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);

    dsReturnData[1] = sTime;
    dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " "
                            + dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_AcknowledgeAlarm
/**
Acknowledge AnalogDelayedAlarm alarms
	The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string

	@param[in]	 sDpName	Device name
	@param[in]	 sDpType	Unicos object
	@param[out]	 dsNeedAck	DPEs names that need acknowledgement.
*/
CPC_MassFlowController_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
    dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.TStopISt", "ProcessInput.FuStopISt", "ProcessInput.AlSt", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName Device name
	@param[in]	sDpType Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] menuList Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
CPC_MassFlowController_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT);

    cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	sDeviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	menuList 	List of requested action
	@param[in]	menuAnswer 	Selected action from the menuList
*/
CPC_MassFlowController_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer)
{
    dyn_string exceptionInfo;

    cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);

    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click",exceptionInfo);
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemStatus 		Parameters of the distributed system
	@param[in] 	bSystemConnected 	Flag
*/
CPC_MassFlowController_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
    string sDeviceName, sSystemName, sPlcName;
    int iAction;
    dyn_string exceptionInfo;
    bool bRemote;

    sDeviceName = unGenericDpFunctions_getDpName($sDpName);
    sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

    unDistributedControl_isRemote(bRemote, sSystemName);
    if (bRemote) g_bSystemConnected = bSystemConnected;
    else g_bSystemConnected = true;

    sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
    g_bUnSystemAlarmPlc = sPlcName != "" && dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName);

    unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

    switch (iAction) {
        case UN_ACTION_DISCONNECT:
            CPC_MassFlowController_AnimatePlcDisconnection();
            break;
        case UN_ACTION_DPCONNECT:
            CPC_MassFlowController_FaceplateConnect(sDeviceName);
            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:
            CPC_MassFlowController_FaceplateDisconnect(sDeviceName);
            CPC_MassFlowController_AnimatePlcDisconnection();
            break;
        case UN_ACTION_NOTHING:
        default:
            break;
    }
}

dyn_string CPC_MassFlowController_FaceplateConnectionDPEs(string deviceName, string systemName, string plcName) {
    dyn_string dpes = makeDynString(systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+plcName+".alarm",
                                    systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+plcName+".enabled",
                                    deviceName + ".eventMask",
                                    deviceName + ".ProcessInput.StsReg01",
                                    deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                    deviceName + ".ProcessInput.StsReg02",
                                    deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                                    deviceName + ".ProcessInput.PosSt",
                                    deviceName + ".ProcessInput.PosSt:_online.._invalid",
                                    deviceName + ".ProcessInput.VoTSt",
                                    deviceName + ".ProcessInput.VoTSt:_online.._invalid",
                                    deviceName + ".ProcessInput.AuVoTOfSt",
                                    deviceName + ".ProcessInput.AuVoTOfSt:_online.._invalid",
                                    deviceName + ".ProcessInput.MaxPosSt",
                                    deviceName + ".ProcessInput.MaxPosSt:_online.._invalid",
                                    deviceName + ".ProcessInput.MDMoRSt",
                                    deviceName + ".ProcessInput.MDMoRSt:_online.._invalid",
                                    deviceName + ".ProcessInput.AuDMoRSt",
                                    deviceName + ".ProcessInput.AuDMoRSt:_online.._invalid",
                                    deviceName + ".ProcessInput.ActDMo",
                                    deviceName + ".ProcessInput.ActDMo:_online.._invalid",
                                    deviceName + ".ProcessInput.MVoTMRSt",
                                    deviceName + ".ProcessInput.MVoTMRSt:_online.._invalid",
                                    deviceName + ".ProcessInput.AuVoTMRSt",
                                    deviceName + ".ProcessInput.AuVoTMRSt:_online.._invalid",
                                    deviceName + ".ProcessInput.ActVoTMo",
                                    deviceName + ".ProcessInput.ActVoTMo:_online.._invalid",
                                    deviceName + ".ProcessInput.MCCRSt",
                                    deviceName + ".ProcessInput.MCCRSt:_online.._invalid",
                                    deviceName + ".ProcessInput.AuCCRSt",
                                    deviceName + ".ProcessInput.AuCCRSt:_online.._invalid",
                                    deviceName + ".ProcessInput.ActCC",
                                    deviceName + ".ProcessInput.ActCC:_online.._invalid",
                                    deviceName + ".ProcessInput.MPosRSt",
                                    deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
                                    deviceName + ".ProcessInput.AuPosRSt",
                                    deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
                                    deviceName + ".ProcessInput.ActOutOV",
                                    deviceName + ".ProcessInput.ActOutOV:_online.._invalid");

                                    
                                    
    if (g_bBoolAlert) {
        dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");
    }
    return dpes;
}
//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_FaceplateConnect
/**
Connects callback functions to animate a faceplate.
As it is not possible to pass constant values in callback functions and to get them inside callback function,
global constants are used.
	@param[in]	sDeviceName	Device DP name
*/
CPC_MassFlowController_FaceplateConnect(string sDeviceName)
{
    int iRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4;
    int iAlertRes;
    dyn_string exceptionInfo, lastError;
    string sPlcName, sSystemName;

    sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
    g_bCallbackConnected = false;

    sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
    if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
    else g_bUnSystemAlarmPlc = false;

    iAlertRes = dpGet(sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                      sDeviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                      sDeviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                      sDeviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    if (iAlertRes >= 0) {
        g_bBoolAlert = ((iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL));
    } else {
        dynAppend(exceptionInfo, "CPC_MassFlowController_FaceplateConnect: problems to get alert handling data, object:" + sDeviceName);
    }

    if (g_bUnSystemAlarmPlc) {
        g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PARAMS_MRESTART, exceptionInfo);
        g_bFSPosOn = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PARAMS_FSPOSON, exceptionInfo);
        g_bNoiseFilter = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PARAMS_NOISE_FILTER, exceptionInfo);
        g_bFeedbackSimulator = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PARAMS_FEEDBACK_SIM, exceptionInfo);
        g_bInhibitManualTotalizer = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PARAMS_INHIBITMANUALTOTALIZER, exceptionInfo);
        g_mFormats["PosSt"] = dpGetFormat(sDeviceName + ".ProcessInput.PosSt");
        g_mUnits["PosSt"] = dpGetUnit(sDeviceName + ".ProcessInput.PosSt");
        g_mFormats["VoTSt"] = dpGetFormat(sDeviceName + ".ProcessInput.VoTSt");
        g_mUnits["VoTSt"] = dpGetUnit(sDeviceName + ".ProcessInput.VoTSt");

        string flowUnit, gasName;
        unGenericDpFunctions_getKeyDeviceConfiguration(sDeviceName, CPC_MFC_CALIBRATION_UNIT, flowUnit, exceptionInfo);
        unGenericDpFunctions_getKeyDeviceConfiguration(sDeviceName, CPC_MFC_CALIBRATION_GAS_NAME, gasName, exceptionInfo);
        g_sCalibration = gasName + " - " + flowUnit;

        cpcGenericObject_StringToMapping(dpGetDescription(sDeviceName + ".ProcessInput.ActCC"), g_mCCLabels);
        cpcGenericObject_StringToMapping(dpGetDescription(sDeviceName + ".ProcessInput.ActVoTMo"), g_mVoTLabels);
        cpcGenericObject_StringToMapping(dpGetDescription(sDeviceName + ".ProcessInput.ActDMo"), g_mDMLabels);

        dyn_string dpes = CPC_MassFlowController_FaceplateConnectionDPEs(sDeviceName, sSystemName, sPlcName);
        iRes = dpConnect("CPC_MassFlowController_FaceplateStatusAnimationCB", dpes);
        lastError = getLastError();
        
        g_bCallbackConnected = (iRes >= 0) && dynlen(lastError) == 0;
        if (dynlen(lastError) > 0) {
            DebugN(lastError);
            fwException_raise(exceptionInfo,"ERROR", "CPC_AnalogInput_FaceplateConnect: dpConnect error: " + lastError, "EMPTY");
        }
    } else {
        CPC_MassFlowController_AnimatePlcDisconnection();
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	deviceName	Device DP name
*/
CPC_MassFlowController_FaceplateDisconnect(string deviceName)
{
    int iRes;
    string plcName, systemName;

    g_bCallbackConnected = false;

    if (g_bUnSystemAlarmPlc) {
        systemName = unGenericDpFunctions_getSystemName(deviceName);
        plcName = unGenericObject_GetPLCNameFromDpName(deviceName);
        dyn_string dpes = CPC_MassFlowController_FaceplateConnectionDPEs(deviceName, systemName, plcName);
        iRes = dpDisconnect("CPC_MassFlowController_FaceplateStatusAnimationCB", dpes);

        g_bCallbackConnected = !(iRes >= 0);
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
CPC_MassFlowController_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values)
{
    cpcFaceplate_animateOnlineValue(dpes, values, "PosSt", g_mUnits["PosSt"], g_mFormats["PosSt"], "cpcColor_Faceplate_Status");
    cpcFaceplate_animateOnlineValue(dpes, values, "VoTSt", g_mUnits["VoTSt"], g_mFormats["VoTSt"], "cpcColor_Faceplate_Status");
    cpcFaceplate_animateOnlineValue(dpes, values, "AuVoTOfSt", g_mUnits["VoTSt"], g_mFormats["VoTSt"], "cpcColor_Faceplate_Status");
    cpcFaceplate_animateOnlineValue(dpes, values, "MaxPosSt", g_mUnits["PosSt"], g_mFormats["PosSt"], "cpcColor_Faceplate_Status");
    cpcGenericObject_CheckboxAnimate("PFSPosOn", g_bFSPosOn);
    cpcGenericObject_CheckboxAnimate("PNoiseFilter", g_bNoiseFilter);
    cpcGenericObject_CheckboxAnimate("PFeedbackSim", g_bFeedbackSimulator);

    cpcFaceplate_animateTextParameter(dpes, values, "Calibration", g_sCalibration);

    cpcFaceplate_animateTranslatedValue(dpes, values, "AuDMoRSt", g_mDMLabels, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateTranslatedValue(dpes, values, "MDMoRSt", g_mDMLabels, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateTranslatedValue(dpes, values, "ActDMo", g_mDMLabels, "cpcColor_Faceplate_Status");

    cpcFaceplate_animateTranslatedValue(dpes, values, "AuVoTMRSt", g_mVoTLabels, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateTranslatedValue(dpes, values, "MVoTMRSt", g_mVoTLabels, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateTranslatedValue(dpes, values, "ActVoTMo", g_mVoTLabels, "cpcColor_Faceplate_Status");

    cpcFaceplate_animateTranslatedValue(dpes, values, "AuCCRSt", g_mCCLabels, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateTranslatedValue(dpes, values, "MCCRSt", g_mCCLabels, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateTranslatedValue(dpes, values, "ActCC", g_mCCLabels, "cpcColor_Faceplate_Status");

    cpcFaceplate_animateOnlineValue(dpes, values, "AuPosRSt", g_mUnits["PosSt"], g_mFormats["PosSt"], "cpcColor_Faceplate_Status");
    cpcFaceplate_animateOnlineValue(dpes, values, "MPosRSt", g_mUnits["PosSt"], g_mFormats["PosSt"], "cpcColor_Faceplate_Status");
    cpcFaceplate_animateOnlineValue(dpes, values, "ActOutOV", g_mUnits["PosSt"], g_mFormats["PosSt"], "cpcColor_Faceplate_Status");

    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_AUMOST", true, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_MMOST", true, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_AUINHMMO", true, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_FOMOST", true, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG01_AUIHFOMOST", true, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG02_SOFTLDST", true, "cpcColor_Faceplate_Status");

    cpcGenericObject_ColorBoxAnimate("AUIHTOTST", g_bInhibitManualTotalizer, 1, "cpcColor_Faceplate_Status",
                    (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));

    if (g_bBoolAlert) {
        cpcFaceplate_animateInterlock(dpes, values, "UN_STSREG01_STARTAL", "StartISt");
        cpcFaceplate_animateInterlock(dpes, values, "UN_STSREG01_TSTOPAL", "TStopISt");
        cpcFaceplate_animateInterlock(dpes, values, "CPC_STSREG02_FUSTOPIST", "FuStopISt");
        cpcFaceplate_animateInterlock(dpes, values, "CPC_STSREG01_ALST", "AlSt");
        cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_ALUNACK", true, "cpcColor_Alarm_Bad");
    }
    
    cpcFaceplate_animateMaskEvent(dpes, values);
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_AUMRW", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG02_ALBW", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_IOERRORW", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_IOSIMUW", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_POSALW", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG01_CONFIGW", true, "unFaceplate_WarningActive");
    bool needRestartInvalid, needRestart;
    cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, "CPC_STSREG02_NEEDRESTART", needRestart, needRestartInvalid);
    cpcGenericObject_ColorBoxAnimate("NeedRestart", g_bMRestart && needRestart == 0, 1, "unFaceplate_WarningActive",
                    needRestartInvalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));

}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_AnimatePlcDisconnection
/**
Animate faceplate disconnection
*/
CPC_MassFlowController_AnimatePlcDisconnection()
{
  unGenericObject_ColoredSquareDisconnect(makeDynString("StartISt", "TStopISt", "FuStopISt", "AlSt"));
  unGenericObject_ColorBoxDisconnect(makeDynString("AUMOST", "MMOST", "AUINHMMO", "FOMOST",
     "AUIHFOMOST", "SOFTLDST", "AUIHTOTST",
     "ALUNACK",
     "AUMRW", "ALBW", "IOERRORW", "IOSIMUW", "POSALW", "CONFIGW",
     "NeedRestart", "eventState"));
  unGenericObject_DisplayValueDisconnect(makeDynString("PosSt", "VoTSt", "AuVoTOfSt", "MaxPosSt", 
      "AuPosRSt", "MPosRSt", "ActOutOV",
      "AuDMoRSt", "MDMoRSt", "ActDMo",
      "AuVoTMRSt", "MVoTMRSt", "ActVoTMo",
      "AuCCRSt", "MCCRSt", "ActCC"));
}

//---------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ButtonRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus		Status of a distributed system.
	@param[in] 	bSystemConnected 	Flag if system is connected or not.
*/
CPC_MassFlowController_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	int iManRegSet;
	dyn_string exceptionInfo, dsFunctions;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);

	if (sDeviceName)
	{
		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
				case UN_ACTION_DISCONNECT:
					CPC_MassFlowController_ButtonDisconnection();
					break;
				case UN_ACTION_DPCONNECT:
					// Send 0 to ManReg01 if necessary
					iRes = dpGet(sDeviceName + ".ProcessOutput.ManReg01", iManReg01);
					if ((iRes < 0) || (iManReg01 != 0))
					{
						iManRegSet = dpSet(sDeviceName + ".ProcessOutput.ManReg01", 0);
						if (iManRegSet < 0)
						{
							dynAppend(exceptionInfo, "CPC_MassFlowController_ButtonRegisterCB: problems to set value for: " + sDeviceName + ".ProcessOutput.ManReg01");
							break;
						}
					}


					// call the cpcMassFlowControllerUserAccess to get the list of allowed buttons.
					sDpType=UN_CONFIG_CPC_MASSFLOWCONTROLLER_DPT_NAME;
					unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);

					if (dynlen(exceptionInfo) <= 0) CPC_MassFlowController_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
					CPC_MassFlowController_ButtonsConnect(sDeviceName);
				
					break;
				case UN_ACTION_DPDISCONNECT_DISCONNECT:
					CPC_MassFlowController_ButtonsDisconnect(sDeviceName);
					CPC_MassFlowController_ButtonDisconnection();
					break;
				case UN_ACTION_NOTHING:
					break;
				default:
					break;

		}
	}
	else dynAppend(exceptionInfo, "CPC_MassFlowController_ButtonRegisterCB: device name is empty");

}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
CPC_MassFlowController_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS)
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	else
	{
		dynAppend(exceptionInfo, "CPC_MassFlowController_ButtonRegisterCB: the list of functions is not correct - number of functions is less than expected");
		return;
	}

	if (sFunction != "")
	{
		evalScript(dsUserAccess, "dyn_string main(string sDeviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" +
					 	sFunction  + "(sDeviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, sDpType);
		g_dsUserAccess=dsUserAccess;
	}
	else
		fwException_raise(exceptionInfo,"ERROR", "CPC_MassFlowController_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ButtonsConnect
/**
Connects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
CPC_MassFlowController_ButtonsConnect(string sDeviceName)
{
    int iRes = -1;

    iRes = dpConnect("CPC_MassFlowController_ButtonAnimationCB",
                sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
                sDeviceName + ".statusInformation.selectedManager",
                sDeviceName + ".ProcessInput.StsReg01",
                sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv",
                sDeviceName + ".ProcessInput.StsReg02",
                sDeviceName + ".ProcessInput.StsReg02:_online.._aut_inv");

    g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
CPC_MassFlowController_ButtonsDisconnect(string sDeviceName)
{
    int iRes = -1;

    iRes = dpDisconnect("CPC_MassFlowController_ButtonAnimationCB",
                sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
                sDeviceName + ".statusInformation.selectedManager",
                sDeviceName + ".ProcessInput.StsReg01",
                sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv",
                sDeviceName + ".ProcessInput.StsReg02",
                sDeviceName + ".ProcessInput.StsReg02:_online.._aut_inv");

    g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ButtonAnimationCB
/**
Animate buttons

@note This function works with an unicos button panel and needs global variables declared in this panel -
		"g_dsUserAccess" (dyn_string). Button identifier is added to this variable if user has access.
*/
CPC_MassFlowController_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
                                        string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad,
                                        string sDpStsReg02, bit32 stsReg02Value, string sDpStsReg02Bad, bool bStsReg02Bad)
{
    CPC_MassFlowController_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_MASSFLOWCONTROLLER_DPT_NAME, g_dsUserAccess,
                                 makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ButtonUserAccess
/**
User access for buttons

	@param[in] 		sDpName	Device name
	@param[in] 		sDpType	dpType
	@param[out] 	dsAccess	List of allowed buttons
	
*/
CPC_MassFlowController_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
	(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
	bActionDefined=true;
	
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							UN_FACEPLATE_BUTTON_ACK_ALARM, CPC_FACEPLATE_BUTTON_ALLOW_RESTART, UN_FACEPLATE_BUTTON_SELECT));
	}
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(CPC_FACEPLATE_BUTTON_MFC_PARAMS, UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
	
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	sDeviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
CPC_MassFlowController_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(sDeviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
CPC_MassFlowController_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
    bool bSelected, buttonEnabled, bit1, bit2;
    dyn_string dsButtons;
    int i, iLength;
    string localManager;
    bool bLocked, bStsReg01Bad, bStsReg02Bad;
    string sSelectedUIManager;
    bit32 stsReg01Value, stsReg02Value;

    if (dynlen(dsData) >= 4) {
        bLocked = dsData[1];
        sSelectedUIManager = dsData[2];
        stsReg01Value = dsData[3];
        bStsReg01Bad = dsData[4];
        stsReg02Value = dsData[5];
        bStsReg02Bad = dsData[6];
    }


    dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, CPC_FACEPLATE_BUTTON_MFC_PARAMS,
                              UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select" button
    iLength = dynlen(dsButtons);
    // 1. Selection state
    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
    bSelected = (localManager == "S");
    // 2. Button state
    for(i = 1; i <= iLength; i++) {
        buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
        switch (dsButtons[i]) {
            case UN_FACEPLATE_BUTTON_FORCED_MODE:
                buttonEnabled = buttonEnabled && bSelected && 
                    !getBit(stsReg01Value, UN_STSREG01_FOMOST) &&
                    !getBit(stsReg01Value, UN_STSREG01_AUINHFMO) &&
                    !bStsReg01Bad && !bStsReg02Bad;
                break;
            case UN_FACEPLATE_BUTTON_MANUAL_MODE:
                buttonEnabled = buttonEnabled && bSelected && 
                    !getBit(stsReg01Value, UN_STSREG01_MMOST) &&
                    !getBit(stsReg01Value, UN_STSREG01_AUINHMMO) &&
                    !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
                    !bStsReg01Bad && !bStsReg02Bad;
                break;
            case UN_FACEPLATE_BUTTON_AUTO_MODE:
                buttonEnabled = buttonEnabled && bSelected && 
                    !getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
                    !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
                    !bStsReg01Bad && !bStsReg02Bad;
                break;
            case UN_FACEPLATE_BUTTON_ACK_ALARM:
                buttonEnabled = buttonEnabled &&
                    !getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
                    !bStsReg01Bad && !bStsReg02Bad;
                break;
            case CPC_FACEPLATE_BUTTON_MFC_PARAMS:
                buttonEnabled = buttonEnabled && bSelected;
                break;
            default:
                break;
        }
        setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
    }
    dyn_string exceptionInfo;
    cpcFaceplateButtonState_ButtonSetState(makeDynString(CPC_FACEPLATE_BUTTON_ALLOW_RESTART), dsUserAccess, deviceName, sDpType, dsData, exceptionInfo);
// 3. Button Select state
    unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_ButtonDisconnection
/**
Animate disconnection
*/
CPC_MassFlowController_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
		   UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_FORCED_MODE, CPC_FACEPLATE_BUTTON_MFC_PARAMS,
		   UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_WidgetRegisterCB
/**
Register callback function

	@param[in]	sDp 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_MassFlowController_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
    string deviceName, sSystemName, sPlcName;
    int iAction;
    dyn_string exceptionInfo;
    bool bRemote;

    deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
    if (deviceName == "") {// In case of disconnection
        deviceName = g_sDpName;
    }
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

    unDistributedControl_isRemote(bRemote, sSystemName);
    if(bRemote) g_bSystemConnected = bSystemConnected;
    else g_bSystemConnected = true;
    g_bUnSystemAlarmPlc = sPlcName != "" && dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName);

    unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
    switch (iAction)
    {
        case UN_ACTION_DISCONNECT:
            CPC_MassFlowController_WidgetDisconnection(g_sWidgetType);
            break;
        case UN_ACTION_DPCONNECT:
            g_sDpName = deviceName;

            if(dpTypeName(deviceName) == UN_CONFIG_CPC_MASSFLOWCONTROLLER_DPT_NAME) {
                CPC_MassFlowController_WidgetConnect(deviceName, sPlcName, sSystemName);
            } else{
                g_bUnSystemAlarmPlc = false;
                CPC_MassFlowController_WidgetDisconnection(g_sWidgetType);
            }
            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:
            CPC_MassFlowController_WidgetDisconnect(deviceName, sPlcName, sSystemName);
            CPC_MassFlowController_WidgetDisconnection(g_sWidgetType);
            break;
        case UN_ACTION_NOTHING:
            break;
        default:
            break;
    }

}

//-----------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
CPC_MassFlowController_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
    int iRes, iCtrlRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4;
    dyn_string exceptionInfo;
    string sDeviceLink;

    iRes = dpGet(sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                 sDeviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                 sDeviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                 sDeviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
    g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PARAMS_MRESTART, exceptionInfo);

    RegulationText1.visible = false;
    RegulationText2.visible = false;

    if (g_bUnSystemAlarmPlc) {
        g_sPosStFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosSt");
        g_sPosStUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosSt");
        unGenericDpFunctions_getDeviceLink(sDeviceName, sDeviceLink);
        g_dsCtrl = cpcGenericObject_getCtrlLink(sSystemName, strsplit(sDeviceLink, UN_CONFIG_GROUP_SEPARATOR));

        if (g_bBoolAlert) {
            iRes = dpConnect("CPC_MassFlowController_WidgetAnimationCB",
                sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
                sDeviceName + ".statusInformation.selectedManager",
                sDeviceName + ".ProcessInput.StsReg01",
                sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                sDeviceName + ".ProcessInput.StsReg02",
                sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
                sDeviceName + ".ProcessInput.PosRSt",
                sDeviceName + ".ProcessInput.PosRSt:_online.._invalid",
                sDeviceName + ".ProcessInput.PosSt",
                sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
                sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
                sDeviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
                sDeviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
                sDeviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                sDeviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_MassFlowController_WidgetAnimationCB_NoAlarm",
                sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
                sDeviceName + ".statusInformation.selectedManager",
                sDeviceName + ".ProcessInput.StsReg01",
                sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                sDeviceName + ".ProcessInput.StsReg02",
                sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
                sDeviceName + ".ProcessInput.PosRSt",
                sDeviceName + ".ProcessInput.PosRSt:_online.._invalid",
                sDeviceName + ".ProcessInput.PosSt",
                sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                sDeviceName + ".eventMask");
        }
        if (dynlen(g_dsCtrl) > 0) {
            iCtrlRes = dpConnect("cpcGenericObject_WidgetCtrlAnimationCB", g_dsCtrl);
        }
        g_bCallbackConnected = (iRes >= 0) && (iCtrlRes >= 0);
    } else {
        CPC_MassFlowController_WidgetDisconnection(g_sWidgetType);
    }
}

//--------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
CPC_MassFlowController_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
    int iRes, iCtrlRes;

    if (g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpDisconnect("CPC_MassFlowController_WidgetAnimationCB",
                sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
                sDeviceName + ".statusInformation.selectedManager",
                sDeviceName + ".ProcessInput.StsReg01",
                sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                sDeviceName + ".ProcessInput.StsReg02",
                sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
                sDeviceName + ".ProcessInput.PosRSt",
                sDeviceName + ".ProcessInput.PosRSt:_online.._invalid",
                sDeviceName + ".ProcessInput.PosSt",
                sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
                sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
                sDeviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
                sDeviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
                sDeviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                sDeviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_MassFlowController_WidgetAnimationCB_NoAlarm",
                sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
                sDeviceName + ".statusInformation.selectedManager",
                sDeviceName + ".ProcessInput.StsReg01",
                sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                sDeviceName + ".ProcessInput.StsReg02",
                sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
                sDeviceName + ".ProcessInput.PosRSt",
                sDeviceName + ".ProcessInput.PosRSt:_online.._invalid",
                sDeviceName + ".ProcessInput.PosSt",
                sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                sDeviceName + ".eventMask");
        }
        if (dynlen(g_dsCtrl) > 0) {
            iCtrlRes = dpDisconnect("cpcGenericObject_WidgetCtrlAnimationCB", g_dsCtrl);
        }
        g_bCallbackConnected = !(iRes >= 0) && !(iCtrlRes >= 0);
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
CPC_MassFlowController_WidgetDisconnection(string sWidgetType)
{
    cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_MASSFLOWCONTROLLER_DPT_NAME);
    RegulationText1.visible = false;
    RegulationText2.visible = false;
}

//---------------------------------------------------------------------------------------------------------------------------------------
CPC_MassFlowController_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
       string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
       string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
       string sDpPosRSt, float fPosRSt, string sDp6, bool bPosRStInvalid,
       string sDpPosSt, float fPosSt, string sDp8, bool bPosStInvalid,
       string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
    CPC_MassFlowController_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
        sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
        sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
        sDpPosRSt, fPosRSt, sDp6, bPosRStInvalid,
        sDpPosSt, fPosSt, sDp8, bPosStInvalid,
        "", 0, "", 0, "", 0,
        sDpAl, iAlValue, sDpEnabled, bPlcEnabled, sEvtMask, iMask);

}

// CPC_MassFlowController_WidgetAnimationCB
/**
Animate widget
*/
CPC_MassFlowController_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
       string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
       string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
       string sDpPosRSt, float fPosRSt, string sDp6, bool bPosRStInvalid,
       string sDpPosSt, float fPosSt, string sDp8, bool bPosStInvalid,
       string sDp9, int iStartIStState, string sDp10, int iTStopIStState, string sDp11, int iFuStopIStState, string sDp12, int iAlSt,
       string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{

    cpcGenericObject_WidgetAnimationDoubleStsReg(g_sWidgetType, UN_CONFIG_CPC_MASSFLOWCONTROLLER_DPT_NAME, 
        bLocked, sSelectedManager, bit32StsReg01, bit32StsReg02,
        makeDynInt(CPC_STSREG01_CONFIGW, UN_STSREG01_AUMRW, UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
        makeDynInt(UN_STSREG02_ALBW),
        makeDynString(CPC_WIDGET_TEXT_CONFIGW, CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),
        makeDynString(CPC_WIDGET_TEXT_WARNING_BLOCKED),
        makeDynInt(CPC_STSREG01_ALST, UN_STSREG01_STARTIST, CPC_STSREG01_TSTOPIST), 
        makeDynString(CPC_WIDGET_TEXT_ALARM_ALST, CPC_WIDGET_TEXT_ALARM_STARTIST, CPC_WIDGET_TEXT_ALARM_TSTOPIST),
        makeDynInt(CPC_STSREG02_FUSTOPIST), 
        makeDynString(CPC_WIDGET_TEXT_ALARM_FUSTOPIST),
        iStartIStState, iTStopIStState, iFuStopIStState, iAlSt,
        makeDynString("CPC_STSREG02_SOFTLDST", "CPC_STSREG01_AUIHFOMOST", "UN_STSREG01_AUINHMMO", "UN_STSREG01_MMOST", "UN_STSREG01_FOMOST"),
        "UN_STSREG01_ONST", "UN_STSREG01_OFFST", UN_STSREG01_ALUNACK,
        bStsReg01Invalid || bStsReg02Invalid || bPosStInvalid || bPosRStInvalid,
        bStsReg01Invalid || bStsReg02Invalid,
        iAlValue, bPlcEnabled);

    cpcGenericObject_WidgetDisplayValueAnimation("Display1", g_sPosStUnit, g_sPosStFormat, fPosRSt, "cpcColor_Widget_Request", bPosRStInvalid);
    cpcGenericObject_WidgetDisplayValueAnimation("Display2", g_sPosStUnit, g_sPosStFormat, fPosSt, "cpcColor_Widget_Status", bPosStInvalid);
    if(g_bSystemConnected)
        eventState.visible = (iMask == 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_MassFlowController_CheckParametersAccess
/**
Write ApSpSt min and max ranges, according to "temp" variable.

	@param[in]	sUser		Current user
	@param[in]	sDeviceName	Device DP name
	@param[out]	granted		Is access granted?
*/
CPC_MassFlowController_CheckParametersAccess(string sUser, string sDeviceName, bool &bGranted)
{
	string sDomain;
	dyn_string dsParameters, exceptionInfo;
	bGranted = false;

	//	gcsAccessControl should be installed
	if(!isFunctionDefined("gcsAccessControlGeneric_isPrivelegeExist")) return;

	unGenericDpFunctions_getParameters(sDeviceName, dsParameters);
	if (dynlen(dsParameters) >= UN_PARAMETER_DOMAIN) sDomain = dsParameters[UN_PARAMETER_DOMAIN];

	if(!gcsAccessControlGeneric_isPrivelegeExist(sDomain, GCS_PRIVELEGES_3)) return;
	fwAccessControl_checkUserPrivilege (sUser, sDomain, GCS_PRIVELEGES_3, bGranted, exceptionInfo);
}

