/**@name LIBRARY: cpcWordParameterConfig.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set WordParameter configuration.
Modifications History:
  $Log: CPC_WordParameterConfig.ctl,v $
	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)

	18/04/2011 amerezhi
		[UCPC-193] updated according unCPC6 specs

  2011/02/08 amerezhi
  Check and set device using UNCCPCConfig' general functions

  Revision 1.24  2010/10/08 12:52:37  amerezhi
  The unConfigGenericFunctions_checkFloatIntBlock is added to CPC_WordParameterConfig_checkCommonConfig.

  Revision 1.23  2008/07/28 12:52:37  gamx
  exception handling has been restored

  Revision 1.22  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.21  2008/01/22 12:52:44  aabadz
  Fixed:
  - Naming
  - Call to other functions to reuse code
  - Some error check

  Revision 1.20  2007/12/10 15:42:45  aabadz
  Fixed error in variable name

  Revision 1.19  2007/11/15 08:38:52  ikoblik
  unConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.18  2007/11/14 16:39:31  ikoblik
  unConfigGenericFunctions code review comments implementation

  Revision 1.17  2007/10/30 16:21:29  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	mapping CPC_WordParameterConfig_getConfig()
	CPC_WordParameterConfig_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
	CPC_WordParameterConfig_checkCustomConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
	CPC_WordParameterConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
	CPC_WordParameterConfig_setCustomConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
*/

// Constants
const string UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME = "CPC_WordParameter";
const unsigned UN_CONFIG_CPC_WORDPARAMETER_LENGTH				= 15;

const unsigned UN_CONFIG_CPC_WORDPARAMETER_UNIT					= 1;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_FORMAT				= 2;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_DEADBAND				= 3;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_DEADBAND_TYPE		= 4;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_RANGEMAX				= 5;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_RANGEMIN				= 6;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ARCHIVE_ACTIVE		= 7;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ARCHIVE_TIME_FILTER	= 8;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_MPOSRST		= 9;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_POSST		= 10;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_MPOSR		= 11;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_STSREG01		= 12;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_EVSTSREG01	= 13;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDRESS_MANREG01		= 14;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_DEFAULT_VALUE		= 15;

const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_LENGTH          = 6;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_MASTER_NAME     = 2;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_PARENTS         = 3;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_CHILDREN        = 4;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_TYPE            = 5;
const unsigned UN_CONFIG_CPC_WORDPARAMETER_ADDITIONAL_SECOND_ALIAS    = 6;

mapping CPC_WordParameterConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg01";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg01"] 		= props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg01";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg01"] 	= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["address"] 		= ".ProcessInput.MPosRSt";
	props["dataType"] 		= CPC_UINT16;
	config["MPosRSt"] 		= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["hasArchive"] 	= true;
	props["hasPvRange"] 	= true;
	props["address"] 		= ".ProcessInput.PosSt";
	props["dataType"] 		= CPC_UINT16;
	config["PosSt"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.ManReg01";
	props["dataType"] 		= CPC_UINT16;
	config["ManReg01"] 		= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["address"] 		= ".ProcessOutput.MPosR";
	props["dataType"] 		= CPC_UINT16;
	config["MPosR"] 		= props;

	return config;
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_WordParameterConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_WordParameterConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_WordParameterConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_WordParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_WordParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, configLine, exceptionInfo);
}


CPC_WordParameterConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
	int iValue, iMax, iMin;
	unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_DEFAULT_VALUE], iValue, exceptionInfo);
	unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_RANGEMIN], iMin, exceptionInfo);
	unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_RANGEMAX], iMax, exceptionInfo);
	if((iValue < iMin) || (iValue > iMax)) {
		fwException_raise(exceptionInfo,"ERROR","CPC_WordParameterConfig_checkCommonConfig: " + getCatStr("unCPCGeneration","BADRANGE"),"");	
	}
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_WordParameterConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for WordParameter object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_WordParameterConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_WordParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_WordParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

CPC_WordParameterConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
  unConfigGenericFunctions_setUnit(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_UNIT], exceptionInfo);
  unConfigGenericFunctions_setFormat(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_FORMAT], exceptionInfo);
  dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORDPARAMETER_DEFAULT_VALUE]);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_WordParameter_ExportConfig
/**
Purpose: Export WordParameter Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_CPC_WordParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	string sObject = UN_CONFIG_CPC_WORDPARAMETER_DPT_NAME;
	int amount = dynlen(dsDpList);
	for(int i = 1; i <= amount; i++) {
		dyn_string dsDpParameters;
		string currentDP = dsDpList[i];

		if(!dpExists(currentDP)) continue;

		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
		cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosRSt", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt",		true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosR", 		false, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
		cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
		cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_WordParameter_ExportConfig
/**
Purpose: Export WordParameter Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_CPC_WordParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_WordParameter_ExportConfig(dsDpList, exceptionInfo);
}

