/**@name LIBRARY: unWordStatus.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 31.07.2003
@brief This library contains WordStatus object functions.
Modifications History:
    05/03/2012: Alexey
        [UCPC-34] update animation of widget and faceplate

    24/11/2011: Alexey
        [UCPC-612] refactor menu creation and handling

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    03/11/2011: Alexey
        use stepPLC code from cpcStepPLC.ctl; same for transitionPLC

	11/08/2011: Alexey
		[UCPC-79] allowed up to 31 bit for transition widget

	18/05/2011: Alexey
		use CPC functions to manage menu

  2011/04/19 amerezhi
	[UCPC-192] updated for unCPC6 - renamed AuPosRSt => PosSt

  22/10/10 amerezhi: merged with ABCO version:
	28/01/2010: Herce
	- in CPC_WordStatus_WidgetAnimationCB and CPC_WordStatus_WidgetDisconnection: 
	check if the WidgetArea object is in the widget before setting it.
  
  Revision 1.20  2009/03/27 10:33:23  gthomas
  -in _WidgetRegisterCB() under UN_ACTION_DPCONNECT: added evaluation on DP type
  -in _WidgetAnimationCB()under
  if(g_sWidgetType != "WordStatusBit"): changed conditions of animation when data does not correspond to any labels
  if(g_sWidgetType == "WordStatusBit"): added evaluation when no labels are defined
  -in _WidgetDisconnection(): added animation of color boxes for WordStatusBitBig widget

  Revision 1.19  2009/03/11 14:33:49  gthomas
  added missing variable declaration in _WidgetAnimationCB
  and added modifs from ABCO_v4-3 in _FaceplateAnimationCB

  Revision 1.18  2008/07/23 09:41:29  gthomas
  removed exceptionInfo after unGenericObject_Connection function.

  Revision 1.17  2008/06/23 11:58:22  gthomas
  fixed bad variable call ("WordStatus" replaced with  "WS") in WidgetAnimationCB fct

  Revision 1.16  2008/06/13 12:29:47  aabadz
  Modifications asked by UNICOS for their version 4.X

  Revision 1.15  2008/05/26 12:18:20  gthomas
  Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
  to be used with PVSS 3.6 QT

  22/04/2008 Quique AB/CO
  	If no message is associated with the input value (specs definition) then the value itself is visualized

  21/04/2008 Quique AB/CO
   	-Delete the functions:
		WordStatus_EventList_Configure_Init()
		WordStatus_EventList_Configure_Clic()
		WordStatus_EventList_Configure_Save()
  Revision 1.14  2008/03/17 10:52:36  aabadz
  Fixed a problem found in animation when we have not database.

  Revision 1.13  2007/11/27 10:47:11  ikoblik
  Tested + bug fixes according to the November, 21st code review.

  Revision 1.12  2007/11/27 09:25:19  ikoblik
  Bug fixes according to the November, 21st code review.

  Revision 1.11  2007/10/30 16:22:45  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

  11/05/2004: Herve AB/CO
  	- function CPC_WordStatus_WidgetRegisterCB
  	bug in case the PLC is not defined and if the PVSS system name contains '-',
  	call unGenericObject_GetPLCNameFromDpName to get the PLC DPName instead (this function corrects the bugs).


*/
/*
External Functions :
	. CPC_WordStatus_WidgetDisconnection : animate widget disconnection
	
Internal Functions :
	. CPC_WordStatus_WidgetRegisterCB : register callback function
	. CPC_WordStatus_WidgetConnect: dpConnect to the datapoints
	. CPC_WordStatus_WidgetDisconnect: dpDisconnect to the datapoints
	. CPC_WordStatus_WidgetAnimationCB : animate widget
	. CPC_WordStatus_WidgetDisconnection: Disconnect the widget

Purpose: This library contains WordStatus object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. WordStatus object (DPType etc.)
	. Global variables and $parameters defined in WordStatus faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/


//------------------------------------------------------------------------------------------------------------
// CPC_WordStatus_MenuConfiguration
/**
Configuration of popup menu

  @param[in]  sDpName Device name
  @param[in]  sDpType Unicos object
  @param[in]  dsAccessOk  List of authorized action
  @param[out] menuList Configuration of menu. Use defined constants UN_POPUPMENU_***
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_DIAG_INFO_TEXT, UN_POPUPMENU_TREND_TEXT);

    cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//------------------------------------------------------------------------------------------------------------------
// CPC_WordStatus_HandleMenu
/**
Handle the answer of the popup menu

  @param[in]  deviceName  The device dp name
  @param[in]  sDpType   The device dp type
  @param[in]  menuList  List of requested action
  @param[in]  menuAnswer  Selected action from the menuList
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_HandleMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer)
{
  dyn_string exceptionInfo;

  cpcGenericObject_HandleUnicosMenu(sDeviceName, sDpType, dsMenuList, iMenuAnswer, exceptionInfo);
  if (dynlen(exceptionInfo) > 0)
  {
    unSendMessage_toExpertException("Right Click",exceptionInfo);
  }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_WordStatus_WidgetRegisterCB
/**
Register callback function

  @param[in]  sConnectionStatus   Distributed system connection status string
  @param[in]  bSystemConnected  Flag
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in WordStatus widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_WidgetRegisterCB(string sConnectionStatus, bool bSystemConnected)
{
	string sDeviceName = "", sPlcName = "", sSystemName = "";
	int iAction = UN_ACTION_NOTHING;
	dyn_string exceptionInfo;
	bool bRemote = false;
    int iRes;

	sDeviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (sDeviceName == "") { // In case of disconnection
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

	if ("" != sSystemName) {
		unDistributedControl_isRemote(bRemote, sSystemName);
	}

	if(bRemote) {
		g_bSystemConnected = bSystemConnected;
	} else {
		g_bSystemConnected = true;
	}

	if(sPlcName != "") {
		string sPlcDpName = "";
		dyn_string dsPlcDpNameExceptions;
		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);
		g_bUnSystemAlarmPlc = 0 == dynlen(dsPlcDpNameExceptions);
	} else {
		g_bUnSystemAlarmPlc = false;
	}

	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  
	switch (iAction) {
		case UN_ACTION_DISCONNECT:
			CPC_WordStatus_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
			if(dpTypeName(sDeviceName) != UN_CONFIG_CPC_WORDSTATUS_DPT_NAME) {
				g_bUnSystemAlarmPlc = false;
				CPC_WordStatus_WidgetDisconnection(g_sWidgetType);
			} else {
                CPC_WordStatus_WidgetConnect(sDeviceName, sPlcName, sSystemName);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
            CPC_WordStatus_WidgetDisconnect(sDeviceName, sPlcName, sSystemName);
            CPC_WordStatus_WidgetDisconnection(g_sWidgetType);
			break;
		default:
			break;
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_WordStatus_WidgetConnect
/**
Purpose: dpConnect to the DataPoint

  @param[in]  deviceName  Device DP name
  @param[in]  sPlcName  PLC name
  @param[in]  sSystemName System name
  
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in WordStatus faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes;
	string sPlcDpName = "";
	dyn_string dsPlcDpNameExceptions;
	if (!dpExists(sDeviceName)) return;

	if(g_bUnSystemAlarmPlc) {
		string sDp = sDeviceName + ".ProcessInput.PosSt";
		g_sPosStFormat = dpGetFormat(sDp);
		g_sPosStUnit = dpGetUnit(sDp);
        cpcGenericObject_StringToMapping(dpGetDescription(sDp), g_mLabels);
        if (g_sWidgetType == "WordStatusBit" || g_sWidgetType == "WordStatusBitBig") { // fill with defaults
            for (int i = 0; i < 16; i++) {
                if (!mappingHasKey(g_mLabels, (string)i)) {
                    g_mLabels[(string)i] = "N/A";
                }
            }
        }

		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);
		if (0 == dynlen(dsPlcDpNameExceptions)) {
			iRes = dpConnect("CPC_WordStatus_WidgetAnimationCB",
							 sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 sDeviceName + ".statusInformation.selectedManager",
							 sDeviceName + ".ProcessInput.PosSt",
							 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
							 sPlcDpName+".alarm",
							 sPlcDpName+".enabled");
			g_bCallbackConnected = (iRes >= 0);
		}
	} else {
		CPC_WordStatus_WidgetDisconnection(g_sWidgetType);
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_WordStatus_WidgetDisconnect
/**
Purpose: dpDisconnect to the DataPoint

  @param[in]  deviceName  Device DP name
  @param[in]  sPlcName  PLC name
  @param[in]  sSystemName System name

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in WordStatus faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_WordStatus_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes;
	string sPlcDpName = "";
	dyn_string dsPlcDpNameExceptions;

	if (!dpExists(sDeviceName)) return;

	if(g_bUnSystemAlarmPlc) {
		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);
		if (0 == dynlen(dsPlcDpNameExceptions)) {
			iRes = dpDisconnect("CPC_WordStatus_WidgetAnimationCB",
								sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
								sDeviceName + ".statusInformation.selectedManager",
								sDeviceName + ".ProcessInput.PosSt",
								sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
								sPlcDpName+".alarm",
								sPlcDpName+".enabled");
			g_bCallbackConnected = !(iRes >= 0);
		}
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_WordStatus_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in WordStatus faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                  string sDpPosSt, int iPosSt, string sDp4, bool bPosStInvalid,
                  string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
{
  string sWarningLetter, sWarningColor;
  int iIndex, iLength, iBitCounter,iBitPos;
  string sBody1Text, sBody1Color = "cpcColor_Widget_Status";

  // 1.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)
  // or the checking is not enabled then set the letter to O and the color to not valid data
  if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)) {
    sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
    sWarningColor = "unDataNotValid";
  }

  // 1.2 Control animation. Check if the datapoint is invalid, then will be displayed in blue colors
  if (bPosStInvalid) { // prepare for invalid dpe
    sWarningLetter = CPC_WIDGET_TEXT_INVALID;
    sWarningColor = "unDataNotValid";
    sBody1Color = "unDataNotValid";
  }

  if (g_sWidgetType == "WordStatusBit") {
    for (int j = 0; j < 16; j++) {
      if (getBit(iPosSt, j) == 1) {
        if (iBitCounter == 0) {
          iBitPos = j;
        }
        iBitCounter = iBitCounter + 1;
      }
    }
    if (iBitCounter == 0) {
      sBody1Text = "No Status";
    } else if (iBitCounter == 1) {
      sBody1Text = mappingHasKey(g_mLabels, (string)iBitPos) ? g_mLabels[(string)iBitPos] : "No message attributed";
    } else {
      sBody1Text = "Multiple Status";
    }
  } else if (g_sWidgetType == "WordStatusBitBig") {
    for (int j = 0; j < mappinglen(g_mLabels); j++) {
      string sTempBodyText = mappingHasKey(g_mLabels, (string)j) ? g_mLabels[(string)j] : "No message attributed";
      sBody1Text = sBody1Text + sTempBodyText + "\n";
      unGenericObject_ColoredSquareAnimate("bit" + j, getBit(iPosSt, j), true, "cpcColor_Widget_Status", "cpcColor_Widget_Status", false);
    }
  } else {
    sBody1Text = mappingHasKey(g_mLabels, (string)iPosSt) ? g_mLabels[(string)iPosSt] : iPosSt + " " + g_sPosStUnit;
  }

  // 2. Animate widget
  if(g_bSystemConnected) {
        setMultiValue("Body1",        "text",     sBody1Text,     "Body1",       "fill",    "[solid]",
                      "Body1",        "foreCol",  sBody1Color,    "Body1",       "backCol", "unWidget_Background",
                      "WarningText",  "text",     sWarningLetter, "WarningText", "foreCol",  sWarningColor);
        if(shapeExists("WidgetArea"))
            setValue("WidgetArea", "visible", true);
  }
}


//------------------------------------------------------------------------------------------------------------------------------
// CPC_WordStatus_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
  @param[in] sWidgetType Widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_WidgetDisconnection(string sWidgetType)
{
    dyn_string dsSquareName=makeDynString("");
    setMultiValue("Body1", "foreCol", "unDataNoAccess", "Body1", "backCol", "unWidget_Background", "Body1", "fill", "[solid]", "WarningText", "text", "");
    if(shapeExists("WidgetArea")) 
       setValue("WidgetArea", "visible", false);
    if(sWidgetType == "WordStatusBitBig") {
        for(int j = 0; j < mappinglen(g_mLabels); j++) {
            dynAppend(dsSquareName,"bit"+j);
            setValue("bit"+j +".border", "backCol", "unDataNoAccess");
            setValue("bit"+j +".body", "backCol", "unWidget_Background");
        }
    }
}
// CPC_WordStatus_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName;
	int iAction, iRes;
	dyn_string exceptionInfo;
	bool bRemote;

	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = TRUE;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch (iAction) {
		case UN_ACTION_DISCONNECT:
			CPC_WordStatus_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			 CPC_WordStatus_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_WordStatus_FaceplateDisconnect(deviceName);
			CPC_WordStatus_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
	}
}
// CPC_WordStatus_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in unWordStatus faceplate
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_FaceplateStatusAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                          string sDpPosSt, int iPosSt, string sDp4, bool bPosStInvalid,
                          string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
{
  string body1Text, sUniqueMessageText, sDecimalValueText, body1Color, sWarningLetter, sWarningColor;
  body1Color = "unWidget_ControlStateAuto";

  // 1.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)
  // or the checking is not enabled then set the letter to O and the color to not valid data
  if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)) {
    sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
    sWarningColor = "unDataNotValid";
    body1Color = "unDataNotValid";
  }

  // 1.2 Control animation. Check if the datapoint is invalid, then will be displayed in blue colors
  if (bPosStInvalid) { //if data is invalid
    sWarningLetter = CPC_WIDGET_TEXT_INVALID;
    sWarningColor = "unDataNotValid";
    body1Color = "unDataNotValid";
  }

  if (g_sWidgetType == "WordStatusBit") {
    for (int j = 0; j < mappinglen(g_mLabels); j++) {
      string sTempBodyText = mappingHasKey(g_mLabels, (string)j) ? g_mLabels[(string)j] : "No message attributed";
      body1Text = body1Text + sTempBodyText + "\n";
      if (g_bSystemConnected) {
        unGenericObject_ColoredSquareAnimate("bit" + j, getBit(iPosSt, j), true, "unFaceplate_StatusActive", "unFaceplate_StatusActive", false);
      }
    }
  } else {
    sUniqueMessageText = mappingHasKey(g_mLabels, (string)iPosSt) ? g_mLabels[(string)iPosSt] : "No Description";
    sDecimalValueText = iPosSt;
  }

  if (g_sWidgetType == "WordStatusBit") {
    if (g_bSystemConnected)
      setMultiValue("Body1", "text", body1Text, "Body1", "fill", "[solid]",
        "Body1", "foreCol", body1Color, "Body1", "backCol", "unWidget_Background",
        "WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor);
  } else {
    if (g_bSystemConnected)
      setMultiValue("UniqueMessageText", "text", sUniqueMessageText, "UniqueMessageText", "fill", "[solid]",
              "UniqueMessageText", "foreCol", body1Color, "UniqueMessageText", "backCol", "unWidget_Background",
              "DecimalValue", "text", sDecimalValueText, "DecimalValue", "fill", "[solid]",
              "DecimalValue", "foreCol", body1Color, "DecimalValue", "backCol", "unWidget_Background",
              "WarningText2", "text", sWarningLetter, "WarningText2", "foreCol", sWarningColor);
  }
}
// CPC_WordStatus_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in WordStatus faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_WordStatus_FaceplateConnect(string deviceName)
{
	int iRes, iAlertType1;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName, sPlcDpName;

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	if(g_bUnSystemAlarmPlc) {
		string sDP = deviceName + ".ProcessInput.PosSt";
		g_sPosStFormat = dpGetFormat(sDP);
		g_sPosStUnit = dpGetUnit(sDP);
		cpcGenericObject_StringToMapping(dpGetDescription(sDP), g_mLabels);
		if (g_sWidgetType == "WordStatusBit") {
			for (int i = 0; i < 16; i++) { // fill with defaults
				if (!mappingHasKey(g_mLabels, (string)i)) {
					g_mLabels[(string)i] = "N/A";
				}
			}
		}

		iRes = dpConnect("CPC_WordStatus_FaceplateStatusAnimationCB",
						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						deviceName + ".statusInformation.selectedManager",
						deviceName + ".ProcessInput.PosSt",
						deviceName + ".ProcessInput.PosSt:_online.._invalid",
						sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		g_bCallbackConnected = (iRes >= 0);
	} else {
		CPC_WordStatus_FaceplateStatusDisconnection();
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_WordStatus_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	deviceName	Device DP name
*/
CPC_WordStatus_FaceplateDisconnect(string deviceName)
{
	int iRes, iAlertType1;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName, sPlcDpName;

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	if(g_bUnSystemAlarmPlc) {
		string sDP = deviceName + ".ProcessInput.PosSt";
		iRes = dpDisconnect("CPC_WordStatus_FaceplateStatusAnimationCB",
						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						deviceName + ".statusInformation.selectedManager",
						deviceName + ".ProcessInput.PosSt",
						deviceName + ".ProcessInput.PosSt:_online.._invalid",
						sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		g_bCallbackConnected = !(iRes >= 0);
	} else {
		CPC_WordStatus_FaceplateStatusDisconnection();
	}
}

// CPC_WordStatus_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.1
	. operating system: XP.
	. distributed system: yes.
*/
CPC_WordStatus_FaceplateStatusDisconnection()
{
	if(g_sWidgetType == "WordStatusBit") {
		setMultiValue("Body1", "foreCol", "unDataNoAccess",
					  "WarningText", "foreCol", "unDataNoAccess");
		for(int j = 0; j < mappinglen(g_mLabels); j++){
			setMultiValue("bit"+j +".border", "backCol", "unDataNoAccess",
						  "bit"+j +".body",   "backCol", "unWidget_Background");
		}
	} else {
		setMultiValue("UniqueMessageText", "foreCol", "unDataNoAccess",
					  "DecimalValue",      "foreCol", "unDataNoAccess",
					  "WarningText2",      "foreCol", "unDataNoAccess");
	}
}
//@}
// CPC_WordStatus__ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate

*/
CPC_WordStatus_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.PosSt", value,
	      sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.PosSt:_online.._bad", bBad);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.PosSt");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	// Status to visualize
	sStatus="OK";
	dsReturnData[4] = sStatus;
	// Colors
	sColor="unEventList_ForeColor";
	dsReturnData[5]=sColor;
}
