/**@name LIBRARY: cpcWordStatusConfig.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set WordStatus configuration.
Modifications History:
  $Log: CPC_WordStatusConfig.ctl,v $
    23/03/2012: Alexey
        [UCPC-254] add SOFT_FE

	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params

	2011/05/31 Alexey
		[UCPC-297] removed MaskEvent

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

  2011/04/19 amerezhi
  [UCPC-192] updated according unCPC6 specs

  2011/02/08 amerezhi
  Check and set device using UNCCPCConfig' general functions

  Revision 1.19  2010/09/08 16:53:00  amerezhi
  The check for StringList (UN_CONFIG_CPC_WORDSTATUS_DESCRIPTION_AUPOSRST) is fixed.
  Setting empty description is fixed.

  Revision 1.24  2008/07/28 09:23:40  gamx
  exception handling has been restored.

  Revision 1.22  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.21  2008/05/26 12:18:18  gthomas
  Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
  to be used with PVSS 3.6 QT

  22/04/08    Enrique (AB/CO) Archive set in UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG instead of BOOLEAN
  Revision 1.20  2008/02/12 15:32:48  gamx
  an extra function is included to remove spaces from stepper description.
  [the format of description is "1=state1,2=state2,...", spaces are not allowed]

  Revision 1.19  2007/12/03 14:01:04  gamx
  fully checked version, fixed syntactic errors

  Revision 1.18  2007/11/29 11:32:24  gamx
  modifications after code review:
  changes to be consistent with naming conventions,
  introduced constants (to keep dpt name) and the set of external functions (to extract archive settings),
  exception checks are added.
  Functions to check and set WS as As have been moved here from AS library.

  Revision 1.17  2007/11/15 08:38:52  ikoblik
  unConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.16  2007/11/14 16:39:31  ikoblik
  unConfigGenericFunctions code review comments implementation

  Revision 1.15  2007/10/30 16:21:35  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	mapping CPC_WordStatusConfig_getConfig()
	CPC_WordStatusConfig_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
	CPC_WordStatusConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
*/

// Constants
const string UN_CONFIG_CPC_WORDSTATUS_DPT_NAME = "CPC_WordStatus";

const unsigned UN_CONFIG_CPC_WORDSTATUS_LENGTH					= 10;

const unsigned UN_CONFIG_CPC_WORDSTATUS_UNIT					= 1;
const unsigned UN_CONFIG_CPC_WORDSTATUS_FORMAT					= 2;
const unsigned UN_CONFIG_CPC_WORDSTATUS_DEADBAND				= 3;
const unsigned UN_CONFIG_CPC_WORDSTATUS_DEADBAND_TYPE			= 4;
const unsigned UN_CONFIG_CPC_WORDSTATUS_RANGEMAX				= 5;
const unsigned UN_CONFIG_CPC_WORDSTATUS_RANGEMIN				= 6;
const unsigned UN_CONFIG_CPC_WORDSTATUS_ARCHIVE_ACTIVE			= 7;
const unsigned UN_CONFIG_CPC_WORDSTATUS_ARCHIVE_TIME_FILTER		= 8;
const unsigned UN_CONFIG_CPC_WORDSTATUS_ADDRESS_POSST			= 9;
const unsigned UN_CONFIG_CPC_WORDSTATUS_DESCRIPTION				= 10;

const unsigned UN_CONFIG_CPC_WORDSTATUS_ADDITIONAL_LENGTH          = 5;
const unsigned UN_CONFIG_CPC_WORDSTATUS_ADDITIONAL_MASTER_NAME     = 1;
const unsigned UN_CONFIG_CPC_WORDSTATUS_ADDITIONAL_PARENTS         = 2;
const unsigned UN_CONFIG_CPC_WORDSTATUS_ADDITIONAL_CHILDREN        = 3;
const unsigned UN_CONFIG_CPC_WORDSTATUS_ADDITIONAL_TYPE            = 4;
const unsigned UN_CONFIG_CPC_WORDSTATUS_ADDITIONAL_SECOND_ALIAS    = 5;

mapping CPC_WordStatusConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["hasArchive"] 	= true;
	props["hasPvRange"] 	= true;
	props["hasDescription"]	= true;
	props["address"] 		= ".ProcessInput.PosSt";
	props["dataType"] 		= CPC_UINT16;
	config["PosSt"] 		= props;

	return config;
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_WordStatusConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_WordStatusConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_WORDSTATUS_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_WORDSTATUS_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_WordStatusConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_WordStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORDSTATUS_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_WordStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORDSTATUS_DPT_NAME, configLine, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_WordStatusConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for WordStatus object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_WordStatusConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_WORDSTATUS_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_WORDSTATUS_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_WORDSTATUS_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_WordStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_WordStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_WordStatus_ExportConfig
/**
Purpose: Export WordStatus Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_CPC_WordStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	string sObject = UN_CONFIG_CPC_WORDSTATUS_DPT_NAME;
	int amount = dynlen(dsDpList);
	for(int i = 1; i <= amount; i++) {
		dyn_string dsDpParameters;
		string currentDP = dsDpList[i];
		string sLimitOn, sLimitOff;

		if(!dpExists(currentDP)) continue;

		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
		cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
		cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
		cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
		cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
		cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt", 	true, dsDpParameters);
		dynAppend(dsDpParameters, dpGetDescription(currentDP + ".ProcessInput.PosSt"));
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_WordStatus_ExportConfig
/**
Purpose: Export WordStatus Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_CPC_WordStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_WordStatus_ExportConfig(dsDpList, exceptionInfo);
}


const unsigned UN_CONFIG_SOFT_FE_CPC_WORDSTATUS_UNIT                   = 1;
const unsigned UN_CONFIG_SOFT_FE_CPC_WORDSTATUS_FORMAT                 = 2;
const unsigned UN_CONFIG_SOFT_FE_CPC_WORDSTATUS_RANGEMAX               = 3;
const unsigned UN_CONFIG_SOFT_FE_CPC_WORDSTATUS_RANGEMIN               = 4;
const unsigned UN_CONFIG_SOFT_FE_CPC_WORDSTATUS_ARCHIVE_ACTIVE         = 5;
const unsigned UN_CONFIG_SOFT_FE_CPC_WORDSTATUS_ARCHIVE_TIME_FILTER    = 6;
const unsigned UN_CONFIG_SOFT_FE_CPC_WORDSTATUS_DESCRIPTION            = 7;
const unsigned UN_CONFIG_SOFT_FE_CPC_WORDSTATUS_LENGTH                 = 7;

void SOFT_FE_CPC_WordStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORDSTATUS_DPT_NAME, configLine, exceptionInfo);
}

void SOFT_FE_CPC_WordStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

void SOFT_FE_CPC_WordStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_WORDSTATUS_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        dynAppend(dsDpParameters, dpGetDescription(currentDP + ".ProcessInput.PosSt"));
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString(), dsDpParameters);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

